/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2006. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */


// Cdmw files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/common/Assert.hpp>


// Cdmw CCM files
#include <CCMContainer/ccmcif/CCMContextExt_impl.hpp>

namespace 
{
    const std::string UNAVAILABLE_SERVICE_IDENTIFIERS[] = {"RootPOA", 
                                                           "POACurrent", 
                                                           "ORBPolicyManager",
                                                           "PolicyCurrent",
                                                           "PICurrent"};
    // size of the table is sizeof / 4 octets
    const int UNAVAILABLE_SERVICE_IDENTIFIERS_NB 
       = sizeof(UNAVAILABLE_SERVICE_IDENTIFIERS)/sizeof(UNAVAILABLE_SERVICE_IDENTIFIERS[0]);

} // end anonymous namespace


namespace Cdmw {

namespace CCM {

namespace CIF {

//
// IDL:omg.org/CdmwComponents/CCMContextExt:1.0
//  
CCMContextExt_impl::CCMContextExt_impl(Components::CCMContext_ptr ctx)
{
   m_context = CdmwCcmContainer::CCM2Context::_narrow(ctx);
   
   if (CORBA::is_nil(m_context.in()))
    {
        // This is to avoid Assertion failure in debug mode as internal ref count
        // equals 1 (CORBA C++ mapping oblige!)
        _set_ref_count(0);

        throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAMWrongContextType, 
                               CORBA::COMPLETED_NO);
    }

}

CCMContextExt_impl::~CCMContextExt_impl()
{
}

//
// IDL:omg.org/Components/CCMContext/get_caller_principal:1.0
//
Components::Principal_ptr 
CCMContextExt_impl::get_caller_principal()
    throw(CORBA::SystemException)
{
    return m_context->get_caller_principal();
}

//
// IDL:omg.org/Components/CCMContext/get_CCM_home:1.0
//
Components::CCMHome_ptr  
CCMContextExt_impl::get_CCM_home()
    throw(CORBA::SystemException)
{
    return m_context->get_CCM_home();
}

//
// IDL:omg.org/Components/CCMContext/get_rollback_only:1.0
//
CORBA::Boolean  
CCMContextExt_impl::get_rollback_only()
    throw(Components::IllegalState,
          CORBA::SystemException)
{
    return m_context->get_rollback_only();
}

//
// IDL:omg.org/Components/CCMContext/get_user_transaction:1.0
//
Components::Transaction::UserTransaction_ptr  
CCMContextExt_impl::get_user_transaction()
    throw(Components::IllegalState,
          CORBA::SystemException)
{
    return m_context->get_user_transaction();
}

//
// IDL:omg.org/Components/CCMContext/is_caller_in_role:1.0
//
CORBA::Boolean  
CCMContextExt_impl::is_caller_in_role(const char* role)
    throw(CORBA::SystemException)
{
    return m_context->is_caller_in_role(role);
}

//
// IDL:omg.org/Components/CCMContext/set_rollback_only:1.0
//
void  
CCMContextExt_impl::set_rollback_only()
    throw(Components::IllegalState,
          CORBA::SystemException)
{
    m_context->set_rollback_only();
}

//
// IDL:omg.org/CdmwCcmComponents/CCMContextExt/get_service:1.0
//
CORBA::Object_ptr 
CCMContextExt_impl::get_service(const char* identifier)
    throw(CdmwComponents::CCMContextExt::ServiceUnavailable,
          CORBA::SystemException)
{
   std::string id_str = identifier;

   for (int i=0 ; i < UNAVAILABLE_SERVICE_IDENTIFIERS_NB ; i++)
   {
       if (id_str == UNAVAILABLE_SERVICE_IDENTIFIERS[i])
           throw CdmwComponents::CCMContextExt::ServiceUnavailable();
   }
   
   // service id is authorised : get the reference
   try
   {
       return m_context->resolve_initial_references(identifier);
   }
   catch (const CdmwCcmContainer::CCM2Context::InvalidName& )
   {
       throw CdmwComponents::CCMContextExt::ServiceUnavailable();
   }
   catch (const CORBA::SystemException& )
   {
       throw;
   }
}


} // end namespace CIF
} // end namespace CCM
} // end namespace Cdmw



