#!/bin/sh
# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES
# 2000-2003. All rights reserved.
# Copyright (C) THALES 2004-2006. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================


# output IOR file
IOR_FILE="IFRepository.IOR"


EXEC=exec

if test "`echo -e xxx`" = "xxx"
then
  echo="echo -e"
else
  echo=echo
fi

#----------------------------------------------------------------------
#  This shell script launches the IFR feed  
#----------------------------------------------------------------------

usage()
{
cat >&2 <<EOF

Usage: $0  [options] [ifr_options] <idl_file>
options:
    --help
            Print this help
    --ifr-ior IOR
            name of IFR IOR file
            
    ifr_options
            ifr tool options

    <idl_file>
            name of the idl file

Description:
      This program feeds <idl_file> into the Interface Repository (IFR).
EOF
}

while_exit=0

while [ $# -gt 0 -a $while_exit = "0" ]
do
    case $1 in
	    --help)
		    usage $0
		    exit 0
		    ;;    
	    --ifr-ior)
		    shift        
		    if [ $# -gt 0 ]
		    then
			  IOR_FILE="$1"
              shift
            fi
            ;; 
        *)
		    while_exit="1"
            ;;
    esac
done

# wait IFR Service is started by testing ior creation
sleep 1
until [ -s $IOR_FILE ]; do
  sleep 1
done

# start IFR feed
$echo "starting the Interface Repository feed utility ..."

#case @config.SystemMngt.cpp.orb@ in
#   TAO)
#       $echo $EXEC tao_ifr -v -ORBInitRef InterfaceRepository=file://$IOR_FILE -I. -I../idl $@
#       $EXEC tao_ifr -v -ORBInitRef InterfaceRepository=file://$IOR_FILE -I. -I../idl $@     
#	    ;;
#
#	ORBacusCPP)
#		;;
#						
#   *)
#		echo " invalid ORB constraint...";
#	    exit 1;
#        ;;
#esac
#


OpenCCM_HOMEDIR=@config.OpenCCM.install.dir@
export OpenCCM_HOMEDIR

PATH=@config.install.dir@/bin:${OpenCCM_HOMEDIR}/bin:${PATH}
export PATH
 
args="org.objectweb.ccm.scripts.IR3Feed -ORBInitRef"
args="$args InterfaceRepository=file:$IOR_FILE -D__OpenCCM_IFR__ -DTAO -I. -I@config.install.dir@/idl -I$TAO_ROOT/tao -I$TAO_ROOT/tao/IFR_Client -I$TAO_ROOT/Tao_idl -I$TAO_ROOT $@"

BOOTJARFILES=${OpenCCM_HOMEDIR}/lib/OpenCCM.jar:@config.OpenORB.install.dir@/lib/openorb-1.3.0.jar:@config.OpenORB.install.dir@/lib/openorb_tools-1.3.0.jar:@config.OpenORB.install.dir@/lib/avalon-framework.jar:@config.OpenORB.install.dir@/lib/excalibur-configuration.jar:@config.OpenORB.install.dir@/lib/logkit.jar:@config.J2SDK.install.dir@/jre/lib/rt.jar:@config.J2SDK.install.dir@/jre/lib/i18n.jar:${OpenCCM_HOMEDIR}/lib/log4j-1.1.3.jar:${OpenCCM_HOMEDIR}/lib/velocity-1.2-dep-modified.jar:${OpenCCM_HOMEDIR}/lib/jta_1.0.1.jar:${OpenCCM_HOMEDIR}/lib/velocity-1.2-modified.jar:${OpenCCM_HOMEDIR}/lib/commons-collections.jar:${OpenCCM_HOMEDIR}/lib/jdbc2_0-stdext.jar:${OpenCCM_HOMEDIR}/lib/jdom.jar:${OpenCCM_HOMEDIR}/lib/xerces.jar:${OpenCCM_HOMEDIR}/lib/openorb_ots-1.3.0.jar:${OpenCCM_HOMEDIR}/lib/dtdparser114.jar:${OpenCCM_HOMEDIR}/lib/zeus.jar

JVM=@config.J2SDK.install.dir@/jre/../bin/java
        
OPT="-Djava.compiler=NONE"

# find the path of Components.idl
components_path=""
idl_path=""

for n in echo $args
do
	include_args=$(echo $n | grep "\-I")
	if [ ! -z $include_args ] 
	then 
		components_path=${include_args#"-I"}
		idl_path=$components_path
		components_path=$components_path/Components.idl
		if [ -e $components_path ]
		    then
		    break
		fi

	fi

done


$JVM $OPT -Xbootclasspath:${BOOTJARFILES} org.objectweb.ccm.scripts.IR3Feed -ORBInitRef InterfaceRepository=file:IFRepository.IOR -D__OpenCCM_IFR__ -I. -I$TAO_ROOT/tao -I$TAO_ROOT/Tao_idl -I$TAO_ROOT -I$idl_path $components_path

$EXEC $JVM $OPT -Xbootclasspath:${BOOTJARFILES} $args

