/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2006. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */


package cdmw.ccm.deployment;

import cdmw.logging.*;


/**
 * Specific Logger class logging traces in PackageManager specific domain.
 */
class PackageManagerLogger extends Logger  {

    /**
     * Logging domain for PackageManager
     */
    private static final String DOMAIN = "CDMW.CCM.PKG_MNGR";
    
    /**
     * Singleton instance
     */
    private static PackageManagerLogger instance = null;

    
    /**
     * Constructor
     */
    protected PackageManagerLogger() {
        super(LogManager.Instance(), DOMAIN);
    }


    /**
     * Returns the singleton instance of PackageManagerLogger
     *
     * @return the singleton instance
     */
    private static PackageManagerLogger getInstance() {
        if (instance == null) {
            instance = new PackageManagerLogger();
        }
        return instance;
    }
    
    /**
     * Log a message with FATAL level in PackageManager domain
     *
     * @param msg The message
     * @param thr A Throwable object to get file name and line number 
     */
    static public void CDMW_FATAL(String msg, Throwable thr) {
        if (getInstance().isFatalEnabled()) {
            getInstance().log(new LevelType(LogLevel.FATAL), msg, thr);
        }
    }

    /**
     * Log a message with ERROR level in PackageManager domain
     *
     * @param msg The message
     * @param thr A Throwable object to get file name and line number 
     */
    static public void CDMW_ERROR(String msg, Throwable thr) {
        if (getInstance().isErrorEnabled()) {
            getInstance().log(new LevelType(LogLevel.ERROR), msg, thr);
        }
    }
    
    /**
     * Log a message with WARN level in PackageManager domain
     *
     * @param msg The message
     * @param thr A Throwable object to get file name and line number 
     */
    static public void CDMW_WARN(String msg, Throwable thr) {
        if (getInstance().isWarnEnabled()) {
            getInstance().log(new LevelType(LogLevel.WARN), msg, thr);
        }
    }
 
    /**
     * Log a message with INFO level in PackageManager domain
     *
     * @param msg The message
     * @param thr A Throwable object to get file name and line number 
     */
    static public void CDMW_INFO(String msg, Throwable thr) {
        if (getInstance().isInfoEnabled()) {
            getInstance().log(new LevelType(LogLevel.INFO), msg, thr);
        }
    }

    /**
     * Log a message with DEBUG level in PackageManager domain
     *
     * @param msg The message
     * @param thr A Throwable object to get file name and line number 
     */
    static public void CDMW_DEBUG(String msg, Throwable thr) {
        if (getInstance().isDebugEnabled()) {
            getInstance().log(new LevelType(LogLevel.DEBUG), msg, thr);
        }
    }
    
    
    
}





