/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES 
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */

#ifndef INCL_CDMW_COMMNSVCS_SERVICE_REGISTRATION_SERVICE_REGISTRATION_IMPL_HPP
#define INCL_CDMW_COMMNSVCS_SERVICE_REGISTRATION_SERVICE_REGISTRATION_IMPL_HPP


#include "Foundation/osthreads/Mutex.hpp"
#include "Foundation/osthreads/Condition.hpp"
#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/idllib/CdmwServiceLocator.skel.hpp"

#include <iostream>
#include <map>

namespace Cdmw
{
namespace CommonSvcs
{
namespace ServiceRegistration
{
class  ServiceRegistration_impl :
        public virtual POA_CdmwServiceLocator::ServiceRegistration,
        public virtual PortableServer::RefCountServantBase
{

public :
    
    static ServiceRegistration_impl* init(CORBA::ORB_ptr orb,
                             PortableGroup::GOA_ptr goa,
                             std::string service_locator_address,
                             std::string registration_service_address,
                             int nb_retries,
                             int time_interval)
       throw (CORBA::SystemException);
    
    ~ServiceRegistration_impl();

    static ServiceRegistration_impl* get_instance() throw (CORBA::SystemException);

    /**
    * Purpose:
    * <p>
    * implements the
    * IDL:thalesgroup.com/Foundation/CdmwServiceLocator/register_service:1.0
    * operation
    */  
    virtual void register_service(const char * name, CORBA::Object_ptr ref)
        throw(CORBA::SystemException);
    
    /**
    * Purpose:
    * <p> register locally the service.
    * 
    */    
    void register_local_service(const char * name, CORBA::Object_ptr ref);

    /**
    * Purpose:
    * <p> Get the service reference from its name.
    * 
    */
    CORBA::Object_ptr get_service(const char * name, bool do_refresh = false);


    typedef std::map<std::string, CORBA::Object_var, std::less<std::string> > ServiceMap;
    
    /**
    * Purpose:
    * <p> Returns a copy of the underlying service map.
    * 
    */
    ServiceMap get_known_services();
    
protected:
    // protect constructor
    ServiceRegistration_impl(CORBA::ORB_ptr orb,
                             PortableGroup::GOA_ptr goa,
                             std::string service_locator_address,
                             std::string registration_service_address,
                             int nb_retries,
                             int time_interval);

private:
    // hide default constructor and copy constructor  
    ServiceRegistration_impl();
    ServiceRegistration_impl(const ServiceRegistration_impl&);
    void operator=(const ServiceRegistration_impl&);
    
    struct CacheData
    {
    CacheData()
        : m_obj(CORBA::Object::_nil()),
          m_is_refreshed(false)
    {
    }
    CacheData(CORBA::Object_ptr obj, bool fresh = true)
        : m_obj(CORBA::Object::_duplicate(obj)),
          m_is_refreshed(fresh)
    {
    }
    CacheData(const CacheData& cd)
        : m_obj(CORBA::Object::_duplicate(cd.m_obj.in())),
          m_is_refreshed(cd.m_is_refreshed)
    {
    }

    CacheData & operator=(const CacheData & rhs)
    {
        if (this != &rhs) {
        m_obj = CORBA::Object::_duplicate(rhs.m_obj.in());
        m_is_refreshed = rhs.m_is_refreshed;
        }
        return *this;
    }
    /**
     * Object reference for a cached service
     */
    CORBA::Object_var m_obj;
    /**
     * Boolean indicated that data has been refreshed or not
     */
    bool              m_is_refreshed;
    
    };
    typedef std::map<std::string, CacheData, std::less<std::string> > InternalServiceMap;

    /**
    * The global services.
    */
    InternalServiceMap m_services;
    
    /**
    * The mutex protecting concurrent access to m_globalServices.
    * (associated to the receiving registration condition)
    */
    OsSupport::Mutex m_servicesMutex;
    
    /**
    * The condition for receiving the registration.
    */
    OsSupport::Condition m_servicesCondition;
    
    /**
    * The orb reference.
    */
    CORBA::ORB_var m_orb;

    /**
    * The poa reference.
    */
    PortableGroup::GOA_var m_goa;
    
    /**
    * The max number of retries to interogate the service locator.
    */
    int m_nb_retries;
    
    /**
    * The time interval between two interogations.
    */
    int m_time_interval;

    /**
    * The service locator to interrogate.
    */
    CdmwServiceLocator::ServiceLocator_var m_service_locator;
    
    /**
    * The string reference of the registration service receiving the answers..
    */
    std::string m_registration_service_address;


    /**
    * The static reference of the serviceRegistration implementation
    */
    static ServiceRegistration_impl* m_instance;
};
    
} // End namespace ServiceRegistration
} // End namespace CommonSvcs
} // End namespace Cdmw

#endif // INCL_CDMW_COMMNSVCS_SERVICE_REGISTRATION_SERVICE_REGISTRATION_IMPL_HPP
