/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES 
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */


#include <iostream>
#include <sstream>
#include <limits>
#include <ctime>




#include <Foundation/commonsvcs/time/Util.hpp>



// definition of local constants in an unnammed namespace
namespace 
{ 
    // nbr of 100 nsec in 1 sec
    const TimeBase::TimeT HUNDRED_OF_NSEC_IN_SEC = static_cast<TimeBase::TimeT>(10000000);
    
    // TimeVal is in sec/microsec from 1st Jan 1970 (This is different in Windows!!!)
    //
    // Time difference in 100 nanoseconds between DCE and POSIX
    // is 141427 days.  
	const TimeBase::TimeT DELTA_POSIX_DCE = 
                static_cast<TimeBase::TimeT>(141427) * 
                static_cast<TimeBase::TimeT>(86400) *
                HUNDRED_OF_NSEC_IN_SEC;
}

  
namespace Cdmw 
{ 
namespace CommonSvcs 
{ 
  namespace Time 
  {
    namespace Util
    {

      OsSupport::OS::Timeval
         TimeT_2_Timeval (const TimeBase::TimeT& input_time)
      {
          OsSupport::OS::Timeval output_tv = naTimeval;

          // compute from input_time the integral number of seconds since the Epoch
    
          // get time in 100nsec from epc
          TimeBase::TimeT timet_epc = input_time - DELTA_POSIX_DCE;
    
          // if epoch time is valid
          if (timet_epc > 0)
          {
              // get time in sec from epc
              TimeBase::TimeT time_sec = timet_epc / HUNDRED_OF_NSEC_IN_SEC;
              TimeBase::TimeT time_hund_of_nsec = 
                      timet_epc - (time_sec * HUNDRED_OF_NSEC_IN_SEC);

              if (time_sec < static_cast<TimeBase::TimeT>(std::numeric_limits<long>::max()))
              {   
                  // assign tv.seconds
                  output_tv.seconds  = static_cast<long>(time_sec);

                  // assign tv.microseconds
                  output_tv.microseconds = static_cast<long>(time_hund_of_nsec/10);
              }
          }

          return output_tv;
      }

      TimeBase::TimeT
         Timeval_2_TimeT(const OsSupport::OS::Timeval& input_tv)
      {
          TimeBase::TimeT out_time = naTimeT;
          
          // if input Timeval is valid convert it
          if (!(input_tv == naTimeval))
          {
              out_time = 
                  DELTA_POSIX_DCE +
                  static_cast<TimeBase::TimeT>(input_tv.seconds ) * HUNDRED_OF_NSEC_IN_SEC +
                  static_cast<TimeBase::TimeT>(input_tv.microseconds) * 10;
          }
          
          return out_time;
      }
      
      TimeBase::TimeT
         change_base(const TimeBase::TimeT& input_time,
                     Format from, Format to)
      {
          TimeBase::TimeT out_time = input_time;
          
          // if conversion requested and input valid 
          if (from != to && out_time != naTimeT)
          {
              if (from == EPOCH && to == GREGORIAN)
              {
                  out_time += DELTA_POSIX_DCE;
              }
              else if (from == GREGORIAN && to == EPOCH)
              {
                  if (out_time >= DELTA_POSIX_DCE)
                  {
                      out_time -= DELTA_POSIX_DCE;
                  }                
              }
          }
        
          return out_time;
      }
    

      TimeBase::TimeT get_current_time()
             throw (::Cdmw::InternalErrorException)
      {
          // first get TimeVal from OsSupport
          TimeBase::TimeT current_time = 
              Timeval_2_TimeT(::Cdmw::OsSupport::OS::get_time());

          return current_time;
      }


    }; // End namespace Util
  }; // End namespace Time
}; // End namespace CommonSvcs
}; // End namespace Cdmw

