/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES 
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */


#ifndef _CDMW_FDS_LOGGING_LOGLEVEL_HPP_
#define _CDMW_FDS_LOGGING_LOGLEVEL_HPP_

#include <string>


namespace Cdmw
{
    namespace Logging
    {
        /**
         * the type of log level.
         */
        typedef unsigned long LevelType;
        
        
        /** 
         * Constant identifying all components
         */ 
        const std::string ALL_COMPONENT_NAMES = "*";


        /** 
         * Constant identifying all domains
         */
        const std::string ALL_DOMAINS = "*";


        /** 
         * Constant identifying all levels set
         */
        const LevelType ALL_LEVELS_SET = 0xFFFFFFFF;

        /** 
         * Constant identifying the component separator
         */ 
        const char COMPONENT_SEP = '.';

        
        /**
         * Default log levels.
         */
        enum LogLevel
        {         
            // this level reports a fatal error from CDMW; CDMW is no more
            // able to process safely.
            FATAL = 1,
        
            // this level reports an erorr from CDMW.
            ERROR = 2,
        
            // maybe the user has made an error, warn him.
            WARN  = 4,
        
            // used for information purposes.
            INFO  = 8,
        
            // used for debug purposes.
            DEBUG = 16,
        
            /**
             * Additional debug levels for CDMW.
             */
            DEBUG_1 = 0x00000020,
            DEBUG_2 = 0x00000040,
            DEBUG_3 = 0x00000080,
            DEBUG_4 = 0x00000100,
            DEBUG_5 = 0x00000200,
            DEBUG_6 = 0x00000400,
            DEBUG_7 = 0x00000800,
            DEBUG_8 = 0x00001000,
            DEBUG_9 = 0x00002000
        };

    } // end namespace Logging
} // end namespace Cdmw


#endif // _CDMW_FDS_LOGGING_LOGLEVEL_HPP_
