/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES 
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */
package cdmw.logging;

import cdmw.common.Assert;

public class FunctionLogger {
    
    private final static String ENTERING="Entering ";
    private final static String EXITING="Exiting ";
    private final static String RETURN_MSG_START="Returning <";
    private final static String RETURN_MSG_END=">";
    private final static String EXCEPTION_MSG_START="Raising exception <";
    private final static String EXCEPTION_MSG_END=">";
    
    private static final LevelType LEVEL = new LevelType(LogLevel.DEBUG_3);
    
    private Logger logger;
    private String function;
    private String message;
    private Throwable thr;
      
    protected FunctionLogger(Logger logger, String args, String function, Throwable thr) {
        this.logger = logger;
        this.function = function;
        this.message = "";
        this.thr = thr;
        
        Assert.check(logger != null);
        if (logger.isEnabledFor(LEVEL)) { 
            logger.log(LEVEL, ENTERING + this.function + "(" + args + ") ...", this.thr);      
        }
    }
    
    public void logReturn(Object message, Throwable thr) {
        this.message = "[" + RETURN_MSG_START + String.valueOf(message) + RETURN_MSG_END + "]";
        if (this.isEnabled()) {
            this.logger.log(LEVEL, EXITING + function + "() ... " + this.message, thr);
        }
    }
    public void logException(Object message, Throwable thr) {
        this.message = "[" + EXCEPTION_MSG_START + String.valueOf(message) + EXCEPTION_MSG_END + "]";
        if (this.isEnabled()) {
            this.logger.log(LEVEL, EXITING + function + "() ... " + this.message, thr);
        } 
    }
    
    public boolean isEnabled() { 
        return this.logger.isEnabledFor(LEVEL);
    }

    public static FunctionLogger CDMW_LOG_FUNCTION(Logger logger,Object X, Throwable thr){
        String strArgs = "";         
        if (logger.isEnabledFor(LEVEL)) {                                            
            strArgs = String.valueOf(X); 
        } 
        
        String fileMethod = "";
        if (thr != null) {
            StackTraceElement[] stack = thr.getStackTrace();
            if (stack[0].getMethodName() != null) {
                fileMethod = stack[0].getMethodName();
           }
        }
        return new FunctionLogger(logger ,strArgs ,fileMethod ,thr);
    }
}
