/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES 
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */

package cdmw.logging;
import java.io.*;
import java.lang.Long;
import cdmw.ossupport.*;
import cdmw.common.*;
import java.text.SimpleDateFormat;
import java.util.GregorianCalendar;

public class LogManagerDefaultDelegate implements LogManagerDelegate
{
    
    BufferedWriter m_ofstream;
    String m_logFileName;

    String sFATAL   = "FATAL";
    String sERROR   = "ERROR";
    String sWARN    = "WARNING";
    String sINFO    = "INFO";
    String sDEBUG   = "DEBUG";
    String sDEBUG_1 = "DEBUG_1";
    String sDEBUG_2 = "DEBUG_2";
    String sDEBUG_3 = "DEBUG_3";
    String sDEBUG_4 = "DEBUG_4";
    String sDEBUG_5 = "DEBUG_5";
    String sDEBUG_6 = "DEBUG_6";
    String sDEBUG_7 = "DEBUG_7";
    String sDEBUG_8 = "DEBUG_8";
    String sDEBUG_9 = "DEBUG_9";

    LogLevel Logging = new LogLevel(); 
    cdmw.common.Options logOption = new cdmw.common.Options();

    /**
     * Return the numeric level as string.
     *
     * @param level the level.
     *
     * @return the level as string.
     */
    String levelToString(LevelType level) {
        long choice = level.getType();
            
        if (choice == Logging.FATAL)   {return sFATAL;}
        else if (choice == Logging.ERROR)   {return sERROR;}
        else if (choice == Logging.WARN)    {return sWARN;}
        else if (choice == Logging.INFO)    {return sINFO;}
        else if (choice == Logging.DEBUG)   {return sDEBUG;}
        else if (choice ==  Logging.DEBUG_1) {return sDEBUG_1;}
        else if (choice ==  Logging.DEBUG_2) {return sDEBUG_2;}
        else if (choice ==  Logging.DEBUG_3) {return sDEBUG_3;}
        else if (choice ==  Logging.DEBUG_4) {return sDEBUG_4;}
        else if (choice ==  Logging.DEBUG_5) {return sDEBUG_5;}
        else if (choice ==  Logging.DEBUG_6) {return sDEBUG_6;}
        else if (choice ==  Logging.DEBUG_7) {return sDEBUG_7;}
        else if (choice ==  Logging.DEBUG_8) {return sDEBUG_8;}
        else if (choice ==  Logging.DEBUG_9) {return sDEBUG_9;}
        else {return Long.toString(level.getType());}
        
    }

    // TODO: This should be moved to OsSupport
    String basename(String f) {
        String DELIM= "/";
        int i = f.lastIndexOf(DELIM);
        if (i != -1) 
        {
            String res = f.substring(i+1,f.length());
            return res;
        } 
        else
        {
            return f;
        }

    } 
   

    public LogManagerDefaultDelegate(String[] args, FilterMngr filterManager) {
       if (args.length != 0)
       {
           String levelOption = logOption.getOptionValue(args,logOption.LOG_LEVEL_OPTION);

            if (levelOption != "no" && levelOption != "yes")
            {
                LevelType level = new LevelType();

                try {
                    level.setType(Long.parseLong(levelOption));
                    filterManager.setDefaultLevel (level);
                } catch (NumberFormatException e) {
                    System.err.println("Invalid option will be ignored: " + 
                                       logOption.LOG_LEVEL_OPTION + "=" + levelOption);
                }
                
            }
              
            // Is there an output log file
            String fileOption = logOption.getOptionValue(args, logOption.LOG_FILE_OPTION);
            if (fileOption != "no" && fileOption != "yes")
            {
                try
                {
                    m_ofstream = new BufferedWriter(new FileWriter(fileOption));
                    m_logFileName = fileOption;
                    m_ofstream.close();
                }
                catch (java.lang.Exception e)
                {
                }
            }
        }
    }

    /**
     * Send a message to the logger.
     */
    public synchronized boolean addLog(LogHeader header, String msg) {

        // write time with second precision in buffer
        SimpleDateFormat tf = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss:SSS");
        GregorianCalendar calendar = new GregorianCalendar();
        calendar.setTime(header.m_timestamping.toDate());
        String buffer1 = tf.format(calendar.getTime());

        // add milliseconds and microseconds in buffer
        String buffer3 =  ":000 " + levelToString(header.m_userLevel) + " [" + header.m_componentName + "]" + " " + header.m_userDomain + " [tid =" + header.m_tid + "] ";

        String buffer3a =  "(" + basename(header.m_fileName) + ":" + header.m_fileLine + ") ";

        String buffer4 =   buffer1 + buffer3 + buffer3a + msg + "\n";
            
        try
        {
            if (m_ofstream == null)
            {
                System.out.println(buffer4);
            }
            else
            {
                m_ofstream = new BufferedWriter(new FileWriter(m_logFileName,true));
                m_ofstream.write(buffer4);
                m_ofstream.close();
            }
        }
        catch (java.lang.Exception e)
        {
            System.out.println("File write failed!");
        }
        return true;
    }
}
