/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES 
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *  
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */


package cdmw.logging.test;

import cdmw.logging.*;

import cdmw.testutils.Testable;
import cdmw.common.*;


public class TestFilterMngr extends Testable 
{

    public TestFilterMngr() {
        this.name = "TestFilterMngr";
        this.out = System.out;
    }

    public void doTests() {
        setNbOfRequestedTestOK(395); 

        println ("Creating an instance of FilterMngr.");
        LogLevel Logging = new LogLevel();
        FilterMngr filterMngr = new FilterMngr (Logging.INFO | Logging.WARN | Logging.ERROR | Logging.FATAL);
 
        println ("Check if INFO, WARN, ERROR, FATAL are activated by default");

        check (filterMngr.isActivated("domain1", Logging.INFO) == true);
        check (filterMngr.isActivated("domain1", Logging.WARN) == true);
        check (filterMngr.isActivated("domain1", Logging.ERROR) == true);
        check (filterMngr.isActivated("domain1", Logging.FATAL) == true);
        check (filterMngr.isActivated("domain1", Logging.DEBUG) == false);
        
        check (filterMngr.isActivated("domain2", Logging.INFO) == true);
        
        check(filterMngr.isActivated("component1", "domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("component1", "domain2", Logging.INFO) == true);
    
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.INFO) == true);
    
        check(filterMngr.isActivated("component1.component1b", "domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("component1.component1b", "domain2", Logging.INFO) == true);
    
        check(filterMngr.isActivated("component2", "domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("component2", "domain2", Logging.INFO) == true);
    
        check(filterMngr.isActivated("component2.component2a", "domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("component2.component2a", "domain2", Logging.INFO) == true);
    
        check(filterMngr.isActivated("component2.component2b", "domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("component2.component2b", "domain2", Logging.INFO) == true);
    
        println ("Deactivate every domain of the default component");
        filterMngr.deactivateLevel(Logging.ALL_DOMAINS, Logging.ALL_LEVELS_SET);

        check(filterMngr.isActivated("domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("domain1", Logging.FATAL) == false);
        check(filterMngr.isActivated("domain1", Logging.DEBUG) == false);
    
        check(filterMngr.isActivated("domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("domain2", Logging.ERROR) == false);
        check(filterMngr.isActivated("domain2", Logging.FATAL) == false);
        check(filterMngr.isActivated("domain2", Logging.DEBUG) == false);
    
        check(filterMngr.isActivated("component1", "domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("component1", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component1", "domain1", Logging.DEBUG) == false);
        check(filterMngr.isActivated("component1", "domain2", Logging.DEBUG) == false);

        println ("Activate every domain of the default component");
        filterMngr.activateLevel(Logging.ALL_DOMAINS, Logging.ALL_LEVELS_SET);

        check(filterMngr.isActivated("domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("domain1", Logging.WARN) == true);
        check(filterMngr.isActivated("domain1", Logging.ERROR) == true);
        check(filterMngr.isActivated("domain1", Logging.FATAL) == true);
        check(filterMngr.isActivated("domain1", Logging.DEBUG) == true);
    
        check(filterMngr.isActivated("domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("domain2", Logging.WARN) == true);
        check(filterMngr.isActivated("domain2", Logging.ERROR) == true);
        check(filterMngr.isActivated("domain2", Logging.FATAL) == true);
        check(filterMngr.isActivated("domain2", Logging.DEBUG) == true);
    
        check(filterMngr.isActivated("component1", "domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("component1", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component1", "domain1", Logging.DEBUG) == false);
        check(filterMngr.isActivated("component1", "domain2", Logging.DEBUG) == false);
     
        println ("Deactivate every domain of the default component");
        filterMngr.deactivateLevel(Logging.ALL_DOMAINS, Logging.ALL_LEVELS_SET);

        check(filterMngr.isActivated("domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("domain2", Logging.ERROR) == false);

        
        // Test C2
        println("Then activate domain1/level Logging.INFO");
        filterMngr.activateLevel("domain1", Logging.INFO);
        check(filterMngr.isActivated("domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("domain2", Logging.ERROR) == false);
    
        check(filterMngr.isActivated("component1", "domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("component1", "domain2", Logging.INFO) == true);

        // Test C3
        println("Then activate domain1/level Logging.ERROR");
        filterMngr.activateLevel("domain1", Logging.ERROR);
        check(filterMngr.isActivated("domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("domain1", Logging.ERROR) == true);
        check(filterMngr.isActivated("domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("domain2", Logging.ERROR) == false);


        // Test C4
        println("Then activate domain2/level Logging.INFO");
        filterMngr.activateLevel("domain2", Logging.INFO);
        check(filterMngr.isActivated("domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("domain1", Logging.ERROR) == true);
        check(filterMngr.isActivated("domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("domain2", Logging.ERROR) == false);


        // Test C5
        println("Then activate domain2/level Logging.ERROR");
        filterMngr.activateLevel("domain2", Logging.ERROR);
        check(filterMngr.isActivated("domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("domain1", Logging.ERROR) == true);
        check(filterMngr.isActivated("domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("domain2", Logging.ERROR) == true);
    

        // Test D1
        println("Deactivate every domain of the default component");
        filterMngr.deactivateLevel(Logging.ALL_DOMAINS, Logging.ALL_LEVELS_SET);
        check(filterMngr.isActivated("domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("domain2", Logging.ERROR) == false);

        // Test D2
        println("Then activate all the levels of domain1");
        filterMngr.activateLevel("domain1", Logging.ALL_LEVELS_SET);
        check(filterMngr.isActivated("domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("domain1", Logging.WARN) == true);
        check(filterMngr.isActivated("domain1", Logging.ERROR) == true);
        check(filterMngr.isActivated("domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("domain2", Logging.ERROR) == false);


        // Test D3
        println("Then activate level Logging.INFO of any domain");
        filterMngr.activateLevel(Logging.ALL_DOMAINS, Logging.INFO);
        check(filterMngr.isActivated("domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("domain1", Logging.WARN) == true);
        check(filterMngr.isActivated("domain1", Logging.ERROR) == true);
        check(filterMngr.isActivated("domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("domain2", Logging.ERROR) == false);


        // Test E1
        println("Deactivate everything");
        filterMngr.deactivateLevel(Logging.ALL_DOMAINS, Logging.ALL_LEVELS_SET);
        check(filterMngr.isActivated("domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("domain2", Logging.ERROR) == false);
    

        // Test E2
        println("Then activate all the levels of domain1");
        filterMngr.activateLevel("domain1", Logging.ALL_LEVELS_SET);
        check(filterMngr.isActivated("domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("domain1", Logging.WARN) == true);
        check(filterMngr.isActivated("domain1", Logging.ERROR) == true);
        check(filterMngr.isActivated("domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("domain2", Logging.ERROR) == false);

        // Test E3
        println("Then deactivate level Logging.INFO of any domain");
        filterMngr.deactivateLevel(Logging.ALL_DOMAINS, Logging.INFO);
        check(filterMngr.isActivated("domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("domain1", Logging.WARN) == true);
        check(filterMngr.isActivated("domain1", Logging.ERROR) == true);
        check(filterMngr.isActivated("domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("domain2", Logging.ERROR) == false);

        // Test E4
        println("Then activate domain1/level Logging.INFO");
        filterMngr.activateLevel("domain1", Logging.INFO);
        check(filterMngr.isActivated("domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("domain1", Logging.WARN) == true);
        check(filterMngr.isActivated("domain1", Logging.ERROR) == true);
        check(filterMngr.isActivated("domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("domain2", Logging.ERROR) == false);



        // Test H1
        println("Activate everything");
        filterMngr.activateLevel(Logging.ALL_DOMAINS, Logging.ALL_LEVELS_SET);
        check(filterMngr.isActivated("domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("domain1", Logging.WARN) == true);
        check(filterMngr.isActivated("domain1", Logging.ERROR) == true);
        check(filterMngr.isActivated("domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("domain2", Logging.WARN) == true);
        check(filterMngr.isActivated("domain2", Logging.ERROR) == true);

        // Test H2
        println("Then deactivate all the levels of domain1");
        filterMngr.deactivateLevel("domain1", Logging.ALL_LEVELS_SET);
        check(filterMngr.isActivated("domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("domain2", Logging.WARN) == true);
        check(filterMngr.isActivated("domain2", Logging.ERROR) == true);
    
        // Test H3
        println("Then activate level Logging.INFO of any domain");
        filterMngr.activateLevel(Logging.ALL_DOMAINS, Logging.INFO);
        check(filterMngr.isActivated("domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("domain2", Logging.WARN) == true);
        check(filterMngr.isActivated("domain2", Logging.ERROR) == true);
    
        // Test H4
        println("Then deactivate domain1/level Logging.INFO");
        filterMngr.deactivateLevel("domain1", Logging.INFO);
        check(filterMngr.isActivated("domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("domain2", Logging.WARN) == true);
        check(filterMngr.isActivated("domain2", Logging.ERROR) == true);
    

        // Test ZD1
        println("Deactivate everything");
        filterMngr.deactivateLevel(Logging.ALL_COMPONENT_NAMES,
                                    Logging.ALL_DOMAINS,
                                    Logging.ALL_LEVELS_SET);
                                
        check(filterMngr.isActivated("domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("component1", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component1", "domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("component1", "domain3", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1b", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1b", "domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("component2", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component2", "domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("component2.component2a", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component2.component2a", "domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("component2.component2b", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component2.component2b", "domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("component3.component3a", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("domain2", Logging.ERROR) == false);

        // Test ZD2
        println("Then activate all the levels of domain1 of component1");
        filterMngr.activateLevel("component1", "domain1", Logging.ALL_LEVELS_SET);
        check(filterMngr.isActivated("component1", "domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("component1", "domain1", Logging.WARN) == true);
        check(filterMngr.isActivated("component1", "domain1", Logging.ERROR) == true);
        check(filterMngr.isActivated("component1", "domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("component1", "domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("component1", "domain2", Logging.ERROR) == false);
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1b", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1b", "domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("component2", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component2", "domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("domain1", Logging.ERROR) == false);

        // Test ZD3
        println("Then activate level Logging.INFO of any domain of component1");
        filterMngr.activateLevel("component1", Logging.ALL_DOMAINS, Logging.INFO);
        check(filterMngr.isActivated("component1", "domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("component1", "domain1", Logging.WARN) == true);
        check(filterMngr.isActivated("component1", "domain1", Logging.ERROR) == true);
        check(filterMngr.isActivated("component1", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component1", "domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("component1", "domain2", Logging.ERROR) == false);
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1b", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1b", "domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("component2", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component2", "domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("domain1", Logging.INFO) == false);

    
        // Test ZD3a
        println("Then activate level Logging.INFO of domain1 of component1.component1a");
        filterMngr.activateLevel("component1.component1a", "domain1", Logging.INFO);
        check(filterMngr.isActivated("component1", "domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("component1", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1b", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1b", "domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("component2", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component2", "domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("domain1", Logging.INFO) == false);
    
        // Test ZD3b
        println("Then deactivate level Logging.INFO of domain1 of component1.component1a");
        filterMngr.deactivateLevel("component1.component1a", "domain1", Logging.INFO);
        check(filterMngr.isActivated("component1", "domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("component1", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1b", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1b", "domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("component2", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component2", "domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("domain1", Logging.INFO) == false);

        // Test ZD4
        println("Then deactivate level Logging.INFO of domain1 of component1");
        filterMngr.deactivateLevel("component1", "domain1", Logging.INFO);
        check(filterMngr.isActivated("component1", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component1", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1b", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1b", "domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("component2", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component2", "domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("domain1", Logging.INFO) == false);
    


        // Test ZD5
        println("Then activate all the levels of domain2 of any component");
        filterMngr.activateLevel(Logging.ALL_COMPONENT_NAMES,
                                  "domain2",
                                  Logging.ALL_LEVELS_SET);
        check(filterMngr.isActivated("component1", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component1", "domain1", Logging.WARN) == true);
        check(filterMngr.isActivated("component1", "domain1", Logging.ERROR) == true);
        check(filterMngr.isActivated("component1", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component1", "domain2", Logging.WARN) == true);
        check(filterMngr.isActivated("component1", "domain2", Logging.ERROR) == true);
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.WARN) == true);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.ERROR) == true);
        check(filterMngr.isActivated("component1.component1b", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1b", "domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("component1.component1b", "domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("component1.component1b", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component1.component1b", "domain2", Logging.WARN) == true);
        check(filterMngr.isActivated("component1.component1b", "domain2", Logging.ERROR) == true);
        check(filterMngr.isActivated("component2", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component2", "domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("component2", "domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("component2", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component2", "domain2", Logging.WARN) == true);
        check(filterMngr.isActivated("component2", "domain2", Logging.ERROR) == true);
        check(filterMngr.isActivated("component2.component2a", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component2.component2a", "domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("component2.component2a", "domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("component2.component2a", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component2.component2a", "domain2", Logging.WARN) == true);
        check(filterMngr.isActivated("component2.component2a", "domain2", Logging.ERROR) == true);
        check(filterMngr.isActivated("component2.component2b", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component2.component2b", "domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("component2.component2b", "domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("component2.component2b", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component2.component2b", "domain2", Logging.WARN) == true);
        check(filterMngr.isActivated("component2.component2b", "domain2", Logging.ERROR) == true);
        check(filterMngr.isActivated("component3.component3a", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component3.component3a", "domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("component3.component3a", "domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("component3.component3a", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component3.component3a", "domain2", Logging.WARN) == true);
        check(filterMngr.isActivated("component3.component3a", "domain2", Logging.ERROR) == true);
        check(filterMngr.isActivated("domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("domain2", Logging.WARN) == true);
        check(filterMngr.isActivated("domain2", Logging.ERROR) == true);


        // Test ZD6
        println("Then deactivate level Logging.WARN of any domain of any component");
        filterMngr.deactivateLevel(Logging.ALL_COMPONENT_NAMES,
                                    Logging.ALL_DOMAINS,
                                    Logging.WARN);
        check(filterMngr.isActivated("component1", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component1", "domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("component1", "domain1", Logging.ERROR) == true);
        check(filterMngr.isActivated("component1", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component1", "domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("component1", "domain2", Logging.ERROR) == true);
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.ERROR) == true);
        check(filterMngr.isActivated("component1.component1b", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1b", "domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("component1.component1b", "domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("component1.component1b", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component1.component1b", "domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("component1.component1b", "domain2", Logging.ERROR) == true);
        check(filterMngr.isActivated("component2", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component2", "domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("component2", "domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("component2", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component2", "domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("component2", "domain2", Logging.ERROR) == true);
        check(filterMngr.isActivated("component2.component2a", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component2.component2a", "domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("component2.component2a", "domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("component2.component2a", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component2.component2a", "domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("component2.component2a", "domain2", Logging.ERROR) == true);
        check(filterMngr.isActivated("component2.component2b", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component2.component2b", "domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("component2.component2b", "domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("component2.component2b", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component2.component2b", "domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("component2.component2b", "domain2", Logging.ERROR) == true);
        check(filterMngr.isActivated("component3.component3a", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component3.component3a", "domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("component3.component3a", "domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("component3.component3a", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component3.component3a", "domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("component3.component3a", "domain2", Logging.ERROR) == true);
        check(filterMngr.isActivated("domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("domain2", Logging.ERROR) == true);
    


        // Test ZD7
        println("Then activate level Logging.INFO of any domain of component1");
        filterMngr.activateLevel("component1", Logging.ALL_DOMAINS, Logging.INFO);
        check(filterMngr.isActivated("component1", "domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("component1", "domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("component1", "domain1", Logging.ERROR) == true);
        check(filterMngr.isActivated("component1", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component1", "domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("component1", "domain2", Logging.ERROR) == true);
        check(filterMngr.isActivated("component1", "domain3", Logging.INFO) == true);
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component2", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component2", "domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("component2", "domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("component2", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component2", "domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("component2", "domain2", Logging.ERROR) == true);
    
        // Test ZD7a
        println("Then deactivate level Logging.INFO of domain3 of component1");
        filterMngr.deactivateLevel("component1", "domain3", Logging.INFO);
        check(filterMngr.isActivated("component1", "domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("component1", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component1", "domain3", Logging.INFO) == false);
    

        // Test ZD8
        println("Then activate level Logging.WARN+Logging.ERROR of any domain of component1");
        filterMngr.activateLevel("component1", Logging.ALL_DOMAINS, Logging.WARN | Logging.ERROR);
        check(filterMngr.isActivated("component1", "domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("component1", "domain1", Logging.WARN) == true);
        check(filterMngr.isActivated("component1", "domain1", Logging.ERROR) == true);
        check(filterMngr.isActivated("component1", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component1", "domain2", Logging.WARN) == true);
        check(filterMngr.isActivated("component1", "domain2", Logging.ERROR) == true);
        check(filterMngr.isActivated("component1", "domain3", Logging.INFO) == false);
        check(filterMngr.isActivated("component1", "domain3", Logging.WARN) == true);
        check(filterMngr.isActivated("component1", "domain3", Logging.ERROR) == true);
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.ERROR) == true);

        // Test ZD9
        println("Then deactivate level Logging.WARN of domain2 of component1");
        filterMngr.deactivateLevel("component1", "domain2", Logging.WARN);
        check(filterMngr.isActivated("component1", "domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("component1", "domain1", Logging.WARN) == true);
        check(filterMngr.isActivated("component1", "domain1", Logging.ERROR) == true);
        check(filterMngr.isActivated("component1", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component1", "domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("component1", "domain2", Logging.ERROR) == true);
        check(filterMngr.isActivated("component1", "domain3", Logging.INFO) == false);
        check(filterMngr.isActivated("component1", "domain3", Logging.WARN) == true);
        check(filterMngr.isActivated("component1", "domain3", Logging.ERROR) == true);
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.ERROR) == true);


        // Test ZD10
        println("Then deactivate all the levels of domain2 of component1");
        filterMngr.deactivateLevel("component1", "domain2", Logging.ALL_LEVELS_SET);
        check(filterMngr.isActivated("component1", "domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("component1", "domain1", Logging.WARN) == true);
        check(filterMngr.isActivated("component1", "domain1", Logging.ERROR) == true);
        check(filterMngr.isActivated("component1", "domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("component1", "domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("component1", "domain2", Logging.ERROR) == false);
        check(filterMngr.isActivated("component1", "domain3", Logging.INFO) == false);
        check(filterMngr.isActivated("component1", "domain3", Logging.WARN) == true);
        check(filterMngr.isActivated("component1", "domain3", Logging.ERROR) == true);
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.ERROR) == true);


        // Test ZD11
        println("Then deactivate level Logging.WARN of any domain of component1");
        filterMngr.deactivateLevel("component1", Logging.ALL_DOMAINS, Logging.WARN);
        check(filterMngr.isActivated("component1", "domain1", Logging.INFO) == true);
        check(filterMngr.isActivated("component1", "domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("component1", "domain1", Logging.ERROR) == true);
        check(filterMngr.isActivated("component1", "domain2", Logging.INFO) == false);
        check(filterMngr.isActivated("component1", "domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("component1", "domain2", Logging.ERROR) == false);
        check(filterMngr.isActivated("component1", "domain3", Logging.INFO) == false);
        check(filterMngr.isActivated("component1", "domain3", Logging.WARN) == false);
        check(filterMngr.isActivated("component1", "domain3", Logging.ERROR) == true);
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.INFO) == false);
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.WARN) == false);
        check(filterMngr.isActivated("component1.component1a", "domain1", Logging.ERROR) == false);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.INFO) == true);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.WARN) == false);
        check(filterMngr.isActivated("component1.component1a", "domain2", Logging.ERROR) == true);
    }
} 
