/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES 
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *  
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */


package cdmw.logging.test;

import  cdmw.logging.*;
import static cdmw.logging.Logger.*;
import cdmw.testutils.Testable;
import cdmw.common.*;


public class TestLogger extends Testable 
{

    class FunctionLoggerUser
    {
        private Logger logger; 
        private FunctionLogger funcLog;
        
        public FunctionLoggerUser(Logger logger) {
            this.logger = logger; 
        }
        
        public void op1() {
            this.funcLog = FunctionLogger.CDMW_LOG_FUNCTION(this.logger,"", new Throwable());
        }
        
        public int op_return(int input) {
            this.funcLog = FunctionLogger.CDMW_LOG_FUNCTION(this.logger,"", new Throwable());
            this.funcLog.logReturn(input, new Throwable());
            return input;
        }
        
        public void op_exception(int input) throws java.lang.Exception {
            this.funcLog = FunctionLogger.CDMW_LOG_FUNCTION(this.logger,"", new Throwable());
            this.funcLog.logException(input, new Throwable());
            throw new java.lang.Exception(String.valueOf(input));
        }
    }

    public TestLogger() {
        this.name = "TestLogger";
        this.out = System.out;
    }

    public void doTests() {
        setNbOfRequestedTestOK(35);

        Logger logger;
        LevelType level;
        Long lineNum;
        String fileName;
        
        println("Obtaining a default logger for the TEST_LOGGING domain");
        println("Only messages with a level <= INFO (ie lvl <= 8) are displayed"); 
       
        lineNum = 1L;
        fileName = "file1";

        logger = Logger.Get_Logger("TEST_LOGGING");
        check (logger != null);
        succeed ("Logger created succesfully");

        println ("Logging an info message");
        
        logger.info ("my info message 1", new Throwable());
        if (logger.isInfoEnabled()) {
            succeed ("INFO LEVEL enabled on logger");
        }
        else {
            fail ("INFO LEVEL failed to be set on logger");
        }

        CDMW_INFO(logger,"my info message 1", new Throwable());
        if (logger.isInfoEnabled()) {
            succeed ("INFO LEVEL enabled on logger");
        }
        else {
            fail ("INFO LEVEL failed to be set on logger");
        }

        println ("Logging a warning message");
        
        logger.warn ("my warning message 1", new Throwable());
        if (logger.isWarnEnabled()) {
            succeed ("WARN LEVEL enabled on logger");
        }
        else {
            fail ("WARN LEVEL failed to be set on logger");
        }

        CDMW_WARN(logger,"my info message 1", new Throwable());
        if (logger.isInfoEnabled()) {
            succeed ("INFO WARN enabled on logger");
        }
        else {
            fail ("WARN LEVEL failed to be set on logger");
        }

        println ("Logging an error message");
        
        logger.error ("my error message 1", new Throwable());
        if (logger.isErrorEnabled()) {
            succeed ("ERROR LEVEL enabled on logger");
        }
        else {
            fail ("ERROR LEVEL failed to be set on logger");
        }


        CDMW_ERROR(logger,"my info message 1", new Throwable());
        if (logger.isErrorEnabled()) {
            succeed ("ERROR LEVEL enabled on logger");
        }
        else {
            fail ("ERROR LEVEL failed to be set on logger");
        }
        
        println ("Logging a fatal error message");
        
        logger.fatal ("my fatal error message 1", new Throwable());
        if (logger.isFatalEnabled()) {
            succeed ("FATAL LEVEL enabled on logger");
        }
        else {
            fail ("FATAL LEVEL failed to be set on logger");
        }
        
        CDMW_FATAL(logger,"my info message 1", new Throwable());
        if (logger.isFatalEnabled()) {
            succeed ("FATAL LEVEL enabled on logger");
        }
        else {
            fail ("FATAL LEVEL failed to be set on logger");
        }

        println ("Logging a debug message");
        
        logger.debug ("my debug message 1",  new Throwable());
        if (logger.isDebugEnabled()) {
            fail ("DEBUG LEVEL shouldn't have been enabled on logger");
        }
        else {
            succeed ("DEBUG LEVEL successfully not set on logger");
        }

        CDMW_DEBUG(logger,"my info message 1", new Throwable());
        if (logger.isFatalEnabled()) {
            succeed ("DEBUG LEVEL enabled on logger");
        }
        else {
            fail ("DEBUG LEVEL failed to be set on logger");
        }

        level = new LevelType (LogLevel.DEBUG_1);
        CDMW_DEBUG_1(logger,"my generic message 1",new Throwable());
        if (logger.isEnabledFor(level)) {
            fail ("Generic message logged.");
        }
        else {
            succeed ("PASS:- DEBUG_1 not logged generic message");
        }

        level = new LevelType (LogLevel.DEBUG_2);
        CDMW_DEBUG_2(logger,"my generic message 2",new Throwable());
        if (logger.isEnabledFor(level)) {
            fail ("Generic message logged.");
        }
        else {
            succeed ("PASS:- DEBUG_2 not logged generic message");
        }

        level = new LevelType (LogLevel.DEBUG_3);
        CDMW_DEBUG_3(logger,"my generic message 3",new Throwable());
        if (logger.isEnabledFor(level)) {
            fail ("Generic message logged.");
        }
        else {
            succeed ("PASS:- DEBUG_3 not logged generic message");
        }

        level = new LevelType (LogLevel.DEBUG_4);
        CDMW_DEBUG_4(logger,"my generic message 4",new Throwable());
        if (logger.isEnabledFor(level)) {
            fail ("Generic message logged.");
        }
        else {
            succeed ("PASS:- DEBUG_4 not logged generic message");
        }

        level = new LevelType (LogLevel.DEBUG_5);
        CDMW_DEBUG_5(logger,"my generic message 5",new Throwable());
        if (logger.isEnabledFor(level)) {
            fail ("Generic message logged.");
        }
        else {
            succeed ("PASS:- DEBUG_5 not logged generic message");
        }

        level = new LevelType (LogLevel.DEBUG_6);
        CDMW_DEBUG_6(logger,"my generic message 6",new Throwable());
        if (logger.isEnabledFor(level)) {
            fail ("Generic message logged.");
        }
        else {
            succeed ("PASS:- DEBUG_6 not logged generic message");
        }

        level = new LevelType (LogLevel.DEBUG_7);
        CDMW_DEBUG_7(logger,"my generic message 7",new Throwable());
        if (logger.isEnabledFor(level)) {
            fail ("Generic message logged.");
        }
        else {
            succeed ("PASS:- DEBUG_7 not logged generic message");
        }

        level = new LevelType (LogLevel.DEBUG_8);
        CDMW_DEBUG_8(logger,"my generic message 8",new Throwable());
        if (logger.isEnabledFor(level)) {
            fail ("Generic message logged.");
        }
        else {
            succeed ("PASS:- DEBUG_8 not logged generic message");
        }

        level = new LevelType (LogLevel.DEBUG_9);
        CDMW_DEBUG_9(logger,"my generic message 9",new Throwable());
        if (logger.isEnabledFor(level)) {
            fail ("Generic message logged.");
        }
        else {
            succeed ("PASS:- DEBUG_9 not logged generic message");
        }

        println ("Logging a generic message with level 7.");

        level = new LevelType (7);
        logger.log (level, "my generic message 1",new Throwable());
        if (logger.isEnabledFor(level)) {
            succeed ("Generic message with level_7 logged.");
        }
        else {
            fail ("Failed to log generic message with level_7.");
        }

        level = new LevelType (20);
        logger.log (level, "my generic message 2",new Throwable());
        if (logger.isEnabledFor(level)) {
            fail ("Fail:-Generic message with level_20 logged.");
        }
        else {
            succeed ("Success:-Could not log generic message with level_20.");
        }

        logger = Logger.Get_Logger ("COMPONENT1", "TEST_LOGGING");
        check (logger != null);
        succeed ("Logger created succesfully");

        println ("Logging an info message");
        
        logger.info ("my info message 2", new Throwable());
        if (logger.isInfoEnabled()) {
            succeed ("INFO LEVEL enabled on logger");
        }
        else {
            fail ("INFO LEVEL failed to be set on logger");
        }

        println ("Logging a warning message");
        
        logger.warn ("my warning message 2", new Throwable());
        if (logger.isWarnEnabled()) {
            succeed ("WARN LEVEL enabled on logger");
        }
        else {
            fail ("WARN LEVEL failed to be set on logger");
        }

        println ("Logging an error message");
        
        logger.error ("my error message 2", new Throwable());
        if (logger.isErrorEnabled()) {
            succeed ("ERROR LEVEL enabled on logger");
        }
        else {
            fail ("ERROR LEVEL failed to be set on logger");
        }
        
        println ("Logging a fatal error message");
        
        logger.fatal ("my fatal error message 2", new Throwable());
        if (logger.isFatalEnabled()) {
            succeed ("FATAL LEVEL enabled on logger");
        }
        else {
            fail ("FATAL LEVEL failed to be set on logger");
        }

        println ("Logging a debug message (lvl >8).");
        logger.debug ("My debug message 2",  new Throwable());
        if (logger.isDebugEnabled()) {
            fail("FAIL:- DEBUG shouldn't be enabled");
        }
        else {
            succeed("PASS:-DEBUG not enabled.");
        }

        println ("Logging a generic message with level 7");
        level = new LevelType (7);
        logger.log (level, "my generic message 3", new Throwable());
        if (logger.isDebugEnabled()) {
            fail("FAIL:- DEBUG shouldn't be enabled");
        }
        else {
            succeed("PASS:-DEBUG not enabled.");
        }
      
        println ("Logging a generic message with level 20");
        level = new LevelType (20);
        logger.log (level, "my generic message 4", new Throwable());
        if (logger.isEnabledFor(level)) {
            fail("FAIL:- Log shouldn't be enabled for level 20.");
        }
        else {
            succeed("PASS:-Log not enabled for level 20.");
        }

        println("Calling the Function logger. VISUAL CHECK!.");
        FunctionLoggerUser flu = new FunctionLoggerUser(logger);

        println("Calling op1 (with no return and no exception).");
        flu.op1();
        succeed("PASS: Calling op1 (with no return and no exception).");

        int retval = 30;
        println("Calling op_return that return value " + retval);
        int res = flu.op_return(retval);
        
        if (res != retval) {
            fail("FAIL:- Calling op_return that return value " + retval);
        }
        else {
            succeed("PASS:- Calling op_return that return value " + retval);
        }
        
        retval = 50;
        println("Calling op_exception that should raise exception with value " + retval);
        try {
            flu.op_exception(retval);
            fail("FAIL:- Calling op_exception that should raise exception with value " + retval);
        } catch (java.lang.Exception ex) {
            if (ex.getLocalizedMessage().equals(String.valueOf(retval))) {       
                succeed("PASS:- Calling op_exception that should raise exception with value " + retval);
            }
            else {
                fail("FAIL:- Calling op_exception that should raise exception with value " + retval);
            }
        }
        
    }
}
        
        

        

        


