/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES 
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */

package cdmw.logging.test.testorblogging;
import cdmw.logging.*;
import cdmw.ossupport.*;
import cdmw.orbsupport.*;
import cdmw.common.*;
import java.io.*;
import org.omg.CORBA.*;
import org.omg.PortableServer.POAHelper;

public class MainTest
{
    int SUCCESS = 0;
    int FAILURE = 1;

    static int POA_THREAD_POOL_SIZE = 5;
   
   public static void main(String[] args) {

    try 
    {
        LogManager.Init(args);
        // ===================================================
        // Initialize the ORB
        // ===================================================
        StrategyList orbStrategies = new StrategyList();
        orbStrategies.addORBThreaded();
        orbStrategies.addPOAThreadPool(POA_THREAD_POOL_SIZE);
        
        org.omg.CORBA.ORB orb = ORBSupport.ORBInit(args, orbStrategies);
        
        System.out.println("Created ORB");

        // ===================================================
        // Add ORB shutdown hooks
        // ===================================================
        
        OrbShutdownSignalHandler sighandler = new OrbShutdownSignalHandler (orb);
        
        try
        {
           ORBSupport.InstallORBShutdownSignalHandler (orb, sighandler);
        }
        catch (java.lang.Exception ex)
        {
            System.err.println ("Caught exception during InstallORBShutdownSignalHandler: " + ex.getMessage());
            ex.printStackTrace();
        }

        System.out.println ("Added shutdown Hooks");
        Thread.sleep(3000);
        // ===================================================
        // Get the root POA 
        // ===================================================
        org.omg.CORBA.Object obj = orb.resolve_initial_references("RootPOA");
        org.omg.PortableServer.POA rootPOA = POAHelper.narrow(obj);
                
        System.out.println("Created POA");
        // ===================================================
        // Activate the root POA manager
        // ===================================================
        org.omg.PortableServer.POAManager poaManager = rootPOA.the_POAManager();

        poaManager.activate();
        
        System.out.println("POA activated");
        // Are we started as the CORBA server ?
        
        if ( cdmw.common.Options.getOptionValue( args, "--server") == "yes" )
        {            
            TestHello_impl hello_impl = new TestHello_impl(orb);
            TestHello_impl hello = hello_impl;
            org.omg.CORBA.Object objImpl = rootPOA.servant_to_reference( hello_impl ); 
            
            // We write its ior in a file
            FileWriter iorFile = new FileWriter("TestHello.ior");
            String strIor = orb.object_to_string( objImpl );            
            iorFile.write(strIor); 
            iorFile.close();
            
            // Setup signal handling for ORB to be shut down
            //OrbSupport.install_orb_shutdown_signal_handlers(orb);

            orb.run();

            //OrbSupport.uninstall_orb_shutdown_signal_handlers(orb);
        }
        else
        {
            System.out.println("else");
            // Else, we are the main test driver
            System.setProperty ("name","MainTest");
            System.out.println(args);

            if (  cdmw.common.Options.getOptionValue( args, "--client-only") == "yes" )
              {
                System.setProperty ("client_only", "yes");
              }
            else
              {
                System.setProperty ("client_only", "no");
              }

            if (  cdmw.common.Options.getOptionValue( args, "--no-ORBlog-check") == "yes" )
              {
                System.setProperty ("no_orb_log_check", "yes");
              }
            else
              {
                System.setProperty ("no_orb_log_check", "no");
              }

            if (  cdmw.common.Options.getOptionValue( args, "--server-only") == "yes" )
              {
                System.setProperty ("server_only", "yes");
              }
            else
              {
                System.setProperty ("server_only", "no");
              }
            System.out.println("Creating new TestORBLoggingDriver");
            TestOrbLoggingDriver testLog = new TestOrbLoggingDriver(orb,"cdmw.logging.test.testorblogging.MainTest");
            testLog.start();
        }

        if (orb != null) 
        {
            try 
            {                
                ORBSupport.ORBCleanup(orb);
                orb.destroy();
            } 
            catch (java.lang.Exception e) 
            {
                System.out.println("Unexpected Exception during orb.detroy():");
                e.printStackTrace() ;
            }
           
            try 
            {                
                ORBSupport.UninstallORBShutdownSignalHandler (sighandler);
            } 
            catch (java.lang.Exception e) 
            {
                System.out.println("Unexpected Exception during UninstallORBShutdownSignalHandler():");
                e.printStackTrace() ;
            }
            
        }
                
    } 
    catch(java.lang.Exception ex)  
    {        
         ex.printStackTrace();
    }
}
}
