/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
 *
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */
package cdmw.orbsupport;

import cdmw.logging.*;

public class LoggerAdapter implements org.apache.avalon.framework.logger.Logger
{
        private cdmw.logging.Logger logger;
       
        /**
         * Correspondance between Jacorb levels and Cardamom levels
         */
        private static final LevelType JACORB_FATAL_LEVEL = new LevelType(LogLevel.FATAL);
        private static final LevelType JACORB_ERROR_LEVEL = new LevelType(LogLevel.ERROR);
        private static final LevelType JACORB_WARN_LEVEL  = new LevelType(LogLevel.WARN);
        private static final LevelType JACORB_INFO_LEVEL  = new LevelType(LogLevel.DEBUG_8);
        private static final LevelType JACORB_DEBUG_LEVEL = new LevelType(LogLevel.DEBUG_9);
        

        /**
         * create LoggerAdapter
         *
         *@param String name
         */ 
        public LoggerAdapter(String name)
        {
            this.logger = Logger.Get_Logger("JacORB", name);
        }
        

        /**
         * create debug log 
         *
         *@param String message 
         */ 
        public void debug(java.lang.String message)
        {
            if (this.logger.isEnabledFor(JACORB_DEBUG_LEVEL))
            {
                this.logger.log(JACORB_DEBUG_LEVEL, message,null);
            }
        }
        
        /**
         * create debug log 
         *
         *@param String message 
         *@param java.lang.Throwable throwable 
         */ 
        public void debug(java.lang.String message, java.lang.Throwable throwable)
        {
            if (this.logger.isEnabledFor(JACORB_DEBUG_LEVEL))
            {        
                this.logger.log(JACORB_DEBUG_LEVEL, message, null);
            }
        }
        
        /**
         * create error log 
         *
         *@param String message 
         */ 
        public void error(java.lang.String message)
        {
            if (this.logger.isEnabledFor(JACORB_ERROR_LEVEL))
            {
                this.logger.log(JACORB_ERROR_LEVEL, message, null);
            }
        }

        /**
         * create error log 
         *
         *@param String message 
         *@param java.lang.Throwable throwable 
         */ 
        public void error(java.lang.String message, java.lang.Throwable throwable)
        {
            if (this.logger.isEnabledFor(JACORB_ERROR_LEVEL))
            {
                this.logger.log(JACORB_ERROR_LEVEL, message, null);
            }
        }
        
        /**
         * create fatalError log 
         *
         *@param String message 
         */ 
        public void fatalError(java.lang.String message)
        {
            if (this.logger.isEnabledFor(JACORB_FATAL_LEVEL))
            {
                this.logger.log(JACORB_ERROR_LEVEL, message, null);
            }
        }

        /**
         * create fatalError log 
         *
         *@param String message 
         *@param java.lang.Throwable throwable 
         */ 
        public void fatalError(java.lang.String message, java.lang.Throwable throwable)
        {
            if (this.logger.isEnabledFor(JACORB_FATAL_LEVEL))
            {
                this.logger.log(JACORB_ERROR_LEVEL, message, null);
            }        
        }

        /**
         * getChildLogger
         *
         *@param String name 
         *
         *@return org.apache.avalon.framework.logger.Logger
         */ 
        public org.apache.avalon.framework.logger.Logger getChildLogger(java.lang.String name)
        {
            return this;
        }

        /**
         * create info log 
         *
         *@param String message 
         */ 
        public void info(java.lang.String message)
        {
            if (this.logger.isEnabledFor(JACORB_INFO_LEVEL))
            {
                this.logger.log(JACORB_INFO_LEVEL, message, null);
            }
        }

        /**
         * create info log 
         *
         *@param String message 
         *@param java.lang.Throwable throwable 
         */ 
        public void info(java.lang.String message, java.lang.Throwable throwable)
        {
            if (this.logger.isEnabledFor(JACORB_INFO_LEVEL))
            {
                this.logger.log(JACORB_INFO_LEVEL, message, null);
            }
        }
        
        /**
         * Determine if messages of priority "debug" will be logged. 
         *
         *@return boolean
         */ 
        public boolean isDebugEnabled()
        {
            return this.logger.isEnabledFor(JACORB_DEBUG_LEVEL);
        }

        /**
         * Determine if messages of priority "error" will be logged. 
         *
         *@return boolean
         */ 
        public boolean isErrorEnabled()
        {
            return this.logger.isEnabledFor(JACORB_ERROR_LEVEL);
        }

        /**
         * Determine if messages of priority "fatalError" will be logged. 
         *
         *@return boolean
         */ 
        public boolean isFatalErrorEnabled()
        {
            return this.logger.isEnabledFor(JACORB_FATAL_LEVEL);
        }

        /**
         * Determine if messages of priority "info" will be logged. 
         *
         *@return boolean
         */ 
        public boolean isInfoEnabled()
        {
            return this.logger.isEnabledFor(JACORB_INFO_LEVEL);
        }

        /**
         * Determine if messages of priority "warn" will be logged. 
         *
         *@return boolean
         */ 
        public boolean isWarnEnabled()
        {
            return this.logger.isEnabledFor(JACORB_WARN_LEVEL);
        }

        /**
         * Determine if messages of priority "warn" will be logged. 
         *
         *@param String message
         *
         */ 
        public void warn(java.lang.String message)
        {
            if (this.logger.isEnabledFor(JACORB_WARN_LEVEL))
                {
                    this.logger.log(JACORB_WARN_LEVEL, message, null);
                }
        }

        /**
         * Determine if messages of priority "warn" will be logged. 
         *
         *@param String message 
         *@param java.lang.Throwable throwable
         *         
         */ 
        public void warn(java.lang.String message, java.lang.Throwable throwable)
        {
            if (this.logger.isEnabledFor(JACORB_WARN_LEVEL))
                {
                    this.logger.log(JACORB_WARN_LEVEL, message, null);
                }
        }
             
}
