/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES 
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */



#include <ccm_ft_lb/CCM_Client_impl.hpp>

#include <ccm_ft_lb/Command_impl.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>


//
// IDL:thalesgroup.com/CcmFtLbTest/CCM_Client:1.0
//
CcmFtLbTest::CCM_Client_impl::CCM_Client_impl()
	: m_expectedPrimaryHostname(""),
      m_expectedBackup1Hostname(""),
      m_hostnameCommandResult(""),
      m_session_context(CcmFtLbTest::CCM_Client_Context::_nil()),
      m_testThread()
{
}

CcmFtLbTest::CCM_Client_impl::~CCM_Client_impl()
{
    std::cout << "   ... CCM_Client_impl::~CCM_Client_impl called!" << std::endl;
}




//
// attribute expectedPrimaryHostname
//
char*
CcmFtLbTest::CCM_Client_impl::expectedPrimaryHostname()
    throw(CORBA::SystemException)
{
    return CORBA::string_dup(m_expectedPrimaryHostname);
}

void
CcmFtLbTest::CCM_Client_impl::expectedPrimaryHostname(const char* a)
    throw(CORBA::SystemException)
{
    m_expectedPrimaryHostname =  CORBA::string_dup(a);
}


//
// attribute expectedBackup1Hostname
//
char*
CcmFtLbTest::CCM_Client_impl::expectedBackup1Hostname()
    throw(CORBA::SystemException)
{
    return CORBA::string_dup(m_expectedBackup1Hostname);
}

void
CcmFtLbTest::CCM_Client_impl::expectedBackup1Hostname(const char* a)
    throw(CORBA::SystemException)
{
    m_expectedBackup1Hostname =  CORBA::string_dup(a);
}


//
// attribute expectedBackup2Hostname
//
char*
CcmFtLbTest::CCM_Client_impl::expectedBackup2Hostname()
    throw(CORBA::SystemException)
{
    return CORBA::string_dup(m_expectedBackup2Hostname);
}

void
CcmFtLbTest::CCM_Client_impl::expectedBackup2Hostname(const char* a)
    throw(CORBA::SystemException)
{
    m_expectedBackup2Hostname =  CORBA::string_dup(a);
}


//
// facet clientHostInfo
//
CcmFtLbTest::CCM_HostInfo_ptr 
CcmFtLbTest::CCM_Client_impl::get_clientHostInfo ()
    throw (CORBA::SystemException)
{
    return CcmFtLbTest::CCM_HostInfo::_duplicate(this);
}

char*
CcmFtLbTest::CCM_Client_impl::get_hostname()
    throw (CORBA::SystemException)
{
    std::string hostname = Cdmw::OsSupport::OS::get_hostname();
    return CORBA::string_dup(hostname.c_str());
}    


    
//
// consumer hostnameResults
//
void 
CcmFtLbTest::CCM_Client_impl::push_hostnameResults (CcmFtLbTest::HostnameResult * event)
    throw (CORBA::SystemException)
{
    std::cout << "   ... CCM_Client_impl::CCM_Client_impl::push_hostnameResults called with event " << event->hostname() << std::endl;
    m_hostnameCommandResult = event->hostname();
}
	 



//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
CcmFtLbTest::CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Client_impl::Set_session_context called!" << std::endl;
   
    m_session_context = ::CcmFtLbTest::CCM_Client_Context::_narrow(ctx); 
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CcmFtLbTest::CCM_Client_impl::ccm_activate()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_Client_impl::Ccm_activate called!" << std::endl;
   Cdmw::TestUtils::CORBATestManager::instance()->add("client_context", get_context());
   Cdmw::TestUtils::CORBATestManager::instance()->add("user_ccm_client", static_cast< ::UserCcmFtLbTest::User_CCM_Client_ptr > (this));
   m_testThread.start();
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CcmFtLbTest::CCM_Client_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_Client_impl::Ccm_passivate called!" << std::endl;
} 



//
// ccm_remove function : it destroys the component (call by the container)
//
void
CcmFtLbTest::CCM_Client_impl::ccm_remove()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_Client_impl::Ccm_remove called!" << std::endl;
}



//
// get result of assynchronous getHostname command
//
char* 
CcmFtLbTest::CCM_Client_impl::getHostnameCommandResult()
{
    char* result = CORBA::string_dup(m_hostnameCommandResult.in());
    m_hostnameCommandResult = "";
    return result;
}



//
// Testable do_tests() operation
//

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(CcmFtLbTest::test_ccm_ft_activate, "test_ccm_ft_activate");

void
CcmFtLbTest::test_ccm_ft_activate::do_tests()
{
    Cdmw::OsSupport::OS::sleep(2000);
    
    CORBA::Object_var obj2 = Cdmw::TestUtils::CORBATestManager::instance()->get_object("user_ccm_client");
    ::UserCcmFtLbTest::User_CCM_Client_var user_ccm_client = ::UserCcmFtLbTest::User_CCM_Client::_narrow(obj2.in());
    m_user_ccm_client = reinterpret_cast< CcmFtLbTest::CCM_Client_impl* > (user_ccm_client.in());

//     set_nbOfRequestedTestOK(0);
    
    // test connections to group (primary hostname is expected)
    test_connections(m_user_ccm_client->expectedPrimaryHostname());
    
    try {
        TEST_INFO("Send kill command to group (only primary will stop)");    
    
        CcmFtLbTest::Command_var command = new CcmFtLbTest::Command_impl();
        command->commandName(CcmFtLbTest::COMMAND_KILL);
    
        TEST_INFO("    emits command " <<  command->commandName());
        m_session_context->push_commandEmits(command.in());
        
        TEST_INFO("    wait new primary election");
        Cdmw::OsSupport::OS::sleep(6000);

    } catch (const CORBA::SystemException& e) {
        TEST_INFO("Unexpected CORBA::SystemException: " << e);
        CPPUNIT_ASSERT(false);
    }
    
    // test connections to group (backup1 hostname is expected)
    test_connections(m_user_ccm_client->expectedBackup1Hostname());


    try {
        TEST_INFO("Send kill command to group (only backup1 will stop)");    
    
        CcmFtLbTest::Command_var command = new CcmFtLbTest::Command_impl();
        command->commandName(CcmFtLbTest::COMMAND_KILL);
    
        TEST_INFO("    emits command " <<  command->commandName());
        m_session_context->push_commandEmits(command.in());
        
        TEST_INFO("    wait new primary election");
        Cdmw::OsSupport::OS::sleep(6000);

    } catch (const CORBA::SystemException& e) {
        TEST_INFO("Unexpected CORBA::SystemException: " << e);
        CPPUNIT_ASSERT(false);
    }
    
    // test connections to group (backup2 hostname is expected)
    test_connections(m_user_ccm_client->expectedBackup2Hostname());

    try {
        TEST_INFO("Send kill command to group (last primary will stop)");    
    
        CcmFtLbTest::Command_var command = new CcmFtLbTest::Command_impl();
        command->commandName(CcmFtLbTest::COMMAND_KILL);
    
        TEST_INFO("    emits command " <<  command->commandName());
        m_session_context->push_commandEmits(command.in());
        
        TEST_INFO("    wait new primary election");
        Cdmw::OsSupport::OS::sleep(6000);

    } catch (const CORBA::SystemException& e) {
        TEST_INFO("Unexpected CORBA::SystemException: " << e);
        CPPUNIT_ASSERT(false);
    }
    
    try {
        TEST_INFO("Test that FT group is empty");
//         add_nbOfRequestedTestOK(1);
        
        CcmFtLbTest::HostInfo_var hostInfo = m_session_context->get_connection_serverHostInfo();
    
        CORBA::String_var hostname = hostInfo->get_hostname();
        TEST_INFO("Request on empty group succeed !! hostname: " << hostname.in());
        CPPUNIT_ASSERT(false);
         
    } catch (const CORBA::SystemException& e) {
        CPPUNIT_ASSERT(true);
    } catch (const CORBA::Exception& e) {
        TEST_INFO("Unexpected CORBA::Exception: " << e);
        CPPUNIT_ASSERT(false);
    }

    
    Cdmw::OsSupport::OS::sleep(2000);
    CORBA::String_var temp = CORBA::string_dup("");
	 
    // perform 10 request with the RoundRobin policy
    for (int i = 0; i<10; i++)
    {
	CORBA::String_var process_name = CORBA::string_dup(test_lb_connections());
	TEST_INFO("REQUEST " << i << "...check the RoundRobin policy...") ;
	TEST_INFO("the process called is ==> " << process_name.in());
	if((strcmp(temp, process_name) == 0) && (!(i==5)))
       //TEST_FAILED();
       CPPUNIT_ASSERT(false);
	else
	{
	    temp = process_name;
        // TEST_SUCCEED();
        CPPUNIT_ASSERT(true);
	}
    }

    
}


void
CcmFtLbTest::test_ccm_ft_activate::test_connections(const char* expectedHostname)
{
//     add_nbOfRequestedTestOK(3);
    
    try {
        TEST_INFO("Test connections to ServerWithFacet group (expected primary host: " 
                  << expectedHostname << ")");    
    
        CcmFtLbTest::HostInfo_var hostInfo = m_session_context->get_connection_serverHostInfo();

        if (CORBA::is_nil(hostInfo.in())) {
            TEST_INFO("hostInfo is NIL !!!! ");
            CPPUNIT_ASSERT(false);
            return;
        }
        
        CORBA::String_var hostname = hostInfo->get_hostname();
        
        if (strcmp(expectedHostname, hostname.in()) == 0) {
            CPPUNIT_ASSERT(true);
        } else {
            TEST_INFO("Server's hostname is not the one expected: " << hostname.in());
            CPPUNIT_ASSERT(false);            
        }
        
    } catch (const CORBA::SystemException& e) {
        TEST_INFO("Unexpected CORBA::SystemException: " << e);
        CPPUNIT_ASSERT(false);
    } catch (const CORBA::Exception& e) {
        TEST_INFO("Unexpected CORBA::Exception: " << e);
        CPPUNIT_ASSERT(false);
    } catch (...) {
        TEST_INFO("Unknown exception !! ");
        CPPUNIT_ASSERT(false);
    }
    

    
    try {
        TEST_INFO("Test connections to ServerWithEvents group (expected primary host: " 
                  << expectedHostname << ")");    
    
        CcmFtLbTest::Command_var command = new CcmFtLbTest::Command_impl();
        command->commandName(CcmFtLbTest::COMMAND_GET_HOSTNAME);
    
        TEST_INFO("    emits command " <<  command->commandName());
        m_session_context->push_commandEmits(command.in());
        
        TEST_INFO("    wait result");
        Cdmw::OsSupport::OS::sleep(2000);
        
        CORBA::String_var hostname1 = m_user_ccm_client->getHostnameCommandResult();
        
        if (strcmp(expectedHostname, hostname1.in()) == 0) {
            CPPUNIT_ASSERT(true);
        } else {
            TEST_INFO("Server's hostname is not the one expected: " << hostname1.in());
            CPPUNIT_ASSERT(false);            
        }
        

        TEST_INFO("    publishes command " <<  command->commandName());
        m_session_context->push_commandPublishes(command.in());
        
        TEST_INFO("    wait result");
        Cdmw::OsSupport::OS::sleep(2000);
        
        CORBA::String_var hostname2 = m_user_ccm_client->getHostnameCommandResult();
        
        if (strcmp(expectedHostname, hostname2.in()) == 0) {
            CPPUNIT_ASSERT(true);
        } else {
            TEST_INFO("Server's hostname is not the one expected: " << hostname2.in());
            CPPUNIT_ASSERT(false);            
        }

    } catch (const CORBA::SystemException& e) {
        TEST_INFO("Unexpected CORBA::SystemException: " << e);
        CPPUNIT_ASSERT(false);
    }
}



char* CcmFtLbTest::test_ccm_ft_activate::test_lb_connections(void)
{
//    add_nbOfRequestedTestOK(1);

    CORBA::String_var process_name = CORBA::string_dup("");

    try {
        TEST_INFO("Testing connections to ServerLBWithFacet group");

        // get the server HostInfo
        CcmFtLbTest::ProcessInfo_var processInfo =
            m_session_context->get_connection_serverProcessInfo();

        if (CORBA::is_nil(processInfo.in())) {
           TEST_INFO("error: processInfo is NIL!");
            //TEST_FAILED();
           CPPUNIT_ASSERT(false);
            return 0;
        }

        // read the hostname from the server HostInfo
        process_name = processInfo->get_process_info();

        if (strcmp(process_name.in(), "") == 0) {
            TEST_INFO("error: hostname is empty string");
            //TEST_FAILED();
            CPPUNIT_ASSERT(false);
        } else {
            //TEST_SUCCEED();
           CPPUNIT_ASSERT(true);
        }
    } catch (const CORBA::SystemException& e) {
        TEST_INFO("Unexpected CORBA::SystemException: " << e);
        //TEST_FAILED();
        CPPUNIT_ASSERT(false);
    } catch (const CORBA::Exception& e) {
        TEST_INFO("Unexpected CORBA::Exception: " << e);
        //TEST_FAILED();
        CPPUNIT_ASSERT(false);
    } catch (...) {
        TEST_INFO("Unknown exception!");
        //TEST_FAILED();
        CPPUNIT_ASSERT(false);
    }

    return CORBA::string_dup(process_name);
}




