/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES 
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */

#include <ccm_reconnect/CCM_Client_impl.hpp>
#include <ccm_reconnect/TestEvent_impl.hpp>

#include <Foundation/common/Assert.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>


namespace 
{

/**
*  This class provides an object to test the ccm_activate.
*
**/
class test_ccm_client_activate;
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(test_ccm_client_activate, "test_ccm_client_activate");

class test_ccm_client_activate : public CppUnit::TestFixture
{
    
    ::ReconnectDemo::CCM_Client_Context_var m_context;
    ::ReconnectDemo::Supported_var m_supported;
    ::ReconnectDemo::EventChecking_var m_event_checking;
    
    
public:
    test_ccm_client_activate()
    {
    };
    
    ~test_ccm_client_activate() {};
    
    
    void perform_test(bool success_expected)
    {
        TEST_INFO("Perform calls on Server component");
        if (! success_expected)
            TEST_INFO("exceptions are expected");
        
        TEST_INFO("Test calling supported interface");
        try {
            m_supported->test_supported();
            CPPUNIT_ASSERT(success_expected);
        } catch (const CORBA::SystemException& e) {
            TEST_INFO("Exception received: " 
                      << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            CPPUNIT_ASSERT(!success_expected);
        }
        
        TEST_INFO("Test sending event");
        try {
            ReconnectDemo::TestEvent_var event = new ReconnectDemo::TestEvent_impl;
            event->name("TEST");
            m_context->push_event_publish(event.in());
            // event sending should always be succesfull
            CPPUNIT_ASSERT(true);
        } catch (const CORBA::SystemException& e) {
            TEST_INFO("Exception received: " 
                      << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            // event sending should always be succesfull
            CPPUNIT_ASSERT(false);
        }
        
        TEST_INFO("Test calling facet and test if event was received");
        try {
            bool event_received = m_event_checking->was_event_received();
            if (success_expected) {
                CPPUNIT_ASSERT(event_received);
            } else {
                // if success wasn't expected, SystemException should 
                // have been received
                CPPUNIT_ASSERT(false); 
            }
        } catch (const CORBA::SystemException& e) {
            TEST_INFO("Exception received: " 
                      << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            CPPUNIT_ASSERT(!success_expected);
        }

        TEST_INFO("Calls on Server component perfomed");
        if (! success_expected)
            TEST_INFO("exceptions were expected");
    }
    
    
    
    void do_tests()
    {
        CORBA::Object_var obj1 = Cdmw::TestUtils::CORBATestManager::instance()->get_object("client_context");
        m_context = ::ReconnectDemo::CCM_Client_Context::_narrow(obj1.in());
        
        try {
            m_supported = m_context->get_connection_to_supported();
            CPPUNIT_ASSERT(!CORBA::is_nil(m_supported.in()));
        } catch (const CORBA::SystemException& e) {
            TEST_INFO("Exception received: " 
                      << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            CPPUNIT_ASSERT(false);
        }
        
        try {
            m_event_checking = m_context->get_connection_to_facet();
            CPPUNIT_ASSERT(!CORBA::is_nil(m_event_checking.in()));
        } catch (const CORBA::SystemException& e) {
            TEST_INFO("Exception received: " 
                      << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            CPPUNIT_ASSERT(false);
        }
        
        // perform test invocations. 
        // Server is present, success is expected
        perform_test(true);
        
        
        char enter;
        std::cout << "----------------------------------------" << std::endl;
        std::cout << "Terminate the server and press any key and enter ..." << std::endl;
        std::cin >> enter;

        
        // perform test invocations. 
        // Server is no more present, failure is expected
        perform_test(false);
        
        
        // wait for server restart
        std::cout << "----------------------------------------" << std::endl;
        std::cout << "Restart the server, rebuild server assembly and press any key and enter ..." << std::endl;
        std::cin >> enter;

        // perform test invocations. 
        // Server is present again, success is expected
        perform_test(true);

    };

    CPPUNIT_TEST_SUITE( test_ccm_client_activate );
    CPPUNIT_TEST( do_tests );
    CPPUNIT_TEST_SUITE_END();
}; // End of class test_ccm_client_activate
	
} // End of anonymous namespace



//
// IDL:thalesgroup.com/ReconnectDemo/CCM_Client:1.0
//
ReconnectDemo::CCM_Client_impl::CCM_Client_impl()
	: m_session_context(ReconnectDemo::CCM_Client_Context::_nil()),
      m_activationThread(0)
{
}

ReconnectDemo::CCM_Client_impl::~CCM_Client_impl()
{
    std::cout << "CCM_Client_impl::~CCM_Client_impl called!" << std::endl;
}


//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
ReconnectDemo::CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    std::cout << "Set_session_context called!" << std::endl;
   
    m_session_context = ::ReconnectDemo::CCM_Client_Context::_narrow(ctx); 
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
ReconnectDemo::CCM_Client_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    m_activationThread = new ClientActivationThread(this);
    m_activationThread->start();
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//

void
ReconnectDemo::CCM_Client_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
   std::cout << "Ccm_passivate called!" << std::endl;
   if (m_activationThread != 0)
       m_activationThread->join();
} 



//
// ccm_remove function : it destroys the component (call by the container)
void
ReconnectDemo::CCM_Client_impl::ccm_remove()
   throw(CORBA::SystemException)
{
   std::cout << "Ccm_remove called!" << std::endl;
}


