/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
 *
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */
package cdmw.ccmreconnect;

import com.acme.ReconnectDemo.*;



public class ClientActivationThread extends cdmw.testutils.Testable
    implements Runnable
{
    
    private ClientExecutorImpl client;
    private CCM_Client_Context context;
    private Supported supported;
    private EventChecking eventChecking;
    
    public ClientActivationThread(ClientExecutorImpl client)
    {
        this.name = "Test ccm_reconnect";
        this.out = System.out;
        this.client = client;
        this.context = null;
        this.supported = null;
        this.eventChecking = null;
    }
    
    
    public void run()
    {
        start();
    }
    
    
    public void doTests()
    {
        // set number of requested successfull tests
        setNbOfRequestedTestOK(11);

        context = client.getContext();
        
        try {
            supported = context.get_connection_to_supported();
            check(supported != null);
        } catch (org.omg.CORBA.SystemException e) {
            fail("Exception received: " + e);
            e.printStackTrace();
        }
        
        try {
            eventChecking = context.get_connection_to_facet();
            check(eventChecking != null);
        } catch (org.omg.CORBA.SystemException e) {
            fail("Exception received: " + e);
            e.printStackTrace();
        }
        
        java.io.BufferedReader inputReader = 
            new java.io.BufferedReader(
                new java.io.InputStreamReader(System.in));
        
        // perform test invocations. 
        // Server is present, success is expected
        performTest(true);
        
        
        println("----------------------------------------");
        println("Terminate the server and press any key and enter ...");
        println("");
        try {
            inputReader.readLine();
        } catch (java.io.IOException e) {
            e.printStackTrace();
        }

        
        // perform test invocations. 
        // Server is no more present, failure is expected
        performTest(false);
        
        
        // wait for server restart
        println("----------------------------------------");
        println("Restart the server, rebuild server assembly and press any key and enter ...");
        println("");
        try {
            inputReader.readLine();
        } catch (java.io.IOException e) {
            e.printStackTrace();
        }

        // perform test invocations. 
        // Server is present again, success is expected
        performTest(true);
        
    }
    
    
    private void performTest(boolean successExpected)
    {
        println("Perform calls on Server component");
        if (! successExpected)
            println("exceptions are expected");
        
        println("Test calling supported interface");
        try {
            supported.test_supported();
            check(successExpected);
        } catch (org.omg.CORBA.SystemException e) {
            println("Exception received: " + e);
            check(!successExpected);
        }
        
        println("Test sending event");
        try {
            TestEvent event = new TestEventImpl();
            event.name = "TEST";
            context.push_event_publish(event);
            // event sending should always be succesfull
            succeed();
        } catch (org.omg.CORBA.SystemException e) {
            // event sending should always be succesfull
            fail("Exception received: " + e);
        }
        
        println("Test calling facet and test if event was received");
        try {
            boolean eventReceived = eventChecking.was_event_received();
            if (successExpected) {
                check(eventReceived);
            } else {
                // if success wasn't expected, SystemException should 
                // have been received
                fail(); 
            }
        } catch (org.omg.CORBA.SystemException e) {
            println("Exception received: " + e);
            check(!successExpected);
        }

        println("Calls on Server component perfomed");
        if (! successExpected)
            println("exceptions were expected");
    }
    
    
    
}


