#!/bin/sh
# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES 
# 2000-2003. All rights reserved.
# Copyright (C) THALES 2004-2005. All rights reserved.
# Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

if [ "${DISPLAY}" = ":0.0" ]; then
export DISPLAY=`hostname`:0.0
fi
if [ -z "${DISPLAY}" ]; then
export DISPLAY=`hostname`:0.0
fi

export SCRIPTNAME=`basename ${0}`

HOST1=${HOST1:-@hostname1val@}
HOST2=${HOST2:-@hostname2val@}
HOST3=${HOST3:-@hostname3val@}
HOST4=${HOST4:-@hostname4val@}
HOST5=${HOST5:-@hostname5val@}

# horizontal size of xterm (nb of columns)
HORIZONTAL_SIZE=81
# vertical size of xterm (nb of lignes)
VERTICAL_SIZE=8
# vertical space between xterm (pixels ?)
VERTICAL_SPACE=130



#------------------------------------------------------------------------------#
#                                print_usage                                   #
#------------------------------------------------------------------------------#
# display the usage of this script
#------------------------------------------------------------------------------#
print_usage()
{
    echo " "
    echo "usage: ${SCRIPTNAME} [--help] <user_login>"
    echo " "
    echo "    <user_login>         : login to be used for xterms"
    echo "    --help               : display this help"
    echo " "
    exit 0
}


print_root_error()
{
    echo " "
    echo "Error: you should be root to run $SCRIPTNAME !"
    exit -1
}

print_user_error()
{
    echo " "
    echo "Error: user $1 is inexistant !"
    exit -1
}

#################
parse_arguments()
{
    # test is this script is started as root
    if [ `whoami` != root ]; then
        print_root_error
    fi

    #parse argument 1
    USERLOGIN=${1}
    if [ "${USERLOGIN}" == "" ]; then
        print_usage
    fi
    if [ "${USERLOGIN}" == "--help" ]; then
        print_usage
    fi
    TEST_USER=`id ${USERLOGIN} 2>/dev/null`
    if [ "${TEST_USER}" = "" ]; then 
        print_user_error ${USERLOGIN}
    fi
    
    QUALIF_PATH=@config.install.dir@
}

# find xterm
if [ -f /usr/bin/xterm ]; then
    TERM=/usr/bin/xterm
elif [ -f /usr/X11R6/bin/xterm ]; then 
    TERM=/usr/X11R6/bin/xterm
elif [ -f USER/usr/X/bin/xterm ]; then
    TERM=/usr/X/bin/xterm
else
    TERM=xterm
fi

# TERM COMMAND
export TERM="${TERM} +ls -sb -sl 10000 -display ${DISPLAY}"

#################################
# start a xterm as specified user
#   arg1 = user login
#   arg2 = hostname
#   arg3 = xterm title
#   arg4 = xterm geometry
#   following args = commands to be executed on host before xterm start
#################################
start_xterm_as_user ()
{
    USER=${1}
    HOST=${2}
    TITLE=${3}
    GEO=${4}
    shift; shift; shift; shift
    
    # set xterm color according to host
    case ${HOST} in
          ${HOST1})
              # light grey
              XTERM_COLOR=rgb:ea/ea/ea ;;
          ${HOST2})
              # yellow
              XTERM_COLOR=rgb:ff/ff/cd ;;
          ${HOST3})
              # purple
              XTERM_COLOR=rgb:e2/d9/ff ;;
          ${HOST4})
              # orange
              XTERM_COLOR=rgb:ff/f0/b7 ;;
          ${HOST5})
              # green
              XTERM_COLOR=rgb:e6/ff/e6 ;;
          *)
              # white
              XTERM_COLOR=rgb:ff/ff/ff ;;
    esac
    case ${USER} in
        root)
          TERM_PRE_COMMAND="cd ${QUALIF_PATH}/bin; ./set_root.sh; . ./set_env.sh; cd ../share; export HOST1=${HOST1};export HOST2=${HOST2};export HOST3=${HOST3};export HOST4=${HOST4};export HOST5=${HOST5}"
        ;;
        *)
          TERM_PRE_COMMAND="cd ${QUALIF_PATH}/bin; . ./set_env.sh; cd ../share; export HOST1=${HOST1};export HOST2=${HOST2};export HOST3=${HOST3};export HOST4=${HOST4};export HOST5=${HOST5}"
        ;;
    esac
        
    echo "${CSCI}_${TITLE}_on_${HOST}@${USER}"
    rsh ${HOST} " ( su -c '${TERM_PRE_COMMAND}; unset PROMPT_COMMAND; export PS1=${CSCI}_${TITLE}_on_${HOST}@${USER}\>\ ; ${TERM} -bg ${XTERM_COLOR} -geometry ${GEO} -title ${TITLE} -l' ${USER} ) " &
    sleep 2
}

##########
###MAIN###
##########
# parse arguments
parse_arguments ${*}

#TERM01
TERM_HOST=${HOST1}
TERM_USER=${USERLOGIN}
TERM_TITLE="01___Platform_Admin"
TERM_GEO="${HORIZONTAL_SIZE}x${VERTICAL_SIZE}+0+0"
start_xterm_as_user ${TERM_USER} ${TERM_HOST} ${TERM_TITLE} ${TERM_GEO}

#TERM02
TERM_HOST=${HOST1}
TERM_USER=${USERLOGIN}
TERM_TITLE="02___Platform_Observer"
TERM_GEO="${HORIZONTAL_SIZE}x${VERTICAL_SIZE}-0+0"
start_xterm_as_user ${TERM_USER} ${TERM_HOST} ${TERM_TITLE} ${TERM_GEO}

#TERM03
TERM_HOST=${HOST2}
TERM_USER=root
TERM_TITLE="03___Test_results_HOST_2"
TERM_GEO="${HORIZONTAL_SIZE}x${VERTICAL_SIZE}-0+${VERTICAL_SPACE}"
start_xterm_as_user ${TERM_USER} ${TERM_HOST} ${TERM_TITLE} ${TERM_GEO}

#TERM04
TERM_HOST=${HOST1}
TERM_USER=root
TERM_TITLE="04___Test_results_HOST_1"
TERM_GEO="${HORIZONTAL_SIZE}x${VERTICAL_SIZE}+0+${VERTICAL_SPACE}"
start_xterm_as_user ${TERM_USER} ${TERM_HOST} ${TERM_TITLE} ${TERM_GEO}

#TERM05
TERM_HOST=${HOST1}
TERM_USER=${USERLOGIN}
TERM_TITLE="05___Platform_Supervision"
TERM_GEO="${HORIZONTAL_SIZE}x${VERTICAL_SIZE}+0+$((VERTICAL_SPACE * 2))"
start_xterm_as_user ${TERM_USER} ${TERM_HOST} ${TERM_TITLE} ${TERM_GEO}

#TERM06
TERM_HOST=${HOST2}
TERM_USER=${USERLOGIN}
TERM_TITLE="06___Deployment_Tool"
TERM_GEO="${HORIZONTAL_SIZE}x${VERTICAL_SIZE}-0+$((VERTICAL_SPACE * 2))"
start_xterm_as_user ${TERM_USER} ${TERM_HOST} ${TERM_TITLE} ${TERM_GEO}

#TERM07
TERM_HOST=${HOST3}
TERM_USER=root
TERM_TITLE="07___Test_results_HOST_3"
TERM_GEO="${HORIZONTAL_SIZE}x${VERTICAL_SIZE}-0+$((VERTICAL_SPACE * 3))"
start_xterm_as_user ${TERM_USER} ${TERM_HOST} ${TERM_TITLE} ${TERM_GEO}

#TERM08
TERM_HOST=${HOST1}
TERM_USER=${USERLOGIN}
TERM_TITLE="08___Primary_FT_Manager"
TERM_GEO="${HORIZONTAL_SIZE}x${VERTICAL_SIZE}+0+$((VERTICAL_SPACE * 4))"
start_xterm_as_user ${TERM_USER} ${TERM_HOST} ${TERM_TITLE} ${TERM_GEO}

#TERM09
TERM_HOST=${HOST2}
TERM_USER=${USERLOGIN}
TERM_TITLE="09___Backup_FT_Manager"
TERM_GEO="${HORIZONTAL_SIZE}x${VERTICAL_SIZE}-0+$((VERTICAL_SPACE * 4))"
start_xterm_as_user ${TERM_USER} ${TERM_HOST} ${TERM_TITLE} ${TERM_GEO}

#TERM10
TERM_HOST=${HOST4}
TERM_USER=root
TERM_TITLE="10___Test_results_HOST_4"
TERM_GEO="${HORIZONTAL_SIZE}x${VERTICAL_SIZE}+0+$((VERTICAL_SPACE * 5))"
start_xterm_as_user ${TERM_USER} ${TERM_HOST} ${TERM_TITLE} ${TERM_GEO}

#TERM11
TERM_HOST=${HOST5}
TERM_USER=root
TERM_TITLE="11___Test_results_HOST_5"
TERM_GEO="${HORIZONTAL_SIZE}x${VERTICAL_SIZE}-0+$((VERTICAL_SPACE * 5))"
start_xterm_as_user ${TERM_USER} ${TERM_HOST} ${TERM_TITLE} ${TERM_GEO}
