#!/bin/sh
# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES 
# 2000-2003. All rights reserved.
# Copyright (C) THALES 2004-2005. All rights reserved.
# Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

# ===================================== #
#  @name R_analisys.sh
#  @brief execute data analisys with R.
# ===================================== #
#  @author Lello Mele <rmele@progesi.it>
# ===================================== #

function echoc(){
  local str=$*
  if [ ! "$str" == "" ]; then
    echo -e "\e[32;40m$str\e[0m"
  fi
}

function echos(){
  local str=$*
  if test "`echo -e xxx`" = "xxx"
  then
      echoc $str
  else
      echo $str
  fi
}


## this shell script
SCRIPTNAME=`basename $0`
if [ "$#" -lt 3 ] || [ ! -f $R_FILENAME ]; then
  cat >&2 <<EOF
* Usage:
  $SCRIPTNAME <R_script_name[.R]> <trigger_file> <time_step_10s>
* where:
  R_script_name[.R]: The name of R file.
  trigger_file.....: File to waiting before R execution.
  time_step_10s....: Waiting step (1 step = 10 seconds) before timeout.
EOF
  exit 1
fi
## Qualification tests name
TESTNAME=`basename $1 .R`
## trigger file
EOF_FILE=$2
## number of step
EXEC_STEP=$3
EXEC_TIMEOUT=10
## R Script file name
R_FILENAME=$TESTNAME.R
## default R options
R_ARGUMENT="--slave --no-save"
## R executable location (see SCOM for installation)
R_EXEC=`which R`
## test R path 
if [ ! -x $R_EXEC ]; then
    echos "ERROR:"
    echo "$R_EXEC path not found!"
    exit 1
fi
## test argument
if [ -f $EOF_FILE ]; then
    echos "WARNING NOTE:"
    echo  "$EOF_FILE exist!"
    echos "-------------"
fi
## waiting for a file or a timeout
istep=1
while [ ! -f "$EOF_FILE" ] && [ $istep -le $EXEC_STEP ];
do
  if [ $istep == 1 ]; then 
    echos "Waiting a while (timeout at $(($EXEC_STEP*$EXEC_TIMEOUT)) sec) for the execution of the scenario..."
  fi
  echos "Waiting step $istep of $EXEC_STEP ..."  
  sleep $EXEC_TIMEOUT;
  istep=$(($istep+1))
done
## test executions
if [ ! -f "$EOF_FILE" ] && [ $istep -ge $EXEC_STEP ]; then
  cat >&2 <<EOF
##############################
$SCRIPTNAME: TIMEOUT ERROR!
##############################
EOF
  exit 1
else
  ## execute R with argument
  $R_EXEC $R_ARGUMENT < $R_FILENAME
  echos "Data analisys and the execution scenario are terminated."
  echos "Hit a key when ready."
  ## move file in results directory
  TESTCPDIR=$TESTNAME-1-results
  mkdir -p $TESTCPDIR 2>/dev/null
  cp -f $TESTNAME\_*.dat $TESTCPDIR 2>/dev/null
  mv -f test_summary.result.$TESTNAME $TESTCPDIR/test_summary.result 2>/dev/null
  cp -f $R_FILENAME $TESTCPDIR 2>/dev/null
  exit 0
fi
