/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES 
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */


#include <SystemMngt/platforminterface/PlatformInterface.hpp>
#include "TraceAndPerf/tracelibrary/FlushAreaMngr.hpp"
#include "namingandrepository/FTNamingAndRepositoryBehaviour.hpp"
//#include "namingandrepository/TestNamingContext_impl.hpp"

#include <Foundation/ossupport/SystemProperties.hpp>
#include <Foundation/ossupport/Signals.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>

#include <Foundation/common/Options.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/common/Locations.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/logging/LogManager.hpp>


#include <FaultTolerance/ftinit/FTServiceInit.hpp>
#include <FaultTolerance/ftcommon/FTConfiguration.hpp>
#include <FaultTolerance/ftcommon/FTSystemProperties.hpp>
#include <ConfAndPlug/cdmwinit/CdmwInterface.hpp> 


#include <sstream>
#include <fstream>
#include <iostream>
#include <string>

namespace
{

    using namespace Cdmw::NamingAndRepository;
    using namespace Cdmw::OsSupport;
    using namespace Cdmw::OrbSupport;
    using namespace Cdmw::Trace;
    using namespace Cdmw::Common;


    const int ORB_THREAD_POOL_SIZE = 10;
    const int REP_ORB_THREADS_STACK_SIZE = 131072;
    const int REP_FLUSHAREAMNGR_THREAD_STACKSIZE = 262144;
    const int REP_FTSTATEMNGR_SCHED_THREADS_STACKSIZE = 131072;
    const int REP_FTSTATEMNGR_TIMER_THREAD_STACKSIZE = 131072;

    std::string CONFIG_OPTION = "--conf";
    std::string ID_OPTION = "--id";

    std::string NO_TRACE_COLLECTOR_OPTION = "--noTraceCollector";
    std::string TRACE_OPTION = "--trace";
    const char *TRACE_PRODUCER_DEFAULT_NAME = "RepositoryTraceProducer";

   // Default value for ORB connection timeout
   std::string CONNECTION_TIMEOUT_PROPERTY_VALUE="100"; // 100 ms

/* Part not used in UT, VT or QT : suppressed, as well as TestNamingContext class
   if there is a need for this class it can be recovered with CVS
    std::string THREADING_TEST_OPTION = "--threading-test";
    const char *THREADING_TEST_NAMING_CONTEXT = "ThreadingTestNamingContext";
    const char *THREADING_TEST_NAMING_CONTEXT_IOR = "/tmp/ThreadingTestNamingContext.IOR";
*/

};

#define COUT(mes) std::cout << "NamingAndRepository : " << mes << std::endl;
#define CERR(mes) std::cout << "NamingAndRepository : FAILURE : " << mes << std::endl;

void exceptionHandler(void)
{

	CERR("UNEXPECTED EXCEPTION HANDLER");

}


void assertionFailedHandler(void)
{

	CERR("ASSERTION FAILED HANDLER");
    ::exit(EXIT_FAILURE);

}


void atExitHandler(void)
{
	COUT("Exited");
}


std::string getSyntax()
{

    std::ostringstream syntax;

    syntax << std::endl << "Syntax is :" << std::endl;
    syntax << Cdmw::Common::Options::LOCALISATION_SERVICE_OPTION << "=Port";
    syntax << " " << ID_OPTION << "=PhysicalID";
    syntax << " " << CONFIG_OPTION << "=XMLConfigFile" << std::endl;
    syntax << " [" << Cdmw::Common::Options::VALIDATE_XML_OPTION << "]";
    syntax << " [" << NO_TRACE_COLLECTOR_OPTION << "]";
    syntax << " [" << TRACE_OPTION << "[=TraceProducerName] ]";
//    syntax << " [" << THREADING_TEST_OPTION << "]";
    syntax << " [" << Cdmw::Common::Options::ORB_THREADS << "=Number" << "]";
    syntax << " [" << Cdmw::Common::Options::ORB_THREADS_STACKSIZE << "=Size in KB" << "]";
    syntax << " [" << Cdmw::Common::Options::FTSTATEMNGR_SCHED_THREADS_STACKSIZE << "=Size in KB" << "]";
    syntax << " [" << Cdmw::Common::Options::FTSTATEMNGR_TIMER_THREAD_STACKSIZE << "=Size in KB" << "]";
    syntax << " [" << Cdmw::Common::Options::FLUSHAREAMNGR_THREAD_STACKSIZE << "=Size in KB" << "]";
    syntax << std::endl;

    return syntax.str();
}

std::string getAdditionalSyntax()
{

    std::ostringstream syntax;

    syntax << Cdmw::Common::Options::LOCALISATION_SERVICE_OPTION << " : The repository will use the specified port for the localisation service (corbaloc)" << std::endl;
//    syntax << THREADING_TEST_OPTION << " : The repository will create a special purpose naming context for threading tests" << std::endl;
    syntax << NO_TRACE_COLLECTOR_OPTION << " : Traces aren't forwarded to a trace collector." << std::endl;
    syntax << TRACE_OPTION << " : Traces are enabled. If no name is specified, the producer interface will be named \"" << TRACE_PRODUCER_DEFAULT_NAME << "\"" << std::endl;
    syntax << Cdmw::Common::Options::VALIDATE_XML_OPTION << " : The XML configuration file will be validated before processing." << std::endl;
    syntax << ID_OPTION << " : Specifies the mandatory physical identifier of the repository" << std::endl;
    syntax << CONFIG_OPTION << " : Specifies the mandatory XML configuration file" << std::endl << std::endl;
    syntax << std::endl;

    return syntax.str();
}

class InitialisationException : public Cdmw::Exception
{
    
    public:
        InitialisationException()
            : Exception( Exception::FATAL, "InitialisationException" )
        {
        }
};


/**
*/ 
void notifyPreInitialisationError(bool platformManaged, const char* message)
{

    if (platformManaged)
    {
        PlatformInterface::Notify_fatal_error("CDMW_NAMREP", message);
    }
    else
    {
        CERR(message);
    }
    
    CDMW_THROW(InitialisationException);

}


enum ExitType
{
    ORB_DESTROY,                // the ORB will be destroyed
    ORB_SHUTDOWN_AND_DESTROY    // the ORB will be shutdown and destroyed
};



int main(int argc, char* argv[])
{

#   ifndef _MSC_VER    

   std::set_unexpected(&exceptionHandler);

   // VC++ contrary to the clause 18.6 of the C++ standard, 
   //  set_unexpected, is not inside the std:: namespace
#   else

   set_unexpected(&exceptionHandler);

#   endif    

   Cdmw::set_assertion_failed(&assertionFailedHandler);

   ::atexit(&atExitHandler);

   try {
      // First initialise the CARDAMOM SystemProperties and the 
      // logging library is initialised.
      (void*) Cdmw::OsSupport::SystemProperties::Init(argc, argv);
      Cdmw::Logging::LogManager::Init(argc, argv);
      // From now on, CARDAMOM logging library is initialised.
   } catch (const std::exception & e ) {
      std::cerr << "Cannot initialise CARDAMOM SystemProperties or Logging library! \n"
                << "Caught exception : " << e.what() << std::endl;
      ::exit(EXIT_FAILURE);
   }

   long orb_thread_nb = ORB_THREAD_POOL_SIZE;

   // get Process Executable Name for trace processing
   std::string processName = argv[0];
	  	
   // suppress process absolute path
   std::string::iterator begin = processName.begin();
   int i = processName.rfind('/') + 1;
   std::string::iterator j = begin + i;
   processName.erase(begin,j);
	
   // no trace collector option
   std::string noTraceCollector = OS::get_option_value (argc, argv, NO_TRACE_COLLECTOR_OPTION);
   bool useTraceLibrary = true;
   if (noTraceCollector != "no") {
       useTraceLibrary = false;
   }
   
   // trace option
   std::string traceProducerName = OS::get_option_value (argc, argv, TRACE_OPTION);
    
   // get the trace producer name if specified or use the default one
   if (traceProducerName == "no")
   {
      // trace will be deactivated
      traceProducerName = "";
   }
   else if (traceProducerName == "yes")
   {
      // use the default name for the trace producer
      traceProducerName = TRACE_PRODUCER_DEFAULT_NAME;
   }

   // xml option
   bool validate = false;
   std::string validateXML_option =
      OS::get_option_value( argc, argv, Cdmw::Common::Options::VALIDATE_XML_OPTION);
   if ( validateXML_option == "yes" || validateXML_option != "no" )
   {
      validate = true;
   }

   // checks the orb threads option
   std::string orb_threads_option =
      OS::get_option_value(argc,
                           argv,
                           Cdmw::Common::Options::ORB_THREADS);

   if (orb_threads_option != "no")
   {
      if (orb_threads_option != "yes")
      {
         orb_thread_nb = ::atoi(orb_threads_option.c_str());

         if (orb_thread_nb <= 0)
         {
            throw CORBA::BAD_PARAM();
         }
      }
   }

   std::string orb_thread_stacksize_option =
      OS::get_option_value(argc, argv, Cdmw::Common::Options::ORB_THREADS_STACKSIZE);

   // Default value chosen for the orb thread stack size in the Repository
   std::size_t orbThreadStackSize(REP_ORB_THREADS_STACK_SIZE);

   if (orb_thread_stacksize_option != "no")
   {
      if (orb_thread_stacksize_option != "yes")
      {
         orbThreadStackSize = ::atoi(orb_thread_stacksize_option.c_str())*1024;
         if (orbThreadStackSize < 0)
         {
            std::cerr << "Bad orbThreadStackSize value" << std::endl;
            throw CORBA::BAD_PARAM();
         }
      }
      else
      {
         std::cerr << "Bad orbThreadStackSize option" << std::endl;
         throw CORBA::BAD_PARAM();
      }
   } // else ignore.

   SystemProperties::Set_Property(ORB_THREADS_STACKSIZE_PROPERTY_NAME, orbThreadStackSize);

   std::string ftstatemanager_sched_thread_stacksize_option =
      OS::get_option_value(argc, argv, Cdmw::Common::Options::FTSTATEMNGR_SCHED_THREADS_STACKSIZE);

   std::size_t ftstatemanagerSchedThreadStackSize(REP_FTSTATEMNGR_SCHED_THREADS_STACKSIZE);

   if (ftstatemanager_sched_thread_stacksize_option != "no")
   {
      if (ftstatemanager_sched_thread_stacksize_option != "yes")
      {
         ftstatemanagerSchedThreadStackSize = ::atoi(ftstatemanager_sched_thread_stacksize_option.c_str())*1024;
         if (ftstatemanagerSchedThreadStackSize < 0)
         {
            std::cerr << "Bad ftstatemanagerSchedThreadStackSize value" << std::endl;
            throw CORBA::BAD_PARAM();
         }
      }
      else
      {
         std::cerr << "Bad ftstatemanagerSchedThreadStackSize option" << std::endl;
         throw CORBA::BAD_PARAM();
      }
   } // else ignore

   SystemProperties::Set_Property(
      Cdmw::FT::FTSTATEMNGR_SCHED_THREADS_STACKSIZE_PROPERTY_NAME,
      ftstatemanagerSchedThreadStackSize );

   std::string ftstatemanager_timer_thread_stacksize_option =
      OS::get_option_value(argc, argv, Cdmw::Common::Options::FTSTATEMNGR_TIMER_THREAD_STACKSIZE);

   std::size_t ftstatemanagerTimerThreadStackSize(REP_FTSTATEMNGR_TIMER_THREAD_STACKSIZE);

   if (ftstatemanager_timer_thread_stacksize_option != "no")
   {
      if (ftstatemanager_timer_thread_stacksize_option != "yes")
      {
         ftstatemanagerTimerThreadStackSize = ::atoi(ftstatemanager_timer_thread_stacksize_option.c_str())*1024;
         if (ftstatemanagerTimerThreadStackSize < 0)
         {
            std::cerr << "Bad ftstatemanagerTimerThreadStackSize value" << std::endl;
            throw CORBA::BAD_PARAM();
         }
      }
      else
      {
         std::cerr << "Bad ftstatemanagerTimerThreadStackSize option" << std::endl;
         throw CORBA::BAD_PARAM();
      }
   } // else ignore

   SystemProperties::Set_Property(
      Cdmw::FT::FTSTATEMNGR_TIMER_THREAD_STACKSIZE_PROPERTY_NAME,
      ftstatemanagerTimerThreadStackSize );

   std::string flushareamngr_thread_stacksize_option =
      OS::get_option_value(argc, argv, Cdmw::Common::Options::FLUSHAREAMNGR_THREAD_STACKSIZE);

   std::size_t flushareamngrThreadStackSize(REP_FLUSHAREAMNGR_THREAD_STACKSIZE);

   if (flushareamngr_thread_stacksize_option != "no")
   {
      if (flushareamngr_thread_stacksize_option != "yes")
      {
         flushareamngrThreadStackSize = ::atoi(flushareamngr_thread_stacksize_option.c_str())*1024;
         if (flushareamngrThreadStackSize < 0)
         {
            std::cerr << "Bad flushareamngrThreadStackSize" << std::endl;
            throw CORBA::BAD_PARAM();
         }
      }
      else
      {
         std::cerr << "Bad flushareamngrThreadStackSize option" << std::endl;
         throw CORBA::BAD_PARAM();
      }
   } // else ignore

   SystemProperties::Set_Property(
      FLUSHAREAMNGR_THREAD_STACKSIZE_PROPERTY_NAME,
      flushareamngrThreadStackSize );

   // the return code 
   int ret_code = 0;

   // FIXME what about exception to be caught?
   Cdmw::CdmwInit::CDMW_preORBInit(argc, argv);

   // Initialize the ORB
   StrategyList orbStrategies;
   orbStrategies.add_OrbThreaded();
   orbStrategies.add_PoaThreadPool(orb_thread_nb);
   orbStrategies.add_multicast();


   CORBA::ORB_var orb = CORBA::ORB::_nil();
   ExitType exitType = ORB_SHUTDOWN_AND_DESTROY;

   try
   {
 
      // Allow specification of a value for the ORB connection timeout from
      // command-line arguments or a property file
      std::string dummy_prop = 
         Cdmw::OsSupport::SystemProperties::Get_Property(Cdmw::OrbSupport::CDMW_ORB_CONNECTION_TIMEOUT_PROPERTY_NAME);
      if (dummy_prop.empty())
         Cdmw::OsSupport::SystemProperties::Set_Property(Cdmw::OrbSupport::CDMW_ORB_CONNECTION_TIMEOUT_PROPERTY_NAME,
                                                         CONNECTION_TIMEOUT_PROPERTY_VALUE);
      orb = OrbSupport::ORB_init(argc, argv, orbStrategies);

      CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
      PortableServer::POA_var poa = PortableServer::POA::_narrow(obj.in());

      PortableServer::POAManager_var poaMgr = poa->the_POAManager();
      poaMgr->activate();

      bool platformManaged = 
         PlatformInterface::Is_launched_by_PlatformManagement(argc, argv);

      if (platformManaged)
      {
         // initialise the platform interface
         // PlatformInterface::Setup(orb.in(), argc, argv); // FIXME dont do this if started in FT mode, otherwise BadOrderException in generated code
      }

      // preinitialisation : command line check up

      if (argc <= 1 )
      {
         std::string message("Missing mandatory arguments.");
         message += getSyntax();
         notifyPreInitialisationError(platformManaged, message.c_str());
      }

      std::string port_option = OS::get_option_value( argc, argv,
                                                      Cdmw::Common::Options::LOCALISATION_SERVICE_OPTION);

      if ( port_option == "no" || port_option == "yes" )
      {
         std::string message("Port not specified.");
         message += getSyntax();
         notifyPreInitialisationError(platformManaged, message.c_str());
      }

      std::string physicalId = OS::get_option_value( argc, argv, ID_OPTION);
      if ( physicalId == "no" || physicalId == "yes" )
      {
         std::string message("Physical identifier not specified.");
         message += getSyntax();
         notifyPreInitialisationError(platformManaged, message.c_str());
      }

      std::string configurationFile = OS::get_option_value( argc, argv, CONFIG_OPTION);
      if ( configurationFile == "no" || configurationFile == "yes" )
      {
         std::string message("Configuration file not specified.");
         message += getSyntax();
         notifyPreInitialisationError(platformManaged, message.c_str());
      }

      if ( ! OS::file_exists(configurationFile) )
      {
         std::string message("Configuration file '");
         message += configurationFile;
         message += "' not found";
         notifyPreInitialisationError(platformManaged, message.c_str());
      }

      // creates the process behaviour
      std::auto_ptr<FTNamingAndRepositoryBehaviour> pProcessBehaviour(
         new FTNamingAndRepositoryBehaviour (orb.in(), platformManaged,
                                             port_option,
                                             physicalId,
                                             configurationFile, 
                                             validate, 
                                             processName,
                                             traceProducerName,
                                             useTraceLibrary));
	
      FTNamingAndRepositoryBehaviour * pProcessBehaviour_ptr =
         pProcessBehaviour.get();

      Cdmw::CdmwInit::CDMW_init(
         orb.in(), argc, argv, pProcessBehaviour_ptr );
/*	
  std::string threadingTest = 
  OS::get_option_value( argc, argv, THREADING_TEST_OPTION);
  if ( threadingTest == "yes" )
  {
		
  // Create a naming context for multithreading access tests
  CosNaming::NamingContextExt_var namingContext = TestNamingContext_impl::create(THREADING_TEST_NAMING_CONTEXT);
		
  // save this naming context's IOR in a file
  CORBA::String_var namingContextIOR = orb->object_to_string(namingContext.in());
		
  std::ofstream namingContextIORFile(THREADING_TEST_NAMING_CONTEXT_IOR);
  namingContextIORFile << namingContextIOR.in();
  namingContextIORFile.close();
		
  COUT("IOR of Naming Context for threading test in " << THREADING_TEST_NAMING_CONTEXT_IOR << " file");		
  }
*/

      // Setup signal handling for ORB to be shut down
      Cdmw::OrbSupport::OrbSupport::install_orb_shutdown_signal_handlers(orb.in());

      // Enter in the main loop
      orb->run();

      Cdmw::OrbSupport::OrbSupport::uninstall_orb_shutdown_signal_handlers(orb.in());

      // FIXME get rid of this useless variable
      exitType = ORB_DESTROY;

//         if (platformManaged)
//         {
// 	    ASSERT(!"FIXME - gone here");

//             // acknowledge the creation of the process
//             PlatformInterface::Acknowledge_creation(pProcessBehaviour.get());
//             pProcessBehaviour.release();

//             orb->run();
//             COUT("The ORB has exited its main loop");

//             //orb->shutdown() is done by the ProcessBehaviour
//             exitType = ORB_DESTROY;
//         }
//         else
//         {	   
// 	    ASSERT(!"FIXME - gone here");

//             CdmwPlatformMngtBase::StartupKind startupKind;
//             startupKind.state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
//             startupKind.startup_mode = 0;
//             startupKind.specified_state_id = CORBA::string_dup("");

//             (pProcessBehaviour.get())->initialise(startupKind);


//             // create a threading test special purpose naming context if required
//             std::string threadingTest = OS::get_option_value( argc, argv, THREADING_TEST_OPTION);
//             if ( threadingTest == "yes" )
//             {

//                 // Create a naming context for multithreading access tests
//                 CosNaming::NamingContextExt_var namingContext = TestNamingContext_impl::create(THREADING_TEST_NAMING_CONTEXT);

//                 // save this naming context's IOR in a file
//                 CORBA::String_var namingContextIOR = orb->object_to_string(namingContext.in());

//                 std::ofstream namingContextIORFile(THREADING_TEST_NAMING_CONTEXT_IOR);
//                 namingContextIORFile << namingContextIOR.in();
//                 namingContextIORFile.close();

//                 COUT("IOR of Naming Context for threading test in " << THREADING_TEST_NAMING_CONTEXT_IOR << " file");

//             }

//             (pProcessBehaviour.get())->run();

//             orb->run();
//             // the repository can only be killed

//             exitType = ORB_DESTROY;

//         }
   }
   catch(const InitialisationException &e)
   {
      // already logged
      ret_code = EXIT_FAILURE;
   }
   catch(const Cdmw::Exception &e)
   {
      CERR(e.what());
      ret_code = EXIT_FAILURE;
   }
   catch(const CORBA::Exception &e)
   {
      CERR("unexpected " << e._name() << " CORBA exception");
      ret_code = EXIT_FAILURE;
   }
   catch(...)
   {
      CERR("unexpected exception");
      ret_code = EXIT_FAILURE;
   }
    

   if(!CORBA::is_nil(orb.in()))
   {
      try {
         Cdmw::CdmwInit::CDMW_cleanup(orb.in());
         Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
         orb->destroy();
      }
      catch(const CORBA::Exception& ex)
      {   
         return EXIT_FAILURE;
      }
   }       

   return ret_code;
}


