/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */

#include "Foundation/common/Options.hpp"
#include <Foundation/common/Locations.hpp>
#include "Foundation/common/String.hpp"
#include "Foundation/common/Exception.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/common/Locations.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include "tao/AnyTypeCode/TypeCode.h"
#include "platformapplicationlibrary/Process_impl.hpp"
#include "platformapplicationlibrary/ManagedProcess_impl.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtDaemon.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtEvent.stub.hpp"
#include "SystemMngt/platforminterface/PlatformInterface.hpp"
#include "SystemMngt/platforminterface/ServiceNames.hpp"
#include "SystemMngt/platformlibrary/Configuration.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "Foundation/logging/Logger.hpp"
#include "SystemMngt/platformlibrary/Configuration.hpp"
#include "SystemMngt/platformlibrary/EventHeaderFactory.hpp"
#include "SystemMngt/platformlibrary/LogMngr.hpp"
#include "SystemMngt/platformlibrary/SupervisionEventNotifier.hpp"
#include "SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp"
#include "SystemMngt/platformapplicationlibrary/Application_impl.hpp"
#include "SystemMngt/platformapplicationlibrary/ApplicationLogMessageId.hpp"
#include "SystemMngt/platformlibrary/EventNotifierManager.hpp"

//#include "SystemMngt/platformsystemlibrary/Host_impl.hpp"

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include "Repository/repositoryinterface/RepositoryInterface.hpp"
#include <set>
#include <string>
#include <iostream>
#include <sstream>
#include <map>
#include <math.h>
#include <ctype.h>
#include <bitset>

using namespace Cdmw::OsSupport;
using namespace Cdmw::OrbSupport;
using namespace Cdmw::PlatformMngt;
using namespace Cdmw;
using namespace Common; 
using namespace PlatformMngt;
using namespace CdmwPlatformMngt;

namespace Cdmw
{
  namespace PlatformMngt
  {

    Process_impl::Process_impl (PortableServer::POA_ptr poa,
				CdmwPlatformMngt::ProcessDef * process_def,
				const char *process_key,
				const char *process_path,
				Application_impl * application,
				CdmwPlatformMngt::
				RWElementRepository_ptr
				element_repository):DeactivableServant_impl
      (poa), ManagedElement_impl (element_repository, poa), m_current_restart_attempt(0)
    {
      if (application == NULL)
        CDMW_THROW2 (BadParameterException, "application", "NULL");

      m_procEntityStatusChangeFactory = NULL;
      m_procStatusChangeFactory = NULL;
      
      // set in Managed Element the instance of Property Event Notifier Thread
      set_property_event_notifier(EventNotifierManager::Instance()->get_property_notifier());
      // set in Managed Element the instance of Status Event Notifier Thread
      set_status_event_notifier(EventNotifierManager::Instance()->get_status_notifier());
  
      try
      {
	// Sets the process's internal data
	std::auto_ptr < ProcessEntityStatusChangeFactory >
	  procEntityStatusChangeFactory (new
					 ProcessEntityStatusChangeFactory ());
	std::auto_ptr < ProcessStatusChangeFactory >
	  procStatusChangeFactory (new ProcessStatusChangeFactory ());

	m_procEntityStatusChangeFactory =
	  procEntityStatusChangeFactory.get ();
	m_procStatusChangeFactory = procStatusChangeFactory.get ();

	m_application = application;
	m_type = process_def->type ();
	m_autoending = process_def->auto_ending ();
        set_requested_restart_warmth ( process_def->default_restart_warmth() );
        set_force_manual_start(process_def->force_manual_start());

        m_last_exit_info.exit_event=CdmwPlatformMngt::UNKNOWN;
        m_last_exit_info.event_value=0;
        m_last_exit_info.core_dumped=CdmwPlatformMngt::CORE_UNKNOWN;
        m_exit_autorestart_info_var = new CdmwPlatformMngt::ExitAutoRestartInfo (process_def->exit_autorestart_info());

	m_process_info = new CdmwPlatformMngt::ProcessInfo ();
	m_process_info->process_exe = CORBA::string_dup ("");
	m_process_info->process_port = 0;
	m_process_info->working_directory = CORBA::string_dup ("");
	m_process_info->process_args = CORBA::string_dup ("");

	m_lifeCycle = new CdmwPlatformMngt::LifeCycleDef ();
	m_lifeCycle->init_steps.length (0);
	m_lifeCycle->run_timeout = 0;
	m_lifeCycle->stop_timeout = 0;

	m_monitoring_info = new CdmwPlatformMngt::CompleteMonitoringInfo ();
	m_monitoring_info->monitoring_model = CdmwPlatformMngt::NO_MONITORING;
	m_monitoring_info->pull_monitoring_interval = 0;
	m_monitoring_info->pull_monitoring_timeout = 0;
	m_monitoring_info->push_monitoring_interval = 0;
	m_monitoring_info->push_monitoring_timeout = 0;

	m_status_info = CORBA::string_dup ("");
	m_stepCounter = 0;

	m_host = CdmwPlatformMngt::Host::_nil ();
	// store element path in managed element
	set_element_path (process_path, process_def->name ());

	// memo the different names : process, host and process key
	m_proc_syst_name =
	  CORBA::string_dup (m_application->get_system_name ());
	m_proc_appli_name =
	  CORBA::string_dup (m_application->get_element_name ());
	m_proc_host_name = CORBA::string_dup (process_def->host_name ());
	m_proc_key = CORBA::string_dup (process_key);
	
	m_startup_kind = new CdmwPlatformMngtBase::StartupKind ();

        m_created_dynamically = true;
	// Release the ownership of the temporary objects
	procEntityStatusChangeFactory.release ();
	procStatusChangeFactory.release ();
      }
      catch (const std::bad_alloc &)
      {
	CDMW_THROW (OutOfMemoryException);
      }

      try
      {
	Cdmw::CommonSvcs::DataStore::DataStoreBase * ds =
	  Cdmw::CommonSvcs::DataStore::DataStoreRegistry::
	  Get_data_store (Configuration::Get_process_datastore_id ());
	m_process_ds =
	  dynamic_cast < Cdmw::PlatformMngt::ProcessDataStore * >(ds);
      }
      catch (const NotExistException & e)
      {
	CDMW_ASSERT (false);
      }
    }


    Process_impl::~Process_impl ()
    {
      if (m_procEntityStatusChangeFactory != NULL)
	{
	  m_procEntityStatusChangeFactory->_remove_ref ();
	}

      if (m_procStatusChangeFactory != NULL)
	{
	  m_procStatusChangeFactory->_remove_ref ();
	}
        }
        
        void Process_impl::remove()
        {
            // first increment counter reference to avoid the destructor calling
            // by servant deactivation
            this->_add_ref();
            
            // deactivate the Corba object
            this->deactivate();

            // remove associated process record in datastore
            remove_record_in_datastore();

            // remove the associated managed element
            this->ManagedElement_impl::remove();
            
            // then decrement the counter reference to call destructor
            this->_remove_ref();        
    }




    void Process_impl::set_record_in_datastore ()
    {
      // test if the supervision is not fault tolerant or if it is, is it the primary?
      if (!Configuration::Is_fault_tolerant ()
	  || Configuration::Is_primary ())
	{
	  if (Configuration::Is_synchronising ())
	    {
	      std::
		cout << "<<Process_impl::set_record_in_datastore() start >> "
		<< m_element_name.in () << std::endl;
	    }
	  CdmwPlatformMngt::ProcessRecord record;
	  record.host_name = m_proc_host_name;
	  record.type = m_type;
	  record.auto_ending = m_autoending;
          record.actual_manual_starting = m_actual_manual_starting;
          record.force_manual_start = m_force_manual_start;

	  record.process_info = m_process_info.in ();
	  record.life_cycle = m_lifeCycle.in ();
	  record.monitoring_info = m_monitoring_info.in ();
	  record.status = this->getStatus ();
	  record.status_info = m_status_info;
	  record.state_machine =
	    CORBA::string_dup (this->get_state_machine_state_name ());
	  record.step_counter = m_stepCounter;
	  record.last_request_status = m_lastRequestStatus;
	  record.last_error_header = m_lastErrorHeader;
	  record.last_error_issuer =
	    CORBA::string_dup (m_lastErrorIssuer.c_str ());
          record.last_exit_code = m_last_exit_info;
          record.current_restart_attempt = m_current_restart_attempt;
	  record.startup_kind = m_startup_kind.in ();
	  

          //fill the filters_ds field of ProcessRecord 

          //see if there is the filters map filled
	  //if yes then fill this field filters_ds
          CORBA::String_var tr_filters;
	  char *test_point;
          test_point=get_trace_filtering();

          if ( test_point == NULL)
	    {
             //in set..datastore test_point is NULL!!
	     //do nothing
	    }
          else
	    {
	    tr_filters =  test_point;

            record.filters_ds = CORBA::string_dup(tr_filters)  ;
	    }


	  if (m_process_ds->contains (m_element_path.in ()))
	    {
	      m_process_ds->update (m_element_path.in (), record);
	    }
	  else
	    {
	      set_element_in_datastore ();
	      m_process_ds->insert (m_element_path.in (), record);
	    }

	  if (Configuration::Is_synchronising ())
	    {
	      std::
		cout << "<<Process_impl::set_record_in_datastore() done >> "
		<< m_element_name.in () << std::endl;
	    }
	}

      m_status_in_ds = this->getStatus ();
    }


    void Process_impl::remove_record_in_datastore ()
    {
      // inhibition if backup
      if (!Configuration::Is_fault_tolerant ()
	  || Configuration::Is_primary ())
	{
	  try
	  {
	    m_process_ds->remove (m_element_path.in ());
	  }
	  catch (Common::NotFoundException &)
	  {
	  }
	}
    }

    CdmwPlatformMngt::ProcessRecord *
      Process_impl::get_record_from_datastore ()
    {
      CdmwPlatformMngt::ProcessRecord * process_record = NULL;

      try
      {
	process_record = m_process_ds->select (m_element_path.in ());
      }
      catch (Common::NotFoundException &)
      {
         CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                    "process record cannot be found in datastore : "
                    << m_element_path.in());
         CDMW_ASSERT (false);
      }

      return process_record;
    }


    void Process_impl::synchro_with_datastore ()
    {
      Cdmw::PlatformMngt::ManagedElement_impl::synchro_with_datastore ();
      CdmwPlatformMngt::ProcessRecord_var process_record =
	get_record_from_datastore ();

      m_autoending = process_record->auto_ending;
      m_actual_manual_starting = process_record->actual_manual_starting;
      m_force_manual_start = process_record->force_manual_start;
      m_process_info =
	new CdmwPlatformMngt::ProcessInfo (process_record->process_info);
      m_lifeCycle =
	new CdmwPlatformMngt::LifeCycleDef (process_record->life_cycle);
      m_monitoring_info =
	new CdmwPlatformMngt::CompleteMonitoringInfo (process_record->
						      monitoring_info);
      m_status_info = process_record->status_info;
      m_stepCounter = process_record->step_counter;
      m_lastRequestStatus = process_record->last_request_status;
      m_lastErrorHeader = process_record->last_error_header;
      m_lastErrorIssuer = (process_record->last_error_issuer).in ();
      m_startup_kind = new CdmwPlatformMngtBase::StartupKind (process_record->startup_kind);

      m_status_in_ds = process_record->status;
      char * test_poin = CORBA::string_dup( process_record->filters_ds );

      std::string test_str(test_poin);

      if ( test_str.compare("") == 0 ) 
        {
	//do nothing string is empty
	}
      else
        {

        //read the filters_ds field and update the filters map
        set_trace_filtering ( process_record->filters_ds ) ;
	}

      m_last_exit_info = process_record->last_exit_code;
      m_current_restart_attempt = process_record->current_restart_attempt;
      this->set_state_machine_state_name (process_record->state_machine.
					  in ());
    }

    void Process_impl::synchro_processing ()
    {
    }


    void Process_impl::monitoringFailure_event (const CdmwPlatformMngtBase::
						EventHeader & header)
      throw (CdmwPlatformMngt::IncompatibleType, CORBA::SystemException)
    {
      throw CdmwPlatformMngt::IncompatibleType ();
    }


    /*
     *Purpose:
     *<p> it returns the string of trace filters ;
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_status:1.0
     * operation
     */
    char * Process_impl::get_status() throw(CORBA::SystemException) 
      {

        return NULL;
      }


    CdmwPlatformMngt::ProcessStatus Process_impl::get_status_in_ds ()
    {
      // return status stored in datastore         
      return m_status_in_ds;
    }

    CdmwPlatformMngtBase::StartupKind* Process_impl::get_startup_kind_from_ds ()
    {
      return m_startup_kind;
    }

    CdmwPlatformMngt::ProcessType Process_impl::type ()throw (CORBA::
							      SystemException)
    {
      return m_type;
    }


    CORBA::Object_ptr Process_impl::embedded_service ()throw (CORBA::
							      SystemException)
    {
      return CORBA::Object::_nil ();
    }


    CdmwPlatformMngt::ProcessPID Process_impl::pid ()throw (CORBA::
							    SystemException)
    {
      // TODO
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
				 CORBA::COMPLETED_NO);
    }


    CORBA::Boolean Process_impl::created_dynamically ()throw (CORBA::
							      SystemException)
    { 
      return m_created_dynamically;
    }

       
    CORBA::Long Process_impl::restart_attempt ()throw (CORBA::SystemException)
    {
        return m_current_restart_attempt;
    }
    
    void Process_impl::set_restart_attempt (CORBA::Long nrestart)
              throw(CORBA::SystemException)
    {
        m_current_restart_attempt = nrestart;
    }


    bool Process_impl::get_run_as_root () throw (CORBA::SystemException)
    {
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
				 CORBA::COMPLETED_NO);
    }

    CdmwPlatformMngtBase::TimeStamp Process_impl::start_time ()throw (CORBA::
								      SystemException)
    {
      // TODO
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
				 CORBA::COMPLETED_NO);
    }

    CdmwPlatformMngtBase::TimeStamp Process_impl::stop_time ()throw (CORBA::
								     SystemException)
    {
      // TODO
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
				 CORBA::COMPLETED_NO);
    }


	/**
	 *Purpose:
	 *<p> facade to get an attribute
	 *
	 */

    void Process_impl::get_attribute (const char *attr_name,
				      CORBA::Any_out ret_attr,
				      CORBA::Char flag) throw (CORBA::
							       SystemException,
							       CdmwPlatformMngt::
							       AttributeNotFound)
    {
      char flag_found = 0;
      std::string tmp1 (attr_name);
      if (flag == 0)
	{			/* case od std attributes */
	  /*active the right methods */
	  if (tmp1 == "status")
	    {
	      flag_found = 1;
	      CdmwPlatformMngt::ProcessStatus stat;
	      CORBA::String_var status_info;
	      CORBA::Any_var tempany = new CORBA::Any;

	      stat = get_status (status_info);
	      tempany.inout () <<= stat;
	      ret_attr = tempany._retn ();
	    }

	  if (tmp1 == "status_info")
	    {
	      flag_found = 1;
	      CdmwPlatformMngt::ProcessStatus stat;
	      CORBA::String_var status_info;
	      CORBA::Any_var tempany = new CORBA::Any;

	      stat = get_status (status_info);
	      tempany.inout () <<= status_info;

	      ret_attr = tempany._retn ();
	    }


	  if (flag_found == 0)
	    throw CdmwPlatformMngt::AttributeNotFound ();
	}
      else
	{
	  /*case of user defined attribute */
	  try
	  {
	    CORBA::Any * outany = get_property_value (attr_name);

	    CORBA::Any_var tempany = new CORBA::Any;

	    ret_attr = outany;
	  }
	  catch (CosPropertyService::PropertyNotFound)
	  {
	    throw CdmwPlatformMngt::AttributeNotFound ();
	  }
	}

    }



	/**
	 *Purpose:
	 *<p> facade to set an attribute
	 *
	 */

    void Process_impl::set_attribute (const char *attr_name,
				      const CORBA::Any & attr,
				      CORBA::Char flag) throw (CORBA::
							       SystemException,
							       CdmwPlatformMngt::
							       AttributeNotFound)
    {
      std::string tmp1 (attr_name);
      if (flag == 0)
	{			/* case od std attributes */
	  /*no attributes at the moment */
	  throw CdmwPlatformMngt::AttributeNotFound ();
	}
      else
	{
	  /* case of user defined attributes */
	  define_property (attr_name, attr);
	}

    }


    CdmwPlatformMngt::ProcessMode Process_impl::
      get_process_mode ()throw (CORBA::SystemException)
    {
      // TODO
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
				 CORBA::COMPLETED_NO);
    }

    bool Process_impl::get_memory_resident () throw (CORBA::SystemException)
    {
      // TODO
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
				 CORBA::COMPLETED_NO);
    }

    void Process_impl::set_memory_resident (bool memory_resident)
      throw (CORBA::SystemException)
    {
      // TODO
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
				 CORBA::COMPLETED_NO);

    }

    CORBA::Long Process_impl::get_core_dump_size ()throw (CORBA::
							  SystemException)
    {
      // TODO
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
				 CORBA::COMPLETED_NO);
    }




    CORBA::Boolean Process_impl::is_auto_ending ()throw (CORBA::
							 SystemException)
    {
      return m_autoending;
    }

    CdmwPlatformMngt::ManagedElement_ptr Process_impl::
      get_parent ()throw (CORBA::SystemException)
    {
      return m_application->_this ();
    }

    CdmwPlatformMngt::Host_ptr Process_impl::get_host ()throw (CORBA::
							       SystemException)
    {
      CdmwPlatformMngt::Host_var host = m_host;

      return host._retn ();
    }

    CdmwPlatformMngt::ProcessInfo *
      Process_impl::get_process_info ()throw (CORBA::SystemException)
    {
      try
      {
	CdmwPlatformMngt::ProcessInfo_var processInfo
	  = new CdmwPlatformMngt::ProcessInfo (m_process_info.in ());

	return processInfo._retn ();
      }
      catch (const std::bad_alloc &)
      {
	throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
      }
    }

    CdmwPlatformMngt::LifeCycleDef *
      Process_impl::get_life_cycle ()throw (CdmwPlatformMngt::
					    IncompatibleType,
					    CORBA::SystemException)
    {
      throw CdmwPlatformMngt::IncompatibleType ();
    }
    
        void Process_impl::set_life_cycle (const CdmwPlatformMngt::
            LifeCycleDef & life_cycle)
            throw (CdmwPlatformMngt::DuplicateStep,
            CdmwPlatformMngt::IncompatibleType,
            CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
        {
            throw CdmwPlatformMngt::IncompatibleType ();
        }
        
        CdmwPlatformMngt::CompleteMonitoringInfo Process_impl::
            get_monitoring_info ()throw (CdmwPlatformMngt::IncompatibleType,
            CORBA::SystemException)
        {
            throw CdmwPlatformMngt::IncompatibleType ();
        }
        
        void Process_impl::set_monitoring_info (const CdmwPlatformMngt::
            CompleteMonitoringInfo &
            monitoring_info)
            throw (CdmwPlatformMngt::IncompatibleType,
            CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
        {
            throw CdmwPlatformMngt::IncompatibleType ();
        }
        
        CdmwPlatformMngt::ExitAutoRestartInfo *
            Process_impl::get_exit_autorestart_info ()throw (CORBA::SystemException)
        {
             try
            {
                CdmwPlatformMngt::ExitAutoRestartInfo_var exit_autorestart_info
                    = new CdmwPlatformMngt::ExitAutoRestartInfo (m_exit_autorestart_info_var.in ());
                
                return exit_autorestart_info._retn ();
            }
            catch (const std::bad_alloc &)
            {
                throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
            }
        }
        
        
        
        void Process_impl::set_exit_autorestart_info (const CdmwPlatformMngt::
            ExitAutoRestartInfo & exit_autorestart_info)
            throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
        {
            try
            {
                m_exit_autorestart_info_var = new CdmwPlatformMngt::ExitAutoRestartInfo (exit_autorestart_info);
       
                // update record in datastore
                set_record_in_datastore ();
            }
            catch (const std::bad_alloc &)
            {
                throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
            }
        }
        
        void Process_impl::perform_exit_autorestart_action ()
              throw (CdmwPlatformMngt::IncompatibleStatus,
                     CORBA::SystemException)
        {
            CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                     "perform_exit_autorestart_action : "
                     << m_element_path.in());
            try 
            {
                CORBA::String_var status_info;
                if ( get_host()->status() != CdmwPlatformMngt::HOST_UNREACHABLE &&
                     !(get_host()->get_reboot_in_progress()) &&
                     CdmwPlatformMngt::System::_narrow(( get_application()->get_parent()))->get_status(status_info)!=CdmwPlatformMngt::SYSTEM_STOPPING &&
                     get_application()->get_internal_status ()!= CdmwPlatformMngt::APPLICATION_STOPPING &&
                     getPreviousStatus()!=CdmwPlatformMngt::PROCESS_STOPPING
                   )
                {
                    //recover the exit action to perform
                    CdmwPlatformMngt::ExitActionType current_exit_action=m_exit_autorestart_info_var->default_exit_action;
                    CORBA::Long current_exit_code=0; 
                    if (m_last_exit_info.exit_event==CdmwPlatformMngt::EXITED) 
                    { 
                        CDMW_DEBUG(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                                   "perform_exit_autorestart_action : "<< m_element_path.in());
                        current_exit_code=m_last_exit_info.event_value;
                        CdmwPlatformMngt::ExitActions exit_actions=m_exit_autorestart_info_var->exit_actions;
                        for (CORBA::ULong i = 0; i < exit_actions.length (); i++)
                        {
                            CDMW_DEBUG(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                                       "perform_exit_autorestart_action : "<< m_element_path.in()); 
                            if (exit_actions[i].exit_code==current_exit_code)
                            {
                                current_exit_action=exit_actions[i].exit_action;
                            }
                        }

                    }  
                    switch (current_exit_action)
                    {
                    case CdmwPlatformMngt::NO_ACTION:
                        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                        "perform_exit_autorestart_action NO_ACTION case : "<< m_element_path.in());
                        break;
                    case CdmwPlatformMngt::RESTART_PROCESS:
                    { 
                        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                        "perform_exit_autorestart_action RESTART_PROCESS case : "<< m_element_path.in());
                        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                                  "perform_exit_autorestart_action : restart_attempt ()= "<<restart_attempt ());
                        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                                  "perform_exit_autorestart_action : max_nbr_of_restart_attempt = " 
                                   <<m_exit_autorestart_info_var->max_nbr_of_restart_attempt);                        
                        if ( restart_attempt () < m_exit_autorestart_info_var->max_nbr_of_restart_attempt)
                        {
                            CdmwPlatformMngtBase::StartupKind* startup_kind=get_startup_kind_from_ds();
                            set_restart_attempt (restart_attempt ()+1);
                            // Done in hot_restart: set_record_in_datastore ();
                            hot_restart(*startup_kind); 
                        }
                    }    
                        break;
                    case CdmwPlatformMngt::STOP_NODE:
                    {
                        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                        "perform_exit_autorestart_action STOP_NODE case : "<< m_element_path.in());
                        get_host()->shutdown();
                    }    
                        break;
                    case CdmwPlatformMngt::RESTART_NODE:
                    {
                        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                        "perform_exit_autorestart_action RESTART_NODE case : "<< m_element_path.in());
                        get_host()->hot_reboot();
                    }   
                        break;
                    case CdmwPlatformMngt::STOP_APPLICATION:
                    {
                        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                        "perform_exit_autorestart_action STOP_APPLICATION case : "<< m_element_path.in());
                        get_application()->stop_and_wait(false);
                    }     
                        break;
                    case CdmwPlatformMngt::RESTART_APPLICATION:
                    {
                        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                        "perform_exit_autorestart_action RESTART_APPLICATION case : "<< m_element_path.in());
                        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                                  "perform_exit_autorestart_action : Application restart_attempt ()= "
                                  << get_application()->restart_attempt ());
                        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                                  "perform_exit_autorestart_action : Application  max_nbr_of_restart_attempt ()= "
                                  << get_application()->get_max_restart_attempt ());
 
                        if ( get_application()->restart_attempt () < get_application()->get_max_restart_attempt ())
                        {
                            const CdmwPlatformMngtBase::StartupKind startup_kind;
                            get_application()->set_restart_attempt (get_application()->restart_attempt ()+1);
                            // Done in hot_restart: get_application()->set_record_in_datastore ();
                            get_application()->hot_restart(startup_kind);
                        }
                        else if ( get_application()->restart_attempt () >= get_application()->get_max_restart_attempt ())
                        {
                            CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                                      "Stop Application max_restart_attempt reached: STOP Application : "<< m_element_path.in());
                            get_application()->stop_and_wait(false);
                        }
 
                    }     
                        break;
                    default:
                        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                        "perform_exit_autorestart_action NO_ACTION case : "<< m_element_path.in());
                    };
                }
                else
                {
                    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                     "Process_impl : NO AUTO ACTION PERFORMED : STOPPING or HOST UNREACHABLE : "
                     << m_element_path.in());
                }

            }
            catch (...)
            {
            }
        }
        CdmwPlatformMngt::ProcessSnapshot *
            Process_impl::get_snapshot ()throw (CORBA::SystemException)
        {
            try
            {
                unsigned int i;
                
                // Get the snapshot of all entities of application
                CdmwPlatformMngt::EntitiesSnapshot_var entitiesSnapshot
                    = new CdmwPlatformMngt::EntitiesSnapshot ();
                
                // If the process is managed
                if (type () != CdmwPlatformMngt::UNMANAGED_PROCESS)
                {
                    // Gets all the entities
                    CdmwPlatformMngt::Entities_var entities;
                    CdmwPlatformMngt::EntitiesIterator_var entitiesIt;
                    CORBA::ULong nbr_of_entities;
                    
                    nbr_of_entities = get_number_of_entities ();
                    
                    get_all_entities (nbr_of_entities,
                        entities.out (), entitiesIt.out ());
                    
                    entitiesSnapshot->length (nbr_of_entities);
                    for (i = 0; i < nbr_of_entities; i++)
                    {
                        entitiesSnapshot[i] = *(entities[i]->get_snapshot ());
                    }
                }
                else
                {
                    entitiesSnapshot->length (0);
                }
                
                //Gets all the properties
                CosPropertyService::PropertiesIterator_var propertiesIt;
                CORBA::ULong nbr_of_properties;
                CdmwPlatformMngt::PropertiesSnapshot_var propertiesSnapshot =
                    new CdmwPlatformMngt::PropertiesSnapshot;
                nbr_of_properties = get_number_of_properties ();
                CosPropertyService::PropertyNames_var prop_names;
                CosPropertyService::PropertyNamesIterator_var prop_namesIt;
                get_all_property_names (nbr_of_properties, prop_names.out (),
                    prop_namesIt.out ());
                propertiesSnapshot->length (nbr_of_properties);
                for (i = 0; i < nbr_of_properties; i++)
                {
                    CdmwPlatformMngt::PropertySnapshot * propertySnapshot =
                        new CdmwPlatformMngt::PropertySnapshot ();
                    propertySnapshot->property_name =
                        CORBA::string_dup (prop_names[i]);
                    CORBA::Any prop_value;
                    prop_value = *(get_property_value (prop_names[i]));
                    CORBA::TypeCode_var tc;
                    tc = prop_value.type ();
                    switch (tc->kind ())
                    {
                    default:
                        break;
                    case CORBA::tk_string:
                        propertySnapshot->property_type =
                            CORBA::string_dup ("String");
                        break;
                    case CORBA::tk_short:
                        propertySnapshot->property_type = CORBA::string_dup ("Short");
                        break;
                    case CORBA::tk_float:
                        propertySnapshot->property_type = CORBA::string_dup ("Float");
                        break;
                    };
                    CosPropertyService::PropertyModeType mode =
                        get_property_mode (prop_names[i]);
                    propertySnapshot->property_value = prop_value;
                    if (mode == 0)
                        propertySnapshot->property_mode =
                        CORBA::string_dup ("Read/Write");
                    else
                        propertySnapshot->property_mode =
                        CORBA::string_dup ("Read/Only");
                    propertySnapshot->property_value = prop_value;
                    propertiesSnapshot[i] = *(propertySnapshot);
                }
                
                
                // Create a process snapshot
                CORBA::String_var statusInfo;
                CdmwPlatformMngt::ProcessSnapshot_var snapshot
                    = new CdmwPlatformMngt::ProcessSnapshot;
                
                // note : the copy of a snapshot sequence (= operator) 
                //        make a deeply copy with a new memory allocation
                //        for the sequence 
                
                snapshot->process_name = CORBA::string_dup (m_element_name.in ());
                snapshot->host_name = CORBA::string_dup (m_proc_host_name.in ());
                snapshot->process_status = get_status (statusInfo.out ());
                snapshot->process_status_info = statusInfo;
                snapshot->process_entities = entitiesSnapshot.in ();
                snapshot->process_properties = propertiesSnapshot.in ();
                
                return snapshot._retn ();
            }
            catch (const std::bad_alloc &)
            {
                throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
            }
    }
    
    CdmwPlatformMngt::RequestStatus Process_impl::
        next_step ()throw (CdmwPlatformMngt::InvalidStep,
        CdmwPlatformMngt::IncompatibleType,
        CdmwPlatformMngt::IncompatibleStatus,
        CORBA::SystemException)
    {
        throw CdmwPlatformMngt::IncompatibleType ();
    }
    
    CdmwPlatformMngt::RequestStatus Process_impl::
        run ()throw (CdmwPlatformMngt::NotReadyToRun,
        CdmwPlatformMngt::IncompatibleType,
        CdmwPlatformMngt::IncompatibleStatus,
        CORBA::SystemException)
    {
        throw CdmwPlatformMngt::IncompatibleType ();
    }
    
    CdmwPlatformMngt::RequestStatus Process_impl::
        cold_restart (const CdmwPlatformMngtBase::
        StartupKind & startup_kind) throw (CORBA::SystemException)
    {
      set_restart_attempt (0);
      bool emergency = false;
      try {

      if (CdmwPlatformMngt::LONG_REQUEST_COMPLETED !=
      	 restart ( CdmwPlatformMngt::COLD_RESTART, startup_kind, emergency ) ){
	 return CdmwPlatformMngt::REQUEST_ERROR;	 
	 }
      }
      catch (...) {
      throw;
      }

      	return CdmwPlatformMngt::REQUEST_SUCCEED;
    }
    
    
    CdmwPlatformMngt::RequestStatus Process_impl::
        hot_restart (const CdmwPlatformMngtBase::
        StartupKind & startup_kind) throw (CORBA::SystemException)
    {
      bool emergency = false;
      try {

      if (CdmwPlatformMngt::LONG_REQUEST_COMPLETED !=
      	 restart ( CdmwPlatformMngt::HOT_RESTART, startup_kind, emergency ) ){
	 return CdmwPlatformMngt::REQUEST_ERROR;	 
	 }
      }
      catch (...) {
      throw;
      }

      	return CdmwPlatformMngt::REQUEST_SUCCEED;
    }
    
    CdmwPlatformMngt::LongRequestStatus Process_impl::restart ( const CdmwPlatformMngt::RestartWarmth restartWarmth,
		const CdmwPlatformMngtBase::StartupKind & startup_kind,
		bool emergency)
		throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
		
    {
        //CdmwPlatformMngt::ProcessStatus procStatus;
	//TODO add control on current status
	//TODO add return values
	set_requested_restart_warmth ( restartWarmth);
 	
	try{
		stop_and_wait( emergency );
	}
 	catch (...) {
	throw;
        }


	try{
		initialise ( startup_kind );
	}
    	catch (...) {
		throw;
        }

	try{
	       if (0 != dynamic_cast<ManagedProcess_impl*> (this))
      	       run ();
	}
    	catch (...) {
		throw;
        }
	
	/* Reset the process default_restart_warmth */
        set_requested_restart_warmth ( get_default_restart_warmth () );  
 	return CdmwPlatformMngt::LONG_REQUEST_COMPLETED;
    }
    
    CORBA::ULong Process_impl::get_number_of_entities ()throw (CORBA::
        SystemException)
    {
        throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
            CORBA::COMPLETED_NO);
    }
    

    void Process_impl::get_all_entity_names (CORBA::ULong how_many,
        CdmwPlatformMngt::
        ElementNames_out entity_names,
        CdmwPlatformMngt::
        ElementNamesIterator_out rest)
        throw (CORBA::SystemException)
    {
        throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
            CORBA::COMPLETED_NO);
    }
    
    CdmwPlatformMngt::Entity_ptr Process_impl::
        get_entity (const char *entity_name) throw (CdmwPlatformMngt::
						  EntityNotFound,
                          CORBA::SystemException)
    {
        throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
            CORBA::COMPLETED_NO);
    }
    
    CORBA::Boolean Process_impl::get_entities (const CdmwPlatformMngt::
        ElementNames & entity_names,
        CdmwPlatformMngt::
        Entities_out nentities)
        throw (CORBA::SystemException)
    {
        throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
            CORBA::COMPLETED_NO);
    }
    
    void Process_impl::get_all_entities (CORBA::ULong how_many,
        CdmwPlatformMngt::
        Entities_out nentities,
        CdmwPlatformMngt::
        EntitiesIterator_out rest)
        throw (CORBA::SystemException)
    {
        throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
            CORBA::COMPLETED_NO);
    }
    
    CdmwPlatformMngt::Entity_ptr Process_impl::
        add_entity (const CdmwPlatformMngt::
        EntityDef & entity_def) throw (CdmwPlatformMngt::
        EntityAlreadyExists,
        CORBA::SystemException)
    {
        throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
            CORBA::COMPLETED_NO);
    }
    
    void Process_impl::remove_entity (const char *entity_name)
        throw (CdmwPlatformMngt::EntityNotFound, CORBA::SystemException)
    {
        throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
            CORBA::COMPLETED_NO);
    }
    
    CdmwPlatformMngtService::ServiceProviderID *
        Process_impl::get_service_def (const char *service_name)
        throw (CdmwPlatformMngtService::ServiceNotFound, CORBA::SystemException)
    {
        throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
            CORBA::COMPLETED_NO);
    }
    
    void Process_impl::add_service_def (const CdmwPlatformMngtService::
        ServiceDef & service_def)
        throw (CdmwPlatformMngtService::ServiceAlreadyExists,
        CORBA::SystemException)
    {
        throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
            CORBA::COMPLETED_NO);
    }
    
    void Process_impl::remove_service_def (const char *service_name)
        throw (CdmwPlatformMngtService::ServiceNotFound, CORBA::SystemException)
    {
        throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
            CORBA::COMPLETED_NO);
    }
    
    void Process_impl::remove_all_service_defs ()
        throw (CORBA::SystemException)
    {
        throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
            CORBA::COMPLETED_NO);
    }
    
    void Process_impl::is_alive ()
    {
        // TODO
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
            CORBA::COMPLETED_NO);
    }
    
    CdmwPlatformMngt::ApplicationAgent_ptr
        Process_impl::getApplicationAgent ()throw (CdmwPlatformMngt::
        HostNotFound,
        CdmwPlatformMngt::
        HostNotReachable,
        CdmwPlatformMngt::
        CreationError,
        CdmwPlatformMngt::
        ApplicationAgentAlreadyExists,
        CdmwPlatformMngt::
        InvalidInfo,
        CdmwPlatformMngt::
        CreationTimeout,
        AgentNotFoundException,
        CORBA::SystemException)
    {
        CdmwPlatformMngt::ApplicationAgent_var agent;
        
        // Get the application agent on the host of this process
        // if does not exist start a new one
        agent = m_application->getAgent (m_proc_host_name.in ());
        
        return agent._retn ();
    }
    
    CdmwPlatformMngt::ApplicationAgent_ptr
        Process_impl::searchApplicationAgent ()throw (CdmwPlatformMngt::
        HostNotFound,
        CdmwPlatformMngt::
        HostNotReachable,
        CORBA::SystemException)
    {
        CdmwPlatformMngt::ApplicationAgent_var agent;
        
        // Get the application agent on the host of this process
        // if does not exist nil is returned
        agent = m_application->searchAgentFromDaemon (m_proc_host_name.in ());
        
        return agent._retn ();
    }
    
    
    
    
    void Process_impl::setAutoending (CORBA::Boolean auto_ending)
    {
        m_autoending = auto_ending;
        
        // update record in datastore
        set_record_in_datastore ();
    }
    
    void Process_impl::notifyEntityStatusChangeEvent (const char *entity_name,
        const char
        *entity_status,
        const char *entity_info)
    {
        try
        {
            CdmwPlatformMngtBase::EventHeader header
                = EventHeaderFactory::createHeader (CdmwPlatformMngtBase::INF);
            
            CdmwPlatformMngt::ProcessEntityStatusChange_var event
                = m_procEntityStatusChangeFactory->create (header,
                m_application->
                get_system_name (),
                m_application->
                get_element_name (),
                m_element_name.in (),
                m_proc_host_name.in (),
                entity_name,
                entity_status,
                entity_info);
            
            CORBA::String_var eventStr = event->to_string ();
            LogMngr::logMessage (eventStr.in ());
            
            m_application->get_event_notifier ()->addEvent (event);
            event._retn ();
        }
        catch (const std::bad_alloc &)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
        catch (...)
        {
            throw CORBA::INTERNAL (OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }
    }
    
    void Process_impl::setHost (const char *host_name)
        throw (CdmwPlatformMngt::HostNotFound, CORBA::SystemException)
    {
        CdmwPlatformMngt::HostContainer_var host_container =
            m_application->get_host_container ();
        m_host = host_container->get_host (host_name);
        
        m_proc_host_name = CORBA::string_dup (host_name);
        
        // update record in datastore
        set_record_in_datastore ();
    }
    
    void Process_impl::setProcessInfo (const CdmwPlatformMngt::
        ProcessInfo & process_info)
    {
        try
        {
            m_process_info = new CdmwPlatformMngt::ProcessInfo (process_info);
            
            // update record in datastore
            set_record_in_datastore ();
        }
        catch (const std::bad_alloc &)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
    }
    
    void Process_impl::setMonitoringInfo (const CdmwPlatformMngt::
        CompleteMonitoringInfo &
        monitoring_info)
    {
        try
        {
            m_monitoring_info =
                new CdmwPlatformMngt::CompleteMonitoringInfo (monitoring_info);
            
            // update record in datastore    
            set_record_in_datastore ();
        }
        catch (const std::bad_alloc &)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
    }
    
    void Process_impl::errorEvent (const CdmwPlatformMngtBase::
        EventHeader & header, const char *issuer,
        const char *error_info)
    {

        // Do nothing if in a failed status
        CdmwPlatformMngt::ProcessStatus status = getStatus ();
        
        if ((status == CdmwPlatformMngt::PROCESS_FAILED_ERROR) ||
            (status == CdmwPlatformMngt::PROCESS_FAILED_TIMEOUT) ||
            (status == CdmwPlatformMngt::PROCESS_FAILED_DISCREPANCY) ||
            (status == CdmwPlatformMngt::PROCESS_FAILED_INVALID) ||
            (status == CdmwPlatformMngt::PROCESS_FAILED_NO_RESPONSE) ||
            (status == CdmwPlatformMngt::PROCESS_FAILED_DEATH))
            return;
        
        // Set the status info's message
        m_status_info = CORBA::string_dup (error_info);
        
        // Store the info about the issuance of this error event.
        m_lastErrorHeader = header;
        m_lastErrorIssuer = issuer;
        
        // Change to the state FailedError
        setState ("FailedError");
        
        // update record in datastore
        set_record_in_datastore ();
    }
    
    
    void Process_impl::endingEvent (const CdmwPlatformMngtBase::
				    EventHeader & header)
    {
        // Do nothing if in a failed status
        CdmwPlatformMngt::ProcessStatus status = getStatus ();
        
        if ((status == CdmwPlatformMngt::PROCESS_FAILED_ERROR) ||
            (status == CdmwPlatformMngt::PROCESS_FAILED_TIMEOUT) ||
            (status == CdmwPlatformMngt::PROCESS_FAILED_DISCREPANCY) ||
            (status == CdmwPlatformMngt::PROCESS_FAILED_INVALID) ||
            (status == CdmwPlatformMngt::PROCESS_FAILED_NO_RESPONSE) ||
            (status == CdmwPlatformMngt::PROCESS_FAILED_DEATH))
            return;
        
        // if process is authorised to end itself    
        if (is_auto_ending ())
        {
            // change to the status Ended
            setState ("Ended");
        }
        
        else
        {
            // Set the status info's message
            m_status_info =
                CORBA::string_dup (LogMngr::
                getFormattedMessage
                (MSG_ID_PROCESS_ENDING_ERROR).c_str ());
            
            // Store the info about the issuance of this event.
            m_lastErrorHeader = header;
            
            // Change to the state FailedDeath
            setState ("FailedDeath");
        }
        
        // update record in datastore
        set_record_in_datastore ();
    }
    
    void Process_impl::invalidateEvent (const CdmwPlatformMngtBase::
        EventHeader & header,
        const char *reason)
    {
        // Do nothing if in a failed status
        CdmwPlatformMngt::ProcessStatus status = getStatus ();
        
        if ((status == CdmwPlatformMngt::PROCESS_FAILED_ERROR) ||
            (status == CdmwPlatformMngt::PROCESS_FAILED_TIMEOUT) ||
            (status == CdmwPlatformMngt::PROCESS_FAILED_DISCREPANCY) ||
            (status == CdmwPlatformMngt::PROCESS_FAILED_INVALID) ||
            (status == CdmwPlatformMngt::PROCESS_FAILED_NO_RESPONSE) ||
            (status == CdmwPlatformMngt::PROCESS_FAILED_DEATH))
            return;
        
        // Set the status info's message
        m_status_info = CORBA::string_dup (reason);
        
        // Store the info about the issuance of this event.
        m_lastErrorHeader = header;
        
        // Change to the state FailedInvalid
        setState ("FailedInvalid");
        
        // update record in datastore
        set_record_in_datastore ();
    }
    
    
    Application_impl *Process_impl::get_application ()
    {
        return m_application;
    }
    
    const char *Process_impl::get_process_host_name ()
    {
        return m_proc_host_name.in ();
    }
    
    
    const char *Process_impl::get_process_key ()
    {
        return m_proc_key.in ();
    }
    
    void Process_impl::set_create_process_result (CdmwPlatformMngt::
						  ProcessCommandResult result,
                          const char *error_info,
                          const char *error_issuer,
                          CdmwPlatformMngt::
                          ProcessDelegateWrapper_ptr
                          process_delegate_wrap)
                          throw (CORBA::SystemException)
    {
        // the process delegate wrapper object is not used in this case
        
        m_starting_result = result;
        
        m_creation_error_info = CORBA::string_dup (error_info);
        m_creation_error_issuer = CORBA::string_dup (error_issuer);
        
        // signals the acknowledgement
        if (!m_process_start_cbk->signal ())
        {
            // The acknowledgement has been performed too late
            throw CORBA::NO_PERMISSION (OrbSupport::NO_PERMISSION,
                CORBA::COMPLETED_NO);
        }
    }
    
    
    
    void Process_impl::set_initialisation_result (CdmwPlatformMngt::
						  ProcessCommandResult result)
                          throw (CORBA::SystemException)
    {
        throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
            CORBA::COMPLETED_NO);
    }
    
    
    void Process_impl::set_next_step_result (CdmwPlatformMngt::
        ProcessCommandResult result,
        CORBA::
        ULong step_nbr) throw (CORBA::
        SystemException)
    {
        throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
            CORBA::COMPLETED_NO);
    }
    
    
    void Process_impl::set_run_result (CdmwPlatformMngt::
        ProcessCommandResult result)
        throw (CORBA::SystemException)
    {
        throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
            CORBA::COMPLETED_NO);
    }
    
    
    void Process_impl::set_stop_result (CdmwPlatformMngt::
        ProcessCommandResult result)
        throw (CORBA::SystemException)
    {
        throw CORBA::BAD_OPERATION (OrbSupport::BAD_OPERATION,
            CORBA::COMPLETED_NO);
    }
    
    bool Process_impl::actual_manual_starting ()
        throw (CORBA::SystemException)
    {
         return m_actual_manual_starting;
    }
 
    void Process_impl::set_force_manual_start( const bool force_manual_start )
      throw (CORBA::SystemException)
    {
        m_force_manual_start = force_manual_start;
        m_actual_manual_starting = m_force_manual_start;
    }

    bool Process_impl::get_force_manual_start()
      throw (CORBA::SystemException)
    {
        return m_force_manual_start;
    }

    void Process_impl::set_last_exit_code( const CdmwPlatformMngt::ProcessExitInfo & last_exit_code )
      throw (CORBA::SystemException)
    {
        m_last_exit_info=last_exit_code;
    }

    CdmwPlatformMngt::ProcessExitInfo Process_impl::get_last_exit_code()
      throw (CORBA::SystemException)
    {
        return m_last_exit_info;
    }
    
    void Process_impl::set_process_arguments( const char * process_arguments )
      throw (CORBA::SystemException)
    {
        // TODO
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }

    char *  Process_impl::get_process_arguments()
      throw (CORBA::SystemException)
    {
        // TODO
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }

    void Process_impl::set_indicator_relevant_status( const bool indicator_relevant_status)
      throw (CORBA::SystemException)
    {
        // TODO
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }

    bool  Process_impl::get_indicator_relevant_status()
      throw (CORBA::SystemException)
    {
        // TODO
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }

    void Process_impl::set_trace_filtering(const char *trace_filtering) throw (CORBA::SystemException, CdmwPlatformMngt::ParsingException , CdmwPlatformMngt::CastException )
    {
    string  dummy;  
    char *mytok; 
    char *newtok;
    int lenstr; 
    unsigned long int lev_value;
    int exponent;
    string component, domain;
    filters_map tmp_map(m_filt_map);

    vector <string> items ; 

    //test if trace_filtering is null or empty string
    if ( (trace_filtering==NULL) || ( trace_filtering[0]=='\0') )
	{
        throw  CdmwPlatformMngt::ParsingException("Error: input string is a NULL pointer or an empty string");       
	return;
	}

    string mystr(trace_filtering);

    newtok = (char *)mystr.c_str();

    int original_size = strlen(newtok);
    int total_len = 0;

    vector <string> final_tokens;


    for(;;)
      {

      mytok = strtok( newtok ,"/");
    

       //insert the token inside a vector

      items.push_back(mytok);


      lenstr = strlen(mytok);

      total_len += (lenstr+1);

      if ( total_len < original_size)
      {
       newtok= &newtok[lenstr+1];
      }
      else
      {
      //end of token reading...
      break;
      }


     }

     int i;
     int size_vec = items.size() ;


     //PARSING ALL ELEMENTS OF items vector

    int count_token;
    string original_str;
    int total_num=0;
    filters_map::const_iterator newit;
    my_key to_search;
    my_key valid_my_key;
    my_key  tmp_key;
    unsigned long new_lev_value;
    int size_str;
    int jj;
    char already_calculated=0;
    string token_lev_value;
    char already_exited=0;

    for (i=0; i<size_vec; i++)
    {
    newtok = (char *)items[i].c_str();

    string original_str( newtok);

    original_size = strlen(newtok);
    total_len = 0;

    count_token=0;


    for(;;)
    {


    if (count_token==0)
       mytok = strtok( newtok ,",");
    else
       mytok = strtok( NULL,",");

    if (mytok == NULL)
      {
      //end of tokens reading
      if(count_token <3)
        { //parsing error, no enough tokens read
        throw  CdmwPlatformMngt::ParsingException("Error: no enough fields filled...");       
	return;
	}

      break;
      }

    count_token++;

    //inserisci il token dentro un vettore 
    final_tokens.push_back(mytok);
    already_exited=0;
    try
    {
    stripLeadingAndTrailingBlanks( final_tokens[total_num]);
    }
    catch(...)
    {
    throw  CdmwPlatformMngt::ParsingException("Error: a field is totally blank!");
    already_exited=1;
    }

    if ( already_exited==1)
	{
        m_filt_map.swap(tmp_map);
        return;  //exit from this method ...
	}

    already_calculated=0;

    //insert the element in tha internal map ...
    if ( count_token==3) 
       {
       token_lev_value = final_tokens[total_num] ;
       //if the string is greater than 2 elements and
       //a character is not a digit, so it is an Error ...
       size_str = token_lev_value.size() ;
       if (size_str >1)
         for (jj=0; jj<size_str;jj++)
	   {
           if ( isdigit( (token_lev_value.c_str())[jj] )==false)
	     {
             throw CdmwPlatformMngt::ParsingException("Error in a input level value, there is an invalid character!");
	     m_filt_map.swap(tmp_map);
	     return;
	     }

	   }

       if (  !(token_lev_value.compare("")) )
         {
	 //Error, a level value field id empty
         throw CdmwPlatformMngt::ParsingException("Error, a level value field id empty");
	 m_filt_map.swap(tmp_map);
	 return;
	 }
       

       if (  !(token_lev_value.compare("!")) )
	     {
             lev_value=0;
	     already_calculated=1;
	     }
       else
         if ( !(token_lev_value.compare("*")) )
	     {
	     lev_value=4294967295UL;
	     already_calculated=1;
	     }
         else
           if (( token_lev_value.size() == 1 ) && ( isdigit( (token_lev_value.c_str())[0]) == false ))
           {
           throw CdmwPlatformMngt::ParsingException("Error! An input level value contain 1 character not valid");
	   m_filt_map.swap(tmp_map);
	   return;
	   }
         
	   if (already_calculated==0)
	    {
	    exponent = atoi( token_lev_value.c_str());
	    if ( (exponent <0) || (exponent >31))
	       {
               throw CdmwPlatformMngt::ParsingException("Error: a level must be inside the range [0,31]");
	       m_filt_map.swap(tmp_map);
	       return;
	       }
            lev_value = (unsigned long int) exp2( (double) exponent) ;
	    }

           /*before insert new value in the map, see if the pair
	   component,domain is already present in the map...*/
	   component =  final_tokens[total_num-2] ;
	   domain =  final_tokens[total_num-1];

        
           /*find component,domain in the map*/
	   to_search.first = component;
	   to_search.second = domain ;

           /* check on '!' character ... */
           if ( !(domain.compare("!")) )
	      {
	      m_filt_map.swap(tmp_map);
              throw CdmwPlatformMngt::ParsingException("Error: the domain can not be '!' character");
	      return;
	      }

           newit = m_filt_map.find( to_search) ;

	   if ( newit != m_filt_map.end() )
	     { //pair is found
             //perform the necessary operations ...

             if ( lev_value > 0)
	       { //perform the OR operation on the bits ...
	       new_lev_value = lev_value | (newit->second)  ;
	       m_filt_map[ to_search ] = new_lev_value;
	       }
             else
	       { //lev. value = 0 , so reset all the levels ...
	       new_lev_value = 0;
	       m_filt_map[ to_search ] = new_lev_value;
	       }

	     }
           else
	     { //pair is not found, so insert it at the end of map
             m_filt_map[ to_search ] = lev_value; 
	     }
          }

      
    total_num++;

    lenstr = strlen(mytok);

    total_len += (lenstr);

    if ( total_len >= original_size)
      {
      //end of tokens reading
      if (count_token < 3)
        { //parsing error, no enough tokens read
        throw  CdmwPlatformMngt::ParsingException("Error: no enough fields filled...");       
	return;
	}
      break;
      }

     }
    }



    //update the filters_ds in datastore
    set_record_in_datastore() ;


    int ret_code;

    //apply the filters...
    ret_code = apply_trace_filters() ;

    switch(ret_code)
      {
      case -1:
        throw CdmwPlatformMngt::CastException( "Error in dynamic_cast<ManagedProcess_impl *>" );
        return;
	break;

      case -2: //case of ProcessDelegate not ready, so only the filters map was filled
        return;
	break;

      case 0: //all ok
        break;
      }

    }


    int Process_impl::apply_trace_filters() throw (CORBA::SystemException)
    {
       //example
       //m_processDelegateWrap->activate_level("com1","dom1",8);


       ManagedProcess_impl *manag = dynamic_cast<ManagedProcess_impl *> (this);

       if (!manag)
         {
	  return(-1);
	 }

       /*test if m_processDelegateWrap is nil*/
       if ( CORBA::is_nil(manag->m_processDelegateWrap.in() ) ) 
         {
	 //in apply_trace_filters m_processDelegateWrap is NIL
	 return (-2);
	 }

       //if filters map is empty, it returns
       if ( m_filt_map.size() == 0)
             return (0);

       //analyzing the map to discover the activate to apply...

       //first of all: deactivate all levels in all components 
       //and in all domains...
       manag->m_processDelegateWrap->deactivate_level(Cdmw::Logging::ALL_COMPONENT_NAMES.c_str(), Cdmw::Logging::ALL_DOMAINS.c_str(), Cdmw::Logging::ALL_LEVELS_SET );
       
       
       //analyzing the map of filters to apply ...
       filters_map::const_iterator it  = m_filt_map.begin();
       filters_map::const_iterator end = m_filt_map.end();
       my_key tmp;

       for(;it!=end;++it)
       {
         tmp = it->first;

         if ( (it->second) > 0)
             {
             manag->m_processDelegateWrap->activate_level( tmp.first.c_str(), tmp.second.c_str(), it->second );
             }
         else
	    { //deactivate all levels in the specific comp. and domain
             manag->m_processDelegateWrap->deactivate_level( tmp.first.c_str(), tmp.second.c_str(),  Cdmw::Logging::ALL_LEVELS_SET);

	    }
         
       } 

    return 0;
    }

    void Process_impl::stripLeadingAndTrailingBlanks(string& StringToModify)
    {
       if(StringToModify.empty()) return; 

       int startIndex = StringToModify.find_first_not_of(" ");
       int endIndex = StringToModify.find_last_not_of(" ");
       string tempString = StringToModify;
       StringToModify.erase();

       StringToModify = tempString.substr(startIndex, (endIndex-startIndex+ 1) );
    }

 
    char * Process_impl::get_trace_filtering() throw (CORBA::SystemException)
    {
    if ( m_filt_map.size() == 0)
	{
        return NULL;
	}

    string final_str("") , cur1_str("");

    filters_map::const_iterator it  = m_filt_map.begin();
    filters_map::const_iterator end = m_filt_map.end();
    my_key tmp = it->first; 
    char level_str[255+1];
    int ii;


    //reading actual filters from suitable map ...

     for(;it!=end;++it)
     {
     my_key tmp = it->first;

     cur1_str = tmp.first + "," + tmp.second;

     //see wich bits are up

     bitset<32>  bs (  it->second ) ;

     sprintf(&level_str[0],"%ld", it->second ); 

     for (ii=0; ii<32; ii++ )
       {
       if ( bs.test(ii) == true ) 
	 {
         sprintf(&level_str[0],"%d", ii ); 
         final_str = final_str + cur1_str +","+ level_str+"/" ;
	 }

       }


     } 

    //end to fill string ...

    //return a suitable char pointer ...

    CORBA::String_var filt_str = CORBA::string_dup(final_str.c_str() );

    return (filt_str._retn());

    }



    void Process_impl::set_component_start_graph(const CdmwPlatformMngt::GraphElements& component_graph)
      throw (CORBA::SystemException)
    {
        // TODO
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }

     CdmwPlatformMngt::GraphElements *Process_impl::get_component_start_graph()
      throw (CORBA::SystemException)
    {
        // TODO
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }

    void Process_impl::set_component_stop_graph(const CdmwPlatformMngt::GraphElements& component_graph)
      throw (CORBA::SystemException)
    {
        // TODO
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }

     CdmwPlatformMngt::GraphElements *Process_impl::get_component_stop_graph()
      throw (CORBA::SystemException)
    {
        // TODO
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }

    CdmwPlatformMngt::ElementType Process_impl::get_me_type()
        throw (CORBA::SystemException)
    {
            return CdmwPlatformMngt::PROCESS_TYPE;
    }

    CdmwPlatformMngt::Children* Process_impl::get_children()
        throw (CORBA::SystemException)
    {
            return 0;
    }
    
    void Process_impl::set_created_dynamically(bool created_dynamically)
    {
         m_created_dynamically = created_dynamically;
    }

  }				// End namespace PlatformMngt
}				// End namespace Cdmw
