/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */


#include <iostream>
#include <sstream>
#include "platformsystemlibrary/CentralisedSupervisionObserver_impl.hpp"
#include "Foundation/logging/LogManager.hpp"
#include "platformapplicationlibrary/Application_impl.hpp"
#include "platformapplicationlibrary/Process_impl.hpp"
#include "platformsystemlibrary/Host_impl.hpp"

namespace Cdmw 
{
namespace PlatformMngt 
{

CentralisedSupervisionObserver_impl::CentralisedSupervisionObserver_impl (CentralisedSupervisionSystem_impl* system)
{
    m_system=system;
}
    
void
CentralisedSupervisionObserver_impl::notify (CdmwPlatformMngt::Event* an_event)
    throw (CORBA::SystemException)
{
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
              "notify event : "<< an_event->to_string()); 

    if (an_event->event_kind()== CdmwPlatformMngt::HOST_STATUS_CHANGE )
    {
         CdmwPlatformMngt::HostStatusChange* event = CdmwPlatformMngt::HostStatusChange::_downcast( an_event );

         std::string hostname = (const char*)(event->host_name());
         CdmwPlatformMngt::HostStatus  status = event->host_status();

         Host_impl* host =  m_system -> getHost (hostname.c_str());
         if ((CdmwPlatformMngt::HOST_RESPONDING == status)&&(true == host->get_reboot_in_progress()))
         {
             m_system -> getHost (hostname.c_str())-> set_reboot_in_progress(false);

             try 
             {
	         m_system -> getHost (hostname.c_str())-> start_processes();
             }
	     catch (...) {
	     CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
              "Process restart on host reboot not completed");
	     }
         }

    }
    if (an_event->event_kind()== CdmwPlatformMngt::PROCESS_STATUS_CHANGE )
    {
        CdmwPlatformMngt::ProcessStatusChange* event = CdmwPlatformMngt::ProcessStatusChange::_downcast( an_event );
        std::string hostname = (const char*)(event->process_id().host_name);
        std::string appname = (const char*)(event->process_id().application_name);
        std::string process_name = (const char*)event->process_id().process_name;
        try
        {

            switch (event->process_status()) {
                case CdmwPlatformMngt::PROCESS_FAILED_DEATH:
                case CdmwPlatformMngt::PROCESS_FAILED_TIMEOUT:
                case CdmwPlatformMngt::PROCESS_FAILED_INVALID:
                case CdmwPlatformMngt::PROCESS_FAILED_NO_RESPONSE:
                case CdmwPlatformMngt::PROCESS_FAILED_DISCREPANCY:
                case CdmwPlatformMngt::PROCESS_FAILED_ERROR:
                {
                    //Application_impl*  app_impl=NULL;
                    //app_impl=m_system->get_application_impl(appname.c_str());
                    Application_impl* application_servant=0;
                    ApplicationControl_impl* app_control_servant=0;
                    m_system->getApplicationServants (appname.c_str(),
                                                      application_servant,
                                                      app_control_servant);
                    application_servant->getProcess(process_name.c_str(), hostname.c_str())
                          ->perform_exit_autorestart_action();
                }
                break;
                default:
                break; 
            }
        }
        catch (CORBA::SystemException& ) {
             //rethrow this exception
             throw;
        }
        catch (...) {
            //std::stringstream ss;
            //ss  << "Exception while handling process status change!" << std::endl;
            //Logging::LogManager::Instance()->logMessage(INFO, ss.str());
        }                 
    }           
}
} // End namespace PlatformMngt
} // End namespace Cdmw

