/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */

/* @author Fabrizio Lorenna <card03@selex-si.com> */

#include <iostream>
#include <limits>

#include "teststdattributes_Host/TestStdAttributes_Host.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"

#include <platformftproxy/InitUtils.hpp>
#include <platformftproxy/SystemProxy.hpp>
#include <SystemMngt/platformftproxy/AccessorLookupService.hpp>
#include <SystemMngt/platformftproxy/DefaultAccessorLookupService.hpp>
#include <SystemMngt/platformlibrary/Iterator.hpp>

#include <FaultTolerance/ftreplicationmanager/InitUtils.hpp>
#include <FaultTolerance/idllib/FT.stub.hpp>
#include <platformftaccessorlibrary/SystemAccessor_impl.hpp>

using namespace std;

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestStdAttributes_Host);

namespace Cdmw
{
  namespace PlatformMngt
  {



    TestStdAttributes_Host::TestStdAttributes_Host () throw ()
    {

    }


    TestStdAttributes_Host::~TestStdAttributes_Host () throw ()
    {

    }

    void TestStdAttributes_Host::setUp()
    {
      TestStdAttributes_Host::one_shot_init();
    }


    bool TestStdAttributes_Host::m_is_one_shot_initialized=false;

    CdmwPlatformMngt::Host_var TestStdAttributes_Host::myhost;
    CdmwPlatformMngt::System_var TestStdAttributes_Host::mysystem;
    char * TestStdAttributes_Host::hostname;
    CdmwPlatformMngt::CompleteMonitoringInfo TestStdAttributes_Host::glob_monitor_info;

    void
    TestStdAttributes_Host::one_shot_init()
    {
      if(!m_is_one_shot_initialized)
      {
        m_is_one_shot_initialized = true;



      PortableServer::POA_var poa ;

      try
      {

        CORBA::ORB_var orb;
        orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager:: instance ()->get_ORB ());

        poa = PortableServer::POA::_duplicate
(Cdmw::TestUtils::CORBATestManager:: instance ()->get_POA ());
        
        PortableServer::POAManager_var poaMgr = PortableServer::POAManager::_duplicate(poa->the_POAManager());
        poaMgr->activate();


        std::string fault_manager_corbaloc = "corbaloc::localhost:4555/fault_manager";

        CORBA::Object_var fault_manager_obj = orb->string_to_object(fault_manager_corbaloc.c_str());
        if (CORBA::is_nil(fault_manager_obj.in()))
        {
            CDMW_THROW2(Cdmw::Exception, Cdmw::Exception::FATAL, "Invalid fault manager corbaloc");
        }

        CdmwFT::FTManager_var fault_manager;

        try
            {
            fault_manager = CdmwFT::FTManager::_duplicate(::CdmwFT::FTManager::_narrow(fault_manager_obj.in()));
            }
        catch (const CORBA::SystemException & ex)
            {
            std::cerr << "Exception while contacting FTManager: " << ex
<< "\n" << fault_manager_corbaloc << " is probably invalid" <<
std::endl;
            }

        CdmwFT::ReplicationManager_var replication_manager = fault_manager->get_replication_manager();

        // get from the replication manager the system accessor group
        // reference
        // from its group name
        ::FT::Name group_name;
        group_name.length(1);
        group_name[0].id =
        Cdmw::PlatformMngt::Configuration::Get_SystemAccessor_group_name().c_str();

        CORBA::Object_var system_accessor_group_obj = replication_manager->get_object_group_ref_from_name(group_name);

        CdmwFTSystMngt::SystemAccessor_var system_accessor = CdmwFTSystMngt::SystemAccessor::_narrow(system_accessor_group_obj.in());

        Cdmw::PlatformMngt::Proxy::SystemProxy *theSystemProxyHandle =
NULL;
        const char * proxyName = "SystemMngtProxy";


        /* create lookup service */
        auto_ptr<Cdmw::PlatformMngt::Proxy::AccessorLookupService> lookupService (new Cdmw::PlatformMngt::Proxy::DefaultAccessorLookupService(system_accessor.in()));

        theSystemProxyHandle = Cdmw::PlatformMngt::Proxy::InitUtils::create_platformmngt_proxy (orb.in(), poa.in(), proxyName, lookupService.get()); /* proxy takes over responsibility for lookup service */

        lookupService.release();

        mysystem = theSystemProxyHandle->getReference();


	if (CORBA::is_nil (mysystem.in ()))
	  {
	    //TEST_INFO ("Error in narrowing CdmwPlatformMngt::System");
	    exit (-1);
	  }

	/* retrieve the current hostname */
	hostname = getenv ("HOSTNAME");

	myhost = mysystem->get_host (hostname);
        }
      catch (const Cdmw::Exception & ex)
      {
        std::cerr << ex.what () << std::endl;
        exit (-1);
      }
      catch (const CORBA::Exception & ex)
      {
        std::cerr << ex << std::endl;
        exit (-1);
      }
      catch (...)
      {
        std::cerr << "unexpected exception" << std::endl;
        exit (-1);
      }
     }
    }

    void TestStdAttributes_Host::get_name()
	{
	//TEST_INFO ("Test to get host name");
	CORBA::String_var name = myhost->name ();

	if (!strcmp (hostname, name.in ()))
	  {
	    CPPUNIT_ASSERT (1);	/* test ok */
	  }
	else
	  {
	    CPPUNIT_ASSERT (0);	/* test not ok */
	  }
         }



    void TestStdAttributes_Host::status()
	{
	//TEST_INFO ("Test on Host::status: ");
	CdmwPlatformMngt::HostStatus host_status = myhost->status ();

	if (host_status == CdmwPlatformMngt::HOST_RESPONDING)
	  {
	    CPPUNIT_ASSERT (1);
	  }
	else
	  {
	    CPPUNIT_ASSERT (0);
	  }

        }

    void TestStdAttributes_Host::host_status_info()
	{
	  //TEST_INFO ("Test on Host::host_status_info: ");
	  try
	   {
	   CORBA::String_var status_info = myhost->host_status_info ();
	   CPPUNIT_ASSERT(1);
	   }
          catch(...)
	   {
           CPPUNIT_ASSERT(0);
	   }
        }

    void TestStdAttributes_Host::get_monitoring_parameters()
	{
	  //TEST_INFO ("Test on Host::get_monitoring_parameters: ");
	  CdmwPlatformMngt::CompleteMonitoringInfo monitor_info;

	  monitor_info = myhost->get_monitoring_parameters ();

	  if ( (monitor_info.monitoring_model == 
	  glob_monitor_info.monitoring_model) &&
	  (monitor_info.pull_monitoring_interval ==
	  glob_monitor_info.pull_monitoring_interval) &&
	  (monitor_info.pull_monitoring_timeout ==
	  glob_monitor_info.pull_monitoring_timeout) &&
	  (monitor_info.push_monitoring_interval ==
	  glob_monitor_info.push_monitoring_interval) &&
	  (monitor_info.push_monitoring_timeout ==
	  glob_monitor_info.push_monitoring_timeout) )
	    CPPUNIT_ASSERT(1);
          else
	    CPPUNIT_ASSERT(0);
	}

    void TestStdAttributes_Host::set_monitoring_parameters()
	{
	  glob_monitor_info.monitoring_model = CdmwPlatformMngt::FULL_MONITORING_MODEL;

	  glob_monitor_info.pull_monitoring_interval = 15000 ;
	  glob_monitor_info.pull_monitoring_timeout = 30000;
	  glob_monitor_info.push_monitoring_interval = 2000;
	  glob_monitor_info.push_monitoring_timeout = 4000;

	  try
	    {
	    myhost->set_monitoring_parameters (glob_monitor_info);
	    CPPUNIT_ASSERT(1);
	    }
          catch(...)
	    {
            CPPUNIT_ASSERT(0);
	    }
        }

    void TestStdAttributes_Host::get_global_supervision_indicator()
	{
	//TEST_INFO ("Test on Host::get_global_supervision_indicator: ");
	  bool mybool = false;
	  mybool = myhost->get_global_supervision_indicator ();
	  if (mybool == true)
	    CPPUNIT_ASSERT(1);
          else
	    CPPUNIT_ASSERT(0);
         }


    void TestStdAttributes_Host::set_global_supervision_indicator()
	{
	  //TEST_INFO ("Test on Host::set_global_supervision_indicator: ");
	  bool mybool = true;
	  try
	   {
	   myhost->set_global_supervision_indicator (mybool);
	   CPPUNIT_ASSERT(1);
	   }
          catch (...)
	   {
	   CPPUNIT_ASSERT(0);
	   }
	}



  }				/* End namespace PlatformMngt */
}				/* End namespace Cdmw */
