/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
 *
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <limits>

#include "test_restart_objects/TestRestartObjs.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include <SystemMngt/platformlibrary/Iterator.hpp>

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "SystemMngt/platformvaluetypes/ProcessDef_impl.hpp"


using namespace std;

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestRestartObjs);

namespace Cdmw
{
  namespace PlatformMngt
  {

  
    
    TestRestartObjs::TestRestartObjs()
    {
    }

    void TestRestartObjs::setUp ()
    {
          try
          {
          process_test = 0;
	  std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";
	  //char *hostname = getenv ("HOSTNAME");
	  CORBA::ORB_var orb;
	  orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::
					instance ()->get_ORB ());

	  CORBA::Object_var system_obj =
	    orb->string_to_object (system_corbaloc.c_str());
 
	  system_test = CdmwPlatformMngt::System::_narrow (system_obj.in ());

	  state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
	  startup_mode = 0;
	  specified_state_id = "";

	  ent_def.entity_name = "NewEntity";
	  //defining Properties
	  val=10;
	  anyp_val <<= val;


	  startupKind = new CdmwPlatformMngtBase::StartupKind();

	  startupKind->state_retrieval = state_retrieval;
	  startupKind->startup_mode = startup_mode;
	  startupKind->specified_state_id
	    = CORBA::string_dup( specified_state_id.c_str() );

	  ///////////////////////////////////////////////////////////////////////////////////////////

	  // Recover Supervised Objects and store the default snapshot values 
	  // for System 
	  default_sys_snap = system_test -> get_snapshot();
	  sleep (3); 
	  // for Application
	  application_test = system_test->get_application("APPL2");
	  default_app_snap = application_test -> get_snapshot();
	  sleep (3);

	  // for Process
	  application_test = system_test->get_application("APPL2");
	  sleep(3);
          }
          catch (const CORBA::SystemException& ex) 
          {
              std::cerr<< ex._name()<<std::endl;
              throw;
          }
          catch(...)
          {
          }
    }

 
    
    void TestRestartObjs::test_restart_system() 
    {
      try
      { 
      system_test->define_property("Property",anyp_val);
      system_test->add_entity(ent_def);

      CdmwPlatformMngt::SystemSnapshot_var sys_hot_snap_before = system_test-> get_snapshot();

      system_test -> hot_restart(startupKind.in());
      
      sleep (7);
      CdmwPlatformMngt::SystemSnapshot_var sys_hot_snap = system_test-> get_snapshot();
      //////
      std::cerr << snapShotToString(sys_hot_snap)<< std::endl<< snapShotToString(sys_hot_snap_before)<<std::endl;
      CPPUNIT_ASSERT((snapShotToString(sys_hot_snap)==snapShotToString(sys_hot_snap_before)));
      sleep (7);

      system_test -> cold_restart(startupKind.in());
      sleep (7);
      /////
      CdmwPlatformMngt::SystemSnapshot_var sys_cold_snap = system_test-> get_snapshot();
      std::cerr << snapShotToString(sys_cold_snap)<<std::endl<<snapShotToString(default_sys_snap)<<std::endl;
      CPPUNIT_ASSERT((snapShotToString(sys_cold_snap)==snapShotToString(default_sys_snap)));
      }
      catch (const CORBA::SystemException& ex)
      {
          std::cerr<< ex._name()<<std::endl;
          throw;
      }
    }


    void TestRestartObjs::test_restart_application()
    {
      try
      {  
      application_test->define_property("Property",anyp_val);
      application_test->add_entity(ent_def);
      CdmwPlatformMngt::ApplicationSnapshot_var appl_snap_hot_before =  application_test-> get_snapshot();
      
      application_test-> hot_restart(startupKind.in());
      sleep (5);
      CdmwPlatformMngt::ApplicationSnapshot_var appl_snap_hot =  application_test-> get_snapshot();
      const char* indent ="";
      CPPUNIT_ASSERT (snapShotToString(appl_snap_hot,indent)==snapShotToString(appl_snap_hot_before,indent));
      ///////
      std::cerr << snapShotToString(appl_snap_hot,indent)<<std::endl<<snapShotToString(appl_snap_hot_before,indent)<< std::endl;
      application_test->cold_restart(startupKind.in());
      sleep(5);
      CdmwPlatformMngt::ApplicationSnapshot_var appl_snap_cold =  application_test-> get_snapshot();

      //////
      std::cerr <<snapShotToString(appl_snap_cold, indent)  <<std::endl<< snapShotToString(default_app_snap, indent) << std::endl;
      CPPUNIT_ASSERT(snapShotToString(appl_snap_cold, indent)==snapShotToString(default_app_snap, indent));

      }
      catch (const CORBA::SystemException& ex)
      {
          std::cerr<< ex._name()<<std::endl;
          throw;
      }

    }


    void TestRestartObjs::test_restart_process()
    {
      try
      { 
     
          CdmwPlatformMngt::Processes_var my_processes_from_app;
          CdmwPlatformMngt::ProcessesIterator_var process_iter;
          CORBA::ULong size_seq = 0;


          my_processes_from_app =
             application_test->get_processes (2, process_iter.out ());

          size_seq = my_processes_from_app->length ();

          for (CORBA::ULong i=0;i<size_seq;i++)
          {
             std::string proc_name =  my_processes_from_app[i]->name() ;
             if ( proc_name == "P21")
                 process_test =  my_processes_from_app[i];
          }

          default_proc_snap = process_test -> get_snapshot();

 
      process_test->define_property("Property",anyp_val);
      process_test->add_entity(ent_def);

      const char* indent ="";
      CdmwPlatformMngt::ProcessSnapshot_var proc_snap_hot_before =  process_test-> get_snapshot();
     
      process_test->hot_restart(startupKind.in());
      sleep(4);
      CdmwPlatformMngt::ProcessSnapshot_var proc_snap_hot =  process_test-> get_snapshot();
      CPPUNIT_ASSERT(snapShotToString(proc_snap_hot, indent)==snapShotToString(proc_snap_hot_before, indent));
      
      process_test->cold_restart(startupKind.in());
      sleep(4);
      CdmwPlatformMngt::ProcessSnapshot_var proc_snap_cold =  process_test-> get_snapshot();
      CPPUNIT_ASSERT (snapShotToString(proc_snap_cold, indent)==snapShotToString(default_proc_snap, indent));
      sleep(4);
  
      }
      catch (const CORBA::SystemException& ex)
      {
          std::cerr<< ex._name()<<std::endl;
          throw;
      }
      catch (...)
      {
      }

    }
   
  }				/* End namespace PlatformMngt */
}				/* End namespace Cdmw */





