/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */



#include "Foundation/common/Options.hpp"
#include <Foundation/common/Locations.hpp>
#include "Foundation/common/String.hpp"
#include "Foundation/common/Exception.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/common/Locations.hpp"

#include "Foundation/orbsupport/CORBA.hpp"

#include "Foundation/ossupport/OS.hpp"
#include "Foundation/osthreads/ThreadHandle.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include <Foundation/logging/LogManager.hpp> 
#include <Foundation/logging/FilterMngr.hpp>

#include "SystemMngt/platforminterface/PlatformInterface.hpp"
#include "SystemMngt/platforminterface/ServiceNames.hpp"

#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include "Repository/repositoryinterface/RepositoryInterface.hpp"

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"

#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/LogManager.hpp>

#include "test_tracing_api/MyProcessBehaviour.hpp"

#include <cstdlib>
#include <string>
#include <exception>
#include <sstream>
#include <iostream>
#include <fstream>
#define CDMW_TRACE_LEVEL 6

const int POA_THREAD_POOL_SIZE = 5;

using Cdmw::NamingAndRepository::RepositoryInterface;
using Cdmw::Logging::Logger;
using Cdmw::Logging::LogManager;

using namespace Cdmw::OsSupport;
using namespace Cdmw::OrbSupport;
using namespace Cdmw::PlatformMngt;

std::string applicationName;
std::string processName;
std::string hostName ;

int main(int argc, char* argv[])
{
    // the return code 
    StrategyList strategy;
    strategy.add_OrbThreaded();
    strategy.add_PoaThreadPool(POA_THREAD_POOL_SIZE);

    CORBA::ORB_var orb;


    try
    {
                
        // initialise the orb    
        orb =  Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategy);

        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");

        PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());

        PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();

        poaMgr->activate();

        std::cout << "Init PlatformInterface" << std::endl;

        // initialise the platform interface
        PlatformInterface::Setup(orb.in(), argc, argv);

        // creates the process behaviour
        std::auto_ptr<MyProcessBehaviour> procBehaviour( new MyProcessBehaviour(orb.in() ));
        // get processId
        OS::ProcessId processId = OS::get_processId();

        // acknowledge the creation of the process
        PlatformInterface::Acknowledge_creation(procBehaviour.get());

        MyProcessBehaviour * pProcBehaviour = procBehaviour.release();

        applicationName = PlatformInterface::Get_application_name();

        processName = PlatformInterface::Get_process_name();


        std::cout << "Application name : "
                      << applicationName.c_str() << std::endl;

        std::cout << "Process name : "
                      << processName.c_str()
                      << " Process id : "
                      << processId << std::endl;


        //resolve system ptr
        CORBA::Object_var objref2; 

        /*define the corbaloc */
        const char *corbaloc_str = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";

        objref2 = orb->string_to_object (corbaloc_str);

        /* resolve the System */
        CdmwPlatformMngt::System_var mysystem = CdmwPlatformMngt::System::_narrow (objref2.in ());

        if (CORBA::is_nil (mysystem.in ()))
          {
            std::cout<<"Error in narrowing CdmwPlatformMngt::System"<<std::endl;
            exit (-1);
          }
 
        CdmwPlatformMngt::Application_var app = mysystem->get_application ("HELLO_WORLD");

        std::string proc_name = "TraceClient";
        char *hostname = getenv ("HOSTNAME");
        CdmwPlatformMngt::Process_var proc = app->get_process (proc_name.c_str (), hostname);
        if (CORBA::is_nil (proc.in ())) 
          {
           std::cout<<"*******ERROR in proc_name, is nil *************"<<std::endl;
           exit (-1);
          }

        if (CORBA::is_nil (proc.in ()))
          {
           std::cout<<"*******ERROR in proc_name, is nil *************"<<std::endl;
           exit (-1);
          }

        //try a getting of filters
        std::string init_filters = proc->get_trace_filtering() ;
        if ( init_filters.compare("TraceClient,HELLO_WORLD,7/") == 0)
	  {
          std::cout<<"TEST NUM. 1 -->  OK: scenario filters ="<<init_filters<<std::endl;
	  }
        else
	  {
          std::cout<<"TEST NUM. 1 -->  Error: scenario filters ="<<init_filters<<std::endl;
	  }

        //try a parsing exception
        char flag_test2=0;

	try
	  {
          proc->set_trace_filtering(",,3");        
	  }
        catch ( CdmwPlatformMngt::ParsingException& ex )
	  {
          std::cout<<"TEST NUM. 2 -->  OK: reached a CdmwPlatformMngt::ParsingException "<<std::endl;
	  flag_test2 = 1 ;
	  }

        if ( flag_test2 == 0)
          std::cout<<"TEST NUM. 2 -->  Error : not reached a CdmwPlatformMngt::ParsingException "<<std::endl;
	  

        //retrieve an unmanaged process reference to test
	//if CastException works well ...
        char flag_test3=0;

        std::string proc_name_unmanaged = "HelloProcess1";
        CdmwPlatformMngt::Process_var proc_unmanaged = app->get_process (proc_name_unmanaged.c_str (), hostname);

        try
	  {
	  proc_unmanaged->set_trace_filtering("HelloProcess1,HELLO_WORLD,2");
	  }
        catch ( CdmwPlatformMngt::CastException& ex )
	  {
          std::cout<<"TEST NUM. 3 -->  OK: reached a CdmwPlatformMngt::CastException "<<std::endl;
	  flag_test3 = 1 ;
	  }

        if ( flag_test3 == 0)
          std::cout<<"TEST NUM. 3 -->  Error : not reached a CdmwPlatformMngt::CastException "<<std::endl;

        //test: adding a setting of filters ...
        char flag_test4 = 0;

        try
	 {
         proc->set_trace_filtering("TraceClient,HELLO_WORLD,9/TraceClient,HELLO_WORLD,12/");
	 }
        catch (...)
	 {
         std::cout<<"TEST NUM. 4 -->  Error : Failed to set trace filters to managed process"<<std::endl;
        
	 flag_test4=1;
	 }

        if ( flag_test4 == 0)
          std::cout<<"TEST NUM. 4 -->  OK : setting of other trace filters to the managed process"<<std::endl;

        //test: getting of these new filters ...
        std::string new_filters = proc->get_trace_filtering() ;

        if ( new_filters.compare("TraceClient,HELLO_WORLD,7/TraceClient,HELLO_WORLD,9/TraceClient,HELLO_WORLD,12/") == 0)
	  {
          std::cout<<"TEST NUM. 5 -->  OK : getting of actual (in managed process) filters="<<new_filters<<std::endl;
	  }
        else
	  {
          std::cout<<"TEST NUM. 5 -->  Error : getting of actual (in managed process) filters="<<new_filters<<std::endl;
	  }

        //test: setting of filters at host level ...
        char flag_test6 = 0;

        CdmwPlatformMngt::Host_var myhost = mysystem->get_host (hostname);

        try
	  {
	  myhost->set_trace_filtering("TraceClient,HELLO_WORLD,14/");
          }
        catch(...)
	  {
          std::cout<<"TEST NUM. 6 --> Error : setting of other trace filters at host level"<<std::endl;
	  flag_test6 = 1 ;
	  }

        if ( flag_test6 == 0)
	  {
          std::cout<<"TEST NUM. 6 --> OK : setting of other trace filters at host level"<<std::endl;
	  }

        //test: getting of filters at host level ...

        CdmwPlatformMngt::Proc_filters_var host_filters ;
        host_filters = myhost->get_trace_filtering();
	CORBA::ULong size_seq_filt = host_filters->length() ;
	char flag_test7=0;

	if ( size_seq_filt != 1)
	  {
          std::cout<<"TEST NUM. 7 --> Error : getting of other trace filters at host level, sequence size is wrong"<<std::endl;
    
	  flag_test7=1;
	  }
	  

        std::string to_compare = "Application=\"HELLO_WORLD\"; Process=\"TraceClient\"; Filters=\"TraceClient,HELLO_WORLD,7/TraceClient,HELLO_WORLD,9/TraceClient,HELLO_WORLD,12/TraceClient,HELLO_WORLD,14/\""; 
	std::string myhost_filters = CORBA::string_dup ( (*host_filters)[0] );

         if ( ( myhost_filters.compare(to_compare) == 0) && ( flag_test7 == 0) )
	  {
          std::cout<<"TEST NUM. 7 --> OK : getting of other trace filters at host level; Filters: "<<to_compare<<std::endl;
	  }
        else
	  {
          std::cout<<"TEST NUM. 7 --> Error : getting of other trace filters at host level; Filters: "<<to_compare<<std::endl;
	  }


        // advise behavior that init is done
        pProcBehaviour->set_init_done();
        pProcBehaviour->get_condition().broadcast();

        // start orb
        orb->run();

        // cleanup the platform interface
        PlatformInterface::Cleanup();


       }
       catch(...)
        {
	    std::cout<<"*****ERROR : reached exception in managed process *****"<<std::endl;
            // cleanup the ORB to exit completely
            orb->shutdown(false);
                    Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
                    orb->destroy();
            
	    return(-1);
        }



    return 0;

}

