/* ========================================================================== *
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati.
 * Copyright (C) SELEX Sistemi Integrati 2004-2006. All rights reserved.
 *  
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *  
 *  
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ========================================================================= */
 
#include "clockservice/FTController_impl.hpp"
#include "clockservice/Executor_impl.hpp"

Cdmw::clock::FTController_impl::FTController_impl(Cdmw::clock::Ident                                controllerId,
                                                  Cdmw::clock::ControllerDataStorageHome&           controllerDataStorageHome,
                                                  Cdmw::clock::Executor_impl&                       clock,
                                                  CosClockService::PeriodicExecution::Periodic_ptr  periodic,
                                                  ACE_Reactor*                                      reactor)
    : Cdmw::clock::Controller_impl(clock, periodic, reactor),
      m_controller_id(controllerId),
      m_controller_data_storage_home(controllerDataStorageHome)
{
    // No-Op
}

Cdmw::clock::FTController_impl::~FTController_impl()
{
    // No-Op
}

void
Cdmw::clock::FTController_impl::activate()
{
    std::cout << "############################################################" << std::endl;
    std::cout << ">>>>>>>>>>>>>>>>>> Activating FTController Replica"           << std::endl;

    try
    {
        Cdmw::clock::ControllerData_var data_var;
        Cdmw::clock::FTController_impl::retrieve_status_lock_free(m_controller_id,
                                                                  m_controller_data_storage_home,
                                                                  data_var);
        this->restore_object_lock_free(data_var.in());
    }
    catch (const Cdmw::FT::NotFoundException& e)
    {
        std::cerr << "FTController_impl::activate: ERROR: storage object data not found" << std::endl;
    }

    std::cout << ">>>>>>>>>>>>>>>>>> Completed FTController Replica Activation" << std::endl;
    std::cout << "############################################################" << std::endl;
}

void
Cdmw::clock::FTController_impl::notify_status_lock_free()
{
    // Counter on the number of Periodic::do_work() successful invocations.
    static long executions_counter = 0;

    try
    {
        Cdmw::clock::ControllerData data;
        this->refresh_data_lock_free(data);
        ControllerDataStorageObject ds_obj = m_controller_data_storage_home.find_by_oid(this->m_controller_id);
        ds_obj.set(data);

        // Notify to user every change in the number of Periodic::do_work() successful invocations.
        if (executions_counter != (long) data.timer_info.controller_executions)
        {
            executions_counter = data.timer_info.controller_executions;
            std::cout << "FTController_impl::notify_status_lock_free: current executions counter: " << executions_counter << std::endl; 
        }
    }
    catch (const Cdmw::FT::NotFoundException& e)
    {
        std::cerr << "FTController_impl::notify_status_lock_free: unable to store timer data" << std::endl;
    }
}

void
Cdmw::clock::FTController_impl::refresh_data_lock_free(Cdmw::clock::ControllerData& data)
{
    data.controller_id = this->m_controller_id;
    Cdmw::clock::Controller_impl::refresh_data_lock_free(data);
}

void
Cdmw::clock::FTController_impl::retrieve_status_lock_free(Cdmw::clock::Ident                      oid,
                                                          Cdmw::clock::ControllerDataStorageHome& cdsh,
                                                          Cdmw::clock::ControllerData_var&        data_var)
    throw (Cdmw::FT::NotFoundException)
{
    data_var = cdsh.find_data_by_oid(oid);
}
