/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
 *
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */


package cdmw.clock;

import cdmw.ossupport.OS;
import cdmw.ossupport.ThreadedProcess;
import org.omg.CORBA.ORB;
import org.omg.CosClockService.ClockCatalog;
import org.omg.CosClockService.ClockCatalogHelper;
import org.omg.CosClockService.TimeSpanValueFactory;
import org.omg.CosClockService.UTCValueFactory;

/**
 * @brief implementation for ClockService Java Wrapper
 *
 * @author Francesca Annunziata <fannunziata@selex-si.com>
 * @author Raffaele Mele <rmele@progesi.it>
 */

public class ClockServiceJavaWrapper {
    private static final String LOCALISATION_SERVICE = " --CdmwLocalisationService=";
    private static final String LOCALCLOCK_CORBALOC = " -ORBInitRef LocalClock=corbaloc::localhost:";
    private static final String LOC_SERV_PORT = "4883";
    private static final String LOCALCLOCK = "LocalClock";
    private static final String CLOCKSERVICE_CORBALOC = " -ORBInitRef ClockService=corbaloc::localhost:";
    private static final String CLOCKSERVICE = "ClockService";
    public static final int MAX_START_RETRY = 10;

    private static cdmw.ossupport.ThreadedProcess proc = null;

    // retry for ClockService process starting.

    public static void startClockService(ORB orb)
    {

        try
        {
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory("IDL:omg.org/CosClockService/UTC:1.0",
                    new UTCValueFactoryImpl());
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory("IDL:omg.org/CosClockService/TimeSpan:1.0",
                    new TimeSpanValueFactoryImpl());

            String clockServiceArgs = "--server"            +
                                      LOCALISATION_SERVICE  +
                                      LOC_SERV_PORT         +
                                      LOCALCLOCK_CORBALOC   +
                                      LOC_SERV_PORT + "/"   +
                                      LOCALCLOCK            +
                                      CLOCKSERVICE_CORBALOC +
                                      LOC_SERV_PORT + "/"   +
                                      CLOCKSERVICE;
            proc = OS.createProcess("cdmw_clockservice_standalone " + clockServiceArgs);
            boolean isStarted = false;
            int startRetry = 0;
            while (!isStarted && (startRetry < MAX_START_RETRY))
            {
                try
                {
                    startRetry++;
                    org.omg.CORBA.Object obj  = orb.resolve_initial_references(CLOCKSERVICE);
                    ClockCatalog catalog = ClockCatalogHelper.narrow(obj);
                    if (catalog != null)
                        isStarted = true;
                }
                catch (org.omg.CORBA.TRANSIENT ex)
                {
                    // ClockService isn't up!!
                }
                catch (org.omg.CORBA.ORBPackage.InvalidName ex)
                {
                    ex.printStackTrace();
                }
            }
            if (!isStarted)
                System.err.println("ClockService Wrapper:>> Panic -- extern process don't start.");
        }
        catch (java.io.IOException ex)
        {
            ex.printStackTrace();
        }
    }
    public static void stopClockService ()
    {
        if (proc != null) proc.killProcess();
    }

}
