/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
 *
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */

/**
 * @brief
 *
 * @author Francesca Annunziata <fannunziata@selex-si.com>
 * @author Raffaele Mele <rmele@progesi.it>
 */

package cdmw.clock.test;

import cdmw.clock.*;
import cdmw.clock.Util;
import org.omg.CORBA.*;
import cdmw.ossupport.OS;
import Cdmw.clock.ControlledClockEx;
import Cdmw.clock.ControlledClockExHelper;
import cdmw.testutils.Testable;
import org.omg.CosClockService.*;
import org.omg.CosClockService.ClockHelper;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntriesHelper;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntry;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntryHolder;
import org.omg.CosClockService.ClockCatalogPackage.UnknownEntry;
import java.lang.Math;

public class ControlledClockTest extends Testable
{

    private final int NUMOFTESTS = 11;
    private final int rate = 10;
    private final long TIMET_IN_A_SEC = 10000000L;
    private final long EPSILON = TIMET_IN_A_SEC/10;

    private ControlledClockEx m_controlledClock;
    private Clock m_clock;
    private ClockCatalog m_ctg;

///////////////////////////////////////////////////////////////////////////////

    public ControlledClockTest ( ORB orb, ClockCatalog ctg )
    {
        name = "ControlledClockTest";
        out = System.out;
        m_ctg = ctg;

        try
        {
            org.omg.CORBA.Object obj = orb.resolve_initial_references("LocalClock");
            m_clock = org.omg.CosClockService.ClockHelper.narrow(obj);
            if (m_clock == null)
            {
                System.out.println("LocalClock: nil reference.");
                System.exit(1);
            }
            ClockEntry entry = m_ctg.get_entry("ControlledClock");
            m_controlledClock = ControlledClockExHelper.narrow(entry.subject);
            if (m_controlledClock == null)
            {
                System.out.println("ControlledClock: nil reference.");
                System.exit(1);
            }
        }
        catch ( UnknownEntry ex )
        {
            System.out.println( "ClockCatalog::UnknownEntry" );
        }
        catch ( org.omg.CORBA.ORBPackage.InvalidName ex )
        {
            ex.printStackTrace();
        }

    }

///////////////////////////////////////////////////////////////////////////////

    public boolean set_test(long to)
    {
        try
        {
            // check for null reference
            if (m_controlledClock == null)
            {
                System.out.println("ERROR: null reference (m_controlledClock)");
                return Macro.TEST_NOK;
            }
            m_controlledClock.set(to);
        }
        catch (org.omg.CosClockService.ControlledClockPackage.NotSupported e)
        {
            e.printStackTrace();
        }
        catch (org.omg.CORBA.NO_IMPLEMENT ni)
        {
            System.out.println("Warnig (NO_IMPLEMENT): m_controlledClock.set " +
                               "operation is avaiable only at API level");
            return Macro.TEST_OK;
        }
        return Macro.TEST_OK;
    }

///////////////////////////////////////////////////////////////////////////////

    public boolean set_rate_test(float ratio)
    {
        try
        {
            // check for null reference
            if (m_controlledClock == null)
            {
                System.out.println("ERROR: null reference (m_controlledClock)");
                return Macro.TEST_NOK;
            }
            m_controlledClock.set_rate(ratio);
        }
        catch (org.omg.CosClockService.ControlledClockPackage.NotSupported e)
        {
            e.printStackTrace();
        }
        catch (org.omg.CORBA.NO_IMPLEMENT ni)
        {
            System.out.println("Warnig (NO_IMPLEMENT): m_controlledClock.set " +
                               "operation is avaiable only at API level");
        }
        return Macro.TEST_OK;
    }

///////////////////////////////////////////////////////////////////////////////

    public boolean pause_test()
    {
        try
        {
            // check for null reference
            if (m_controlledClock == null)
            {
                System.out.println("ERROR: null reference (m_controlledClock)");
                return Macro.TEST_NOK;
            }
            m_controlledClock.pause();
        }
        catch (org.omg.CosClockService.ControlledClockPackage.NotSupported e)
        {
            e.printStackTrace();
        }
        catch (org.omg.CORBA.NO_IMPLEMENT ni)
        {
            System.out.println("Warnig (NO_IMPLEMENT): m_controlledClock.set " +
                               "operation is avaiable only at API level");
            return Macro.TEST_OK;
        }
        return Macro.TEST_OK;
    }

///////////////////////////////////////////////////////////////////////////////

    public boolean resume_test()
    {
        try
        {
            // check for null reference
            if (m_controlledClock == null)
            {
                System.out.println("ERROR: null reference (m_controlledClock)");
                return Macro.TEST_NOK;
            }
            m_controlledClock.resume();
        }
        catch (org.omg.CosClockService.ControlledClockPackage.NotSupported e)
        {
            e.printStackTrace();
        }
        catch (org.omg.CORBA.NO_IMPLEMENT ni)
        {
            System.out.println("Warnig (NO_IMPLEMENT): m_controlledClock.set " +
                               "operation is avaiable only at API level");
            return Macro.TEST_OK;
        }
        return Macro.TEST_OK;
    }

///////////////////////////////////////////////////////////////////////////////

    public boolean terminate_test()
    {
        try
        {
            // check for null reference
            if (m_controlledClock == null)
            {
                System.out.println("ERROR: null reference (m_controlledClock)");
                return Macro.TEST_NOK;
            }
            m_controlledClock.terminate();
        }
        catch (org.omg.CosClockService.ControlledClockPackage.NotSupported e)
        {
            e.printStackTrace();
        }
        catch (org.omg.CORBA.NO_IMPLEMENT ni)
        {
            System.out.println("Warnig (NO_IMPLEMENT): m_controlledClock.set " +
                               "operation is avaiable only at API level");
            return Macro.TEST_OK;
        }
        return Macro.TEST_OK;
    }

///////////////////////////////////////////////////////////////////////////////

    public boolean get_rate_test()
    {
        try
        {
            float ratio = m_controlledClock.get_rate();
        }
        catch (org.omg.CosClockService.ControlledClockPackage.NotSupported e)
        {
            e.printStackTrace();
        }
        return Macro.TEST_OK;
    }

///////////////////////////////////////////////////////////////////////////////

    public boolean attach_test()
    {
        try
        {
            dummy("attach()");
        }
        catch (org.omg.CORBA.NO_IMPLEMENT ni)
        {
            return Macro.TEST_OK;
        }
        return Macro.TEST_OK;
    }

///////////////////////////////////////////////////////////////////////////////

    public boolean detach_test()
    {
        try
        {
            dummy("detach()");
        }
        catch (org.omg.CORBA.NO_IMPLEMENT ni)
        {
            return Macro.TEST_OK;
        }

        return Macro.TEST_OK;
    }

///////////////////////////////////////////////////////////////////////////////

    public boolean get_time_origins_test()
    {
        org.omg.CORBA.LongHolder dummy1 = new org.omg.CORBA.LongHolder();
        org.omg.CORBA.LongHolder dummy2 = new org.omg.CORBA.LongHolder();

        try
        {
            m_controlledClock.get_time_origins(dummy1,dummy2);
        }
        catch (org.omg.CosClockService.ControlledClockPackage.NotSupported e)
        {
            e.printStackTrace();
        }
        return Macro.TEST_OK;
    }

///////////////////////////////////////////////////////////////////////////////

    public boolean real2controlled_test()
    {
        org.omg.CORBA.LongHolder virt_time = new org.omg.CORBA.LongHolder();
        org.omg.CORBA.LongHolder realTime0 = new org.omg.CORBA.LongHolder();
        org.omg.CORBA.LongHolder virtualTime0 = new org.omg.CORBA.LongHolder();
        org.omg.CORBA.LongHolder time_val = new org.omg.CORBA.LongHolder();

        // check for null reference
        if (m_controlledClock == null)
        {
            System.out.println("ERROR: null reference (m_controlledClock)");
            return Macro.TEST_NOK;
        }
        try {
            m_controlledClock.set_rate(rate);
            m_controlledClock.set(m_clock.current_time());

            // wait one second to retrieve a new time
            OS.sleep(1000);

            // retrieve actual virtual time: V(t0+1s)
            virt_time.value = m_controlledClock.current_time();

            // retrieve to,V(t0) informations
            m_controlledClock.get_time_origins(realTime0,
                                               virtualTime0);
            // compute reference time
            //long
            time_val.value = realTime0.value + TIMET_IN_A_SEC;

            // retrieve one second in the future
            org.omg.CORBA.LongHolder
            computed_time = Util.real2controlled(m_controlledClock, time_val.value);

            long
            delta_diff = delta(virtualTime0.value,
                               computed_time.value);

            check_doubles_equal(delta_diff/rate,
                                TIMET_IN_A_SEC,
                                EPSILON);

            // the same test in the past
            m_controlledClock.set_rate(-rate);
            m_controlledClock.set(m_clock.current_time());
            OS.sleep(1000);
            virt_time.value = m_controlledClock.current_time();

            m_controlledClock.get_time_origins(realTime0,
                                               virtualTime0);
            time_val.value = realTime0.value + TIMET_IN_A_SEC;

            // retrieve one second in the past
            computed_time = Util.real2controlled(m_controlledClock,
                                                 time_val.value);

            delta_diff = delta(virtualTime0.value,
                               computed_time.value);

            check_doubles_equal(delta_diff/rate,
                                TIMET_IN_A_SEC,
                                EPSILON);
        }
        catch (org.omg.CosClockService.ControlledClockPackage.NotSupported ex)
        {
            ex.printStackTrace();
            return (Macro.TEST_NOK);
        }
        catch (org.omg.CosClockService.TimeUnavailable ex)
        {
            ex.printStackTrace();
            return (Macro.TEST_NOK);
        }
        catch (java.lang.InterruptedException ex)
        {
            ex.printStackTrace();
            return (Macro.TEST_NOK);
        }
        return Macro.TEST_OK;
    }

///////////////////////////////////////////////////////////////////////////////

    public boolean controlled2real_test()
    {
        org.omg.CORBA.LongHolder virt_time = new org.omg.CORBA.LongHolder();
        org.omg.CORBA.LongHolder realTime0 = new org.omg.CORBA.LongHolder();
        org.omg.CORBA.LongHolder virtualTime0 = new org.omg.CORBA.LongHolder();
        org.omg.CORBA.LongHolder time_val = new org.omg.CORBA.LongHolder();

        // check for null reference
        if (m_controlledClock == null)
        {
            System.out.println("ERROR: null reference (m_controlledClock)");
            return Macro.TEST_NOK;
        }

        try {

            ClockEntry entry = m_ctg.get_entry("ControlledClock");
            ControlledClockEx extControlledClock = 
                ControlledClockExHelper.narrow(entry.subject);
            if (extControlledClock == null)
            {
                System.out.println("ControlledClock: nil reference.");
                return Macro.TEST_NOK;
            }

            extControlledClock.set_rate(rate);
            extControlledClock.set(m_clock.current_time());

            // wait one second to retrieve a new time
            OS.sleep(1000);

            // retrieve actual virtual time: V(t0+1s)
            virt_time.value = extControlledClock.current_time();

            // retrieve to,V(t0) informations
            extControlledClock.get_time_origins (realTime0,
                                                 virtualTime0);
            // compute current time
            time_val.value = realTime0.value + TIMET_IN_A_SEC;

            // retrieve one second in the future
            long computed_time = Util.controlled2real(extControlledClock, virt_time.value);
            long delta_diff = delta(realTime0.value, computed_time);

            // here time is real
            check_doubles_equal(delta_diff, TIMET_IN_A_SEC, EPSILON);
        }
        catch (org.omg.CosClockService.ControlledClockPackage.NotSupported ex)
        {
            ex.printStackTrace();
            return (Macro.TEST_NOK);
        }
        catch (org.omg.CosClockService.TimeUnavailable ex)
        {
            ex.printStackTrace();
            return (Macro.TEST_NOK);
        }
        catch (org.omg.CosClockService.ClockCatalogPackage.UnknownEntry ex)
        {
            ex.printStackTrace();
            return (Macro.TEST_NOK);
        }
        catch (java.lang.InterruptedException ex)
        {
            ex.printStackTrace();
            return (Macro.TEST_NOK);
        }
        return Macro.TEST_OK;
    } 

///////////////////////////////////////////////////////////////////////////////

    private long
    delta(long num1,
          long num2)
    {
        if (num1 > num2)
            return num1-num2;
        return num2-num1;
    }

///////////////////////////////////////////////////////////////////////////////

    private int
    delta_sign(long num1,
               long num2)
    {
        if (num1 >= num2)
            return 1;
        return -1;
    }

///////////////////////////////////////////////////////////////////////////////

    private
    boolean check_doubles_equal ( long expected, long actual, long delta)
    {
        return Math.abs( expected - actual ) > delta;
    }

///////////////////////////////////////////////////////////////////////////////

    private void dummy (String name)
    {
        System.out.println("Warning (NO_IMPLEMENT): " + name +
                           " is avaliable only at API level");
        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

///////////////////////////////////////////////////////////////////////////////

    public void doTests()
    {
        setNbOfRequestedTestOK(NUMOFTESTS);

        long to = 1;
        check(set_test(to));
        float ratio = 10;
        check(attach_test());
        check(detach_test());
        check(set_rate_test(ratio));
        check(pause_test());
        check(resume_test());
        check(get_rate_test());
        check(get_time_origins_test());
        check(real2controlled_test());
        check(controlled2real_test());
        check(terminate_test());
    }
}

///////////////////////////////////////////////////////////////////////////////
