/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
 *
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */

package cdmw.clock.test;


import cdmw.clock.*;
import Cdmw.clock.ControlledExecutor;
import Cdmw.clock.ControlledExecutorHelper;
import Cdmw.clock.ControllerEx;
import Cdmw.clock.ControllerExHelper;
import cdmw.clock.Util;
import cdmw.ossupport.OS;
import cdmw.testutils.Testable;
import java.lang.Math;
import org.omg.CORBA.*;
import org.omg.CosClockService.*;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntriesHelper;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntry;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntryHolder;
import org.omg.CosClockService.ClockCatalogPackage.UnknownEntry;
import org.omg.CosClockService.ClockHelper;
import org.omg.CosClockService.PeriodicExecution.ControllerPackage.TimePast;
import org.omg.CosClockService.PeriodicExecution.Controller;

public class ControllerTest extends Testable
{

    private final int NUMOFTESTS = 6;

    private ControlledExecutor m_executor;
    private Clock m_clock;
    private ClockCatalog m_ctg;
    private org.omg.CosClockService.PeriodicExecution.Controller m_controller;
//    private Cdmw.clock.ControllerEx m_controller;
    private org.omg.CosClockService.PeriodicExecution.Periodic p;
    private MyPeriodicExecution periodic;
    private ORB m_orb;

    public ControllerTest (ORB orb, ClockCatalog ctg)
    {
        name = "ControllerTest";
        out = System.out;
        m_ctg = ctg;
        m_orb = orb;
        try
        {
            ClockEntry entry = m_ctg.get_entry("ControlledExecutor");
            m_executor = ControlledExecutorHelper.narrow(entry.subject);
            if (m_executor == null)
            {
                out.println("ControlledExecutor: nil reference.");
                System.exit(1);
            }
            periodic = new MyPeriodicExecution();
            if (periodic == null)
            {
                out.println("MyPeriodicExecution: nil reference.");
                System.exit(1);
            }
            p = periodic._this(m_orb);
            if (p == null)
            {
                out.println("my_periodic: nil reference.");
                System.exit(1);
            }
            m_controller = (org.omg.CosClockService.PeriodicExecution.Controller)m_executor.enable_periodic_execution(p);
//            m_controller = (Cdmw.clock.ControllerEx)m_executor.enable_periodic_execution(p);
            if (m_controller == null)
            {
                out.println("m_controller: nil reference.");
                System.exit(1);
            }
        }
        catch ( UnknownEntry ex )
        {
            out.println( "ClockCatalog::UnknownEntry" );
        }
//         catch ( org.omg.CORBA.ORBPackage.InvalidName ex )
//         {
//             ex.printStackTrace();
//         }
    }

    public boolean start_test(long period, long with_offset, int execution_limit, org.omg.CORBA.Any params)
    {
        try
        {
            m_controller.start(period,with_offset,execution_limit,params);
        }
        catch (org.omg.CORBA.NO_IMPLEMENT ni)
        {
            out.println("Warning (NO_IMPLEMENT): start operation is avaiable only at API level");
            return Macro.TEST_OK;
        }

        return Macro.TEST_OK;
    }
    public boolean start_at_test(long period, long at_time, int execution_limit, org.omg.CORBA.Any params)
    {
        try
        {
            m_controller.start_at(period,at_time,execution_limit,params);
        }
        catch (org.omg.CosClockService.PeriodicExecution.ControllerPackage.TimePast tp)
        {
        }
        catch (org.omg.CORBA.NO_IMPLEMENT ni)
        {
            out.println("Warning (NO_IMPLEMENT): start_at operation is avaiable only at API level");
            return Macro.TEST_OK;
        }

        return Macro.TEST_OK;
    }

    public boolean pause_and_resume_test()
    {
        try
        {
            m_controller.pause();
            OS.sleep(1000);
            m_controller.resume();
        }
        catch (org.omg.CORBA.NO_IMPLEMENT ni)
        {
            out.println("Warning (NO_IMPLEMENT): pause and resume operations are avaiable only at API level");
            return Macro.TEST_OK;
        }
        catch (java.lang.InterruptedException ex)
        {
            ex.printStackTrace();
            return Macro.TEST_OK;
        }

        return Macro.TEST_OK;
    }
    public boolean pause_and_resume_at_test(long at_time)
    {
        try
        {
            m_controller.pause();
            m_controller.resume_at(at_time);
        }
        catch (org.omg.CosClockService.PeriodicExecution.ControllerPackage.TimePast tp)
        {
        }
        catch (org.omg.CORBA.NO_IMPLEMENT ni)
        {
            out.println("Warning (NO_IMPLEMENT): pause and resume_at operations are avaiable only at API level");
            return Macro.TEST_OK;
        }

        return Macro.TEST_OK;
    }

    public boolean terminate_test()
    {
        try
        {
            m_controller.terminate();
        }
        catch (org.omg.CORBA.NO_IMPLEMENT ni)
        {
            out.println("Warning (NO_IMPLEMENT): terminate operation is avaiable only at API level");
            return Macro.TEST_OK;
        }

        return Macro.TEST_OK;
    }
    public boolean executions_test()
    {
        try
        {
            m_controller.executions();
        }
        catch (org.omg.CORBA.NO_IMPLEMENT ni)
        {
            out.println("Warning (NO_IMPLEMENT): execution() operation is avaiable only at API level");
            return Macro.TEST_OK;
        }

        return Macro.TEST_OK;

    }
    public void doTests()
    {
        try {
            setNbOfRequestedTestOK(NUMOFTESTS);
            long period = 100000;
            long with_offset = 0;
            long at_time = m_executor.current_time()+100000;
            int execution_limit = 100;
            org.omg.CORBA.Any params = m_orb.create_any();

            params.insert_long(1234566789);
            check(start_test(period,with_offset,execution_limit,params));
            check(pause_and_resume_test());
            check(start_at_test(period,at_time,execution_limit,params));
            check(pause_and_resume_at_test(at_time));
            check(terminate_test());
            check(executions_test());
        }
        catch (org.omg.CosClockService.TimeUnavailable ex)
        {
            ex.printStackTrace();
        }
    }
}

