/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
 *
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */

package cdmw.clock.test;
import cdmw.testutils.Testable;
import cdmw.ossupport.Timeval;
import cdmw.clock.*;
import cdmw.ossupport.OS;
import java.text.SimpleDateFormat;
import java.text.ParseException;

import java.util.Date;
import java.util.Calendar;
import java.util.GregorianCalendar;


public class ImportExportTimevalTest extends Testable
{
    private final int       NUMOFTESTS = 4;


    private final String dateSample    = "Fri Sep 08 18:07:41 CEST 2006";
    private final long dateInTimeT     = 133770244610000000L;

    public ImportExportTimevalTest()
    {
      this.name = "ImportExportTimevalTest";
      this.out = System.out;
    }

    public boolean checkTimevalToTimeT(Timeval dateInTimeval) 
    {

         long utcTime = Time.timeval2TimeT(dateInTimeval);
         return ( (long) utcTime == dateInTimeT ) ;
    }
    public boolean checkTimeTtoTimeval(Timeval dateInTimeval) 
    {
         Timeval tv = Time.TimeT2timeval(dateInTimeT); 
         return (Util.compare(tv,dateInTimeval) == 0);
    }

    public boolean createUtcT()
    {
        java.util.Calendar cal = java.util.Calendar.getInstance();
        int offset =
            cal.get( java.util.Calendar.ZONE_OFFSET ) +  // zone difference
            cal.get( java.util.Calendar.DST_OFFSET );    // day-light-saving

        long inaccuracy = 10000L;
        short tdf = ( short ) ( offset / 60000 );  // millis to minutes
        short inacchi = (short) ((inaccuracy >> 32) &  0xffffL);
        int inacclo = (int) (inaccuracy & 0xffffffffL);

        org.omg.TimeBase.UtcT utct = Time.createUtcT(dateInTimeT);
        if ( (utct.time == dateInTimeT) &&
             (utct.inacclo == inacclo) &&
             (utct.inacchi == inacchi) &&
             (utct.tdf == tdf) )
            return Macro.TEST_OK;
        return Macro.TEST_NOK;
    }

    public boolean toStringTest()
    {
        final String date = "2006-09-08 18:07:41.00+0200";
        org.omg.TimeBase.UtcT utct = Time.createUtcT(dateInTimeT);
        if (date.compareTo(Time.toString(utct)) == 0)
            return Macro.TEST_OK;
        return  Macro.TEST_NOK;
    }

    public void doTests()
    {
       setNbOfRequestedTestOK(NUMOFTESTS);
      try 
      {
        SimpleDateFormat df = new SimpleDateFormat("EEE MMM dd HH:mm:ss zzz yyyy");

        // Fri Sep 08 18:07:41 CEST 2006
        // TimeT = 133770244610300000L
        Date d = df.parse(dateSample) ;

        Timeval dateInTimeval = new Timeval(d.getTime());
        check(checkTimevalToTimeT (dateInTimeval));
        check(checkTimeTtoTimeval (dateInTimeval));
        check(createUtcT());
        check(toStringTest());
      }
      catch (ParseException pe)
      {
      }

    }

}
