/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
 *
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */

/**
 * @brief
 *
 * @author Francesca Annunziata <fannunziata@selex-si.com>
 * @author Raffaele Mele <rmele@progesi.it>
 */

package cdmw.clock.test;

import cdmw.ossupport.OS;
import cdmw.ossupport.ThreadedProcess;

import cdmw.orbsupport.ORBSupport;
import cdmw.orbsupport.ORBUtils;

import java.io.File;
import java.io.IOException;
import java.util.*;

import org.omg.CORBA.ORB;
import org.omg.CORBA.ORBPackage.InvalidName;

import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;
import org.omg.PortableServer.POAManager;
import org.omg.PortableServer.POAManagerPackage.AdapterInactive;

import org.omg.CosPropertyService.PropertySetDefFactory;
import org.omg.CosPropertyService.PropertySetDefFactoryHelper;

import org.omg.CosClockService.ClockCatalog;
import org.omg.CosClockService.ClockCatalogHelper;

import cdmw.testutils.Testable;

///////////////////////////////////////////////////////////////////////////////

public class MainTest
{
    public final static int SUCCESS = 0;
    public final static int FAILURE = 1;

///////////////////////////////////////////////////////////////////////////////

    public static int run(ORB orb, String testname, String[] args)
    throws AdapterInactive,InvalidName
    {
        //
        // Resolve the Root POA
        //
        org.omg.CORBA.Object obj = orb.resolve_initial_references("RootPOA");
        POA root = POAHelper.narrow(obj);
        POAManager poaManager = root.the_POAManager();

        //
        // Initialize the server so that incoming requests are
        // handled from this point on
        //
        poaManager.activate();

        //Testing Clock
        runTest(orb, testname, args);

        return SUCCESS;

    }

///////////////////////////////////////////////////////////////////////////////

    public static void runTest(ORB orb,String testname, String[] strArgs)
    {
        try
        {
            //cdmw.clock.CdmwCosClock.init(orb,strArgs);
            cdmw.clock.CdmwCosClock.init(orb);
            org.omg.CORBA.Object obj  = orb.resolve_initial_references("ClockService");
            ClockCatalog catalog = ClockCatalogHelper.narrow(obj);
            if (catalog == null)
            {
                System.out.println("Error : Unable to resolve clock catalog");
            }

            // create a generic catalog for tests
            GenericCatalog<Testable> testCtg = new GenericCatalog<Testable>();

            // add all tests to the tests catalog
            testCtg.add("CreateMultipleVirtualClockTest",
                        new CreateMultipleVirtualClockTest(orb,catalog,strArgs));

            testCtg.add("ClockTest",
                        new ClockTest(orb));

            testCtg.add("ClockCatalogTest",
                        new ClockCatalogTest(orb, catalog));

            testCtg.add("ControlledClockTest",
                        new ControlledClockTest(orb, catalog));

            testCtg.add("AdditionAndSubtractionTest",
                        new TimevalOperationsTest("checkAdditionSubtractionTest"));

            testCtg.add("EqualityTest",
                        new TimevalOperationsTest("checkEqualityTest"));

            testCtg.add("ComparationTest",
                        new TimevalOperationsTest("checkCompareTest"));

            testCtg.add("TimeInfo",
                        new TimeInfo(orb,catalog,strArgs));

            testCtg.add("ImportExportTimevalTest",
                        new ImportExportTimevalTest());


            testCtg.add("ControllerTest",
                        new ControllerTest(orb, catalog));

            testCtg.add("ExecutorTest",
                        new ExecutorTest(orb));

            testCtg.add("UnpackTimeTest",
                        new PackUnpackTimeOperationsTest("checkUnpackTimeTest"));
            testCtg.add("PackTimeTest",
                        new PackUnpackTimeOperationsTest("checkPackTimeTest"));

            testCtg.add("UTCTest",
                        new UTCTest());

            testCtg.add("TimeSpanTest",
                        new TimeSpanTest());

            testCtg.add("ControllerUpdateTest",
                        new ControllerUpdateTest(orb));

            Testable testToExecute;
            if (testname.equals("ALL"))
            {
                Iterator<Testable> i = testCtg.getIterator();
                while (i.hasNext())
                    i.next().start();
            }
            else if ( (testToExecute = testCtg.search(testname) ) != null)
            {
                testToExecute.start();
            }
            else
            {
                System.out.println("Error: Test " + testname + " not found in tests catalog");
                throw new InvalidName();
            }
        }
        catch (InvalidName in)
        {
            in.printStackTrace();
        }
    }

///////////////////////////////////////////////////////////////////////////////

    public static void main(String[] args)
    {

        String option = "--CdmwPropertyPort";
        String portNumber = OS.getOptionValue(args,option) ;

        if ( portNumber.equals("no") )
        {
            portNumber = "5200";
        }

        option = "--TestName";
        String testName = OS.getOptionValue(args,option) ;

        if (testName.equals("no") ||
                testName.equals("")   ||
                testName.equalsIgnoreCase ("ALL")) // default behaviour: execute all tests
        {
            testName = "ALL";
        }

        int status = SUCCESS;
        ThreadedProcess propertyp = null;

        option = "--PropertyService";
        String propertyService = OS.getOptionValue(args,option) ;
        String[] strArgs= {
                              " --CdmwLocalisationService=4883" ,
                              "-ORBInitRef" ,
                              "LocalClock=corbaloc::localhost:4883/LocalClock",
                              "-ORBInitRef" ,
                              "ClockService=corbaloc::localhost:4883/ClockService" ,
                              "", // reserve 2 places for Property Service arguments
                              ""
                          };
        if ( propertyService.equals("no") == false ) {
            // standalone Property Service requested
            try
            {
                propertyp = ORBUtils.startPropertyService(portNumber);
                OS.sleep(cdmw.testutils.Testable.getTimescale() * 3000);
                String[] propArgs= {
                                       "-ORBInitRef" ,
                                       "PropertyService=corbaloc::localhost:" +
                                       portNumber + "/PropertyService"
                                   };
                // copy Property Service arguments
                System.arraycopy(propArgs, 0, strArgs, 5, 2);
            }
            catch (IOException ioe)
            {
                System.out.println("error during start of Property Service" + ioe.getMessage());
                status = FAILURE;
            }
            catch (InterruptedException ie)
            {
                System.out.println(ie.getMessage());
                status = FAILURE;
            }
        }
        ORB orb = ORB.init(strArgs, ORBUtils.getProperties());
        try
        {
            status = run(orb, testName, strArgs);
        }
        catch (InvalidName in)
        {
            in.printStackTrace();
            status = FAILURE;
        }
        catch (AdapterInactive ai)
        {
            ai.printStackTrace();
            status = FAILURE;
        }

        if (status == SUCCESS)
        {
            System.out.println("Main test successful.");
        }
        else
        {
            System.out.println("Main test failed.");
        }

        if (orb != null)
        {
            try
            {
                orb.destroy();
            }
            catch (Exception ex)
            {
                ex.printStackTrace();
                status = FAILURE;
            }
        }
        if (propertyp != null)
        {
            System.out.println("..............Killing Property Service JAVA process");
            OS.killProcess( propertyp );
        }

        cdmw.clock.CdmwCosClock.close();
    }
}


