/**
 * @brief _implementation for ControllerUpdateAdminTest with cppUnit library.
 *
 * @author Francesca Annunziata<fannunziata@selex-si.com>, 
 * @author Fabrizio Morciano <fmorciano@selex-si.com>
 */

#include "testpolicies/ControllerUpdateAdminTest.hpp"
#include "clockservice/ControllerCancelAllPolicy.hpp"
#include "clockservice/ControllerUpdateAdmin_impl.hpp"

#include <string>
#include <iostream>
#include <sstream>

#include "Time/clockservice/test/testcommon/TimeTestManager.hpp"


///////////////////////////////////////////////////////////////////////////////

CPPUNIT_TEST_SUITE_REGISTRATION(ControllerUpdateAdminTest);

///////////////////////////////////////////////////////////////////////////////

ControllerUpdateAdminTest::ControllerUpdateAdminTest()
{  
}

///////////////////////////////////////////////////////////////////////////////

void  
ControllerUpdateAdminTest::setUp()
{
    m_alias_name = "PolicyControllerUpdateAdmin";  
    Cdmw::TestUtils::CORBATestManager::instance()->add(m_alias_name);
    try
    {
        m_obj = Cdmw::TestUtils::CORBATestManager::instance()->get_object(m_alias_name);
    }
    catch(Cdmw::TestUtils::CORBATestManager::InvalidObject&)
    {
        CPPUNIT_FAIL("Cdmw::TestUtils::CORBATestManager::InvalidObject");
    }
    
    m_admin = Cdmw::clock::policy::ControllerUpdateAdmin::_narrow(m_obj);    
    CPPUNIT_ASSERT(!CORBA::is_nil(m_admin.in() ) );
}

///////////////////////////////////////////////////////////////////////////////

void  
ControllerUpdateAdminTest::tearDown()
{
}

///////////////////////////////////////////////////////////////////////////////

void
ControllerUpdateAdminTest::cxx_register()
{
	Cdmw::clock::ControllerCancelAllPolicy* 
       cancelAllPolicy = new Cdmw::clock::ControllerCancelAllPolicy();
	Cdmw::clock::policy::ControllerUpdate_ptr 
        policy = cancelAllPolicy->_this();            
	m_admin->_cxx_register(Cdmw::clock::policy::USER_DEFINED_0,
                           policy);
}

///////////////////////////////////////////////////////////////////////////////

void
ControllerUpdateAdminTest::already_present()
{
    Cdmw::clock::ControllerCancelAllPolicy* 
       cancelAllPolicy = new Cdmw::clock::ControllerCancelAllPolicy();
    Cdmw::clock::policy::ControllerUpdate_ptr 
        policy = cancelAllPolicy->_this();            
    m_admin->_cxx_register(Cdmw::clock::policy::USER_DEFINED_0,
                           policy);    
}

///////////////////////////////////////////////////////////////////////////////

void
ControllerUpdateAdminTest::unregister()
{
    m_admin->unregister(Cdmw::clock::policy::USER_DEFINED_0);    
}

///////////////////////////////////////////////////////////////////////////////
void
ControllerUpdateAdminTest::get_policy()
{
    Cdmw::clock::policy::ControllerUpdate_ptr 
        policy;
    policy = m_admin->get_policy(Cdmw::clock::policy::USER_DEFINED_0);
    CPPUNIT_ASSERT(!CORBA::is_nil(policy)); // just to avoid warning
}

// End Of File
///////////////////////////////////////////////////////////////////////////////
