/* =========================================================================== *
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati.
 * Copyright (C) SELEX Sistemi Integrati 2004-2006. All rights reserved.
 *  
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *  
 *  
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * =========================================================================== */

// Standard Files
#include <string>
#include <iostream>

// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <ConfAndPlug/cdmwinit/CdmwInterface.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <FaultTolerance/ftinit/FTServiceInit.hpp>

#include "ftclockclient/ClientExecutorPeriodicProcessBehaviour.hpp"
#include "ftclockclient/ClientClockProcessBehaviour.hpp"

using namespace std;

int main(int argc, char* argv[])
{
    CORBA::ORB_var orb;
    try
    {
        // Initialise FT service (to be done before ORB_init)
        Cdmw::CdmwInit::CDMW_preORBInit(argc, argv);

        // Set ORB strategies
        Cdmw::OrbSupport::StrategyList strategyList;
        strategyList.add_OrbThreaded();
        strategyList.add_PoaThreadPerConnection();

        // Initialises the ORB
        orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategyList);

        // Get the root POA
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());

        // Activate the POA Manager
        PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();
        poaMgr->activate();

        // Create the process behaviour
        typedef Cdmw::clock::valid::ClientExecutorPeriodicProcessBehaviour PRCBehaviour;
        std::auto_ptr<PRCBehaviour> process_behaviour_auto_ptr(new PRCBehaviour(orb.in(), rootPOA.in()));

        // Initialize Cardamom
        Cdmw::CdmwInit::CDMW_init(orb.in(), argc, argv, process_behaviour_auto_ptr.get());

        // Enter in the main loop
        orb->run();
    }
    catch (const CORBA::Exception &e)
    {
        std::cerr << __FILE__ << " " << __LINE__ << ": CORBA exception raised: " << e._name() << std::endl;
        return EXIT_FAILURE;
    }
    catch (const Cdmw::Exception &e)
    {
        std::cerr << __FILE__ << " " << __LINE__ << ": Cdmw exception raised: " << e.what() << std::endl;
        return EXIT_FAILURE;
    }
    catch (...)
    {
        std::cerr << __FILE__ << " " << __LINE__ << ": unexpected exception raised" << std::endl;
        return EXIT_FAILURE;
    }

    // Call generated Cardamom cleanup
    Cdmw::CdmwInit::CDMW_cleanup(orb.in());

    // Call ORB cleanup
    Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());

    // Destroy orb
    if (!CORBA::is_nil(orb.in()))
    {
        try
        {
            orb->destroy();
        }
        catch (const CORBA::Exception& ex)
        {
            std::cerr << __FILE__ << " " << __LINE__ << ": CORBA exception raised: " << ex._name() << std::endl;
            return EXIT_FAILURE;
        }
    }

    // Exit the main program
    return EXIT_SUCCESS;
}
