/* =========================================================================== *
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati.
 * Copyright (C) SELEX Sistemi Integrati 2004-2006. All rights reserved.
 *  
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *  
 *  
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * =========================================================================== */

#include <iostream>

// Cdmw Files
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>

#include "ftclockclient/ClientExecutorPeriodicProcessBehaviour.hpp"
#include "clockservice/StateTransferConfig.hpp"

Cdmw::clock::valid::ClientExecutorPeriodicProcessBehaviour::ClientExecutorPeriodicProcessBehaviour(CORBA::ORB_ptr          orb,
                                                                                                   PortableServer::POA_ptr rootPOA)
{
    m_orb           = CORBA::ORB::_duplicate(orb);
    m_client_thread = NULL;
}

Cdmw::clock::valid::ClientExecutorPeriodicProcessBehaviour::~ClientExecutorPeriodicProcessBehaviour()
    throw ()
{
}

CORBA::ULong
Cdmw::clock::valid::ClientExecutorPeriodicProcessBehaviour::nb_steps()
    throw (CORBA::SystemException)
{
    return 1;
}

void
Cdmw::clock::valid::ClientExecutorPeriodicProcessBehaviour::on_initialise(const CdmwPlatformMngtBase::StartupKind& startup_kind)
    throw (CORBA::SystemException)
{
    std::cout << "   -------- Client: on_initialise [BEGIN] -----------" << std::endl;

    // Get NamingInterface to object_groups (for reading)
    std::cout << "       ---- Client: get domain naming interface to object groups" << std::endl;
    Cdmw::CommonSvcs::Naming::NamingInterface obj_grps_naming_interface =
        Cdmw::NamingAndRepository::RepositoryInterface::get_domain_naming_interface(REPOSITORY_CLOCK_GROUP_NAME.c_str());

    // Get references to database object groups from naming interface
    CORBA::Object_var executor_obj;
    CORBA::Object_var periodic_obj;
    try
    {
        // Get executor object group
        std::cout << "       ---- Client: get executor group reference from database" << std::endl;
        executor_obj = obj_grps_naming_interface.resolve(EXECUTOR_GROUP_NAME.c_str());
        m_executor_group = CosClockService::PeriodicExecution::Executor::_narrow(executor_obj.in());

        // Get periodic object group
        std::cout << "       ---- Client: get periodic group reference from database" << std::endl;
        periodic_obj = obj_grps_naming_interface.resolve(PERIODIC_GROUP_NAME.c_str());
        m_periodic_group = CosClockService::PeriodicExecution::Periodic::_narrow(periodic_obj.in());
    }
    catch (const CosNaming::NamingContext::NotFound& e)
    {
        std::cerr << __FILE__ << "  " << __LINE__ << ": NamingContext::NotFound excepton raised" << std::endl;
    }
    catch (const CosNaming::NamingContext::CannotProceed& e)
    {
        std::cerr << __FILE__ << "  " << __LINE__ << ": NamingContext::CannotProceed exception raised" << std::endl;
    }
    catch (const CosNaming::NamingContext::InvalidName& e)
    {
        std::cerr << __FILE__ << "  " << __LINE__ << ": NamingContext::InvalidName exception raised" << std::endl;
    }
    catch (const CORBA::SystemException& e)
    {
        std::cerr << __FILE__ << "  " << __LINE__ << ": CORBA::SystemException exception raised" << std::endl;
    }

    std::cout << "   -------- Client: on_initialise [ END ] -----------" << std::endl;
}

void
Cdmw::clock::valid::ClientExecutorPeriodicProcessBehaviour::on_next_step()
    throw (CdmwPlatformMngt::ProcessDelegate::InvalidStep, CORBA::SystemException)
{
}

void
Cdmw::clock::valid::ClientExecutorPeriodicProcessBehaviour::on_run()
    throw (CdmwPlatformMngt::ProcessDelegate::NotReadyToRun, CORBA::SystemException)
{
    std::cout << "   -------- Client: on_run [BEGIN] --------------" << std::endl;

    // Launch the test in a thread
    m_client_thread = new ClientExecutorPeriodicThread(m_executor_group, m_periodic_group);
    m_client_thread->start();

    std::cout << "   -------- Client: on_run [ END ] --------------" << std::endl;
}

void
Cdmw::clock::valid::ClientExecutorPeriodicProcessBehaviour::on_stop()
    throw (CORBA::SystemException)
{
    std::cout << "   -------- Client: on_stop [BEGIN] --------------" << std::endl;

    m_client_thread->stop();
    m_client_thread->join();
    delete m_client_thread;

    m_orb->shutdown(false);

    std::cout << "   -------- Client: on_stop [ END ] --------------" << std::endl;
}
