/* =========================================================================== *
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati.
 * Copyright (C) SELEX Sistemi Integrati 2004-2006. All rights reserved.
 *  
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *  
 *  
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * =========================================================================== */

#include <ftclockclient/ClientExecutorPeriodicThread.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/common/Assert.hpp>

static void readKey()
{
    char ch[16];
    fread(ch, 2, 1, stdin);
}

void
ClientExecutorPeriodicThread::run() throw ()
{
    try
    {
        // Start-up synchronization code
        Cdmw::OsSupport::OS::sleep(1000);
        std::cout << "[ClientExecutorPeriodicThread]:>> hit a key to start client..." << std::endl;
        readKey();

        // Run client
        std::cout << "[ClientExecutorPeriodicThread]:>> client is running..." << std::endl;
        std::cout << "[ClientExecutorPeriodicThread]:>> the client is ready to start periodic execution" << std::endl;
        try
        {
            // First burst of requests for current time on the FT executor
            std::cout << "[ClientExecutorPeriodicThread]:>> invoking requests on the clock" << std::endl;
            for (int i = 0; i < 1000; ++i)
                std::cout << "[" << i << "]: " << m_FT_executor->current_time() / 10 << " usec" << std::endl;
            std::cout << "[ClientExecutorPeriodicThread]:>> done invoking requests on the clock" << std::endl;
            std::cout << "[ClientExecutorPeriodicThread]:>> hit a key to continue..." << std::endl;
            readKey();

            // Get an FT controller from an FT executor passing an FT periodic
            CosClockService::PeriodicExecution::Controller_var controller = m_FT_executor->enable_periodic_execution(m_FT_periodic.in());

            // Start the FT controller
            TimeBase::TimeT period            = 10000000; // 100000 * 100 nsec =  1000 msec
            TimeBase::TimeT offset            = 10000000;
            const int       executions_number = 0;
            CORBA::Any      any;
            std::cout << "[ClientExecutorPeriodicThread]:>> programming timer with:" << std::endl
                      << "\tPeriod: " << period            << std::endl
                      << "\toffset: " << offset            << std::endl
                      << "\texecs : " << executions_number << std::endl;
            controller->start(period, offset, executions_number, any);
            std::cout << "[ClientExecutorPeriodicThread]:>> controller scheduled..." << std::endl;

            // Monitor executions of periodic's do_work()
            for ( ; ; )
            {
                Cdmw::OsSupport::OS::sleep(5000);
                std::cout << "[ClientExecutorPeriodicThread]:>> client is running..." << std::endl;
            }
        }
        catch (CORBA::Exception& e)
        {
            std::cerr << __FILE__ << " " << __LINE__ << ": unexpected CORBA exception: " << Cdmw::OrbSupport::OrbSupport::exception_to_string(e) << std::endl;
        }
    }
    catch (std::exception& e)
    {
        std::cerr << __FILE__ << " " << __LINE__ << ": unexpected std exception: " << e.what() << std::endl;
    }
    catch (CORBA::Exception& e)
    {
        std::cerr << __FILE__ << " " << __LINE__ << ": unexpected CORBA exception: " << Cdmw::OrbSupport::OrbSupport::exception_to_string(e) << std::endl;
    }
    catch (...)
    {
        std::cerr << __FILE__ << " " << __LINE__ << ": unexpected generic exception" << std::endl;
    }
}
