/* =========================================================================== *
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati.
 * Copyright (C) SELEX Sistemi Integrati 2004-2006. All rights reserved.
 *  
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *  
 *  
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * =========================================================================== */

#ifndef CARDAMOM_GROUPS_CREATOR_PROCESS_BEHAVIOUR_HPP
#define CARDAMOM_GROUPS_CREATOR_PROCESS_BEHAVIOUR_HPP

// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>
#include <ConfAndPlug/cdmwinit/ProcessControl.hpp>
#include <Repository/idllib/CdmwNamingAndRepository.stub.hpp>
#include <FaultTolerance/idllib/FT.stub.hpp>

#include <string>
#include <map>

namespace Cdmw {
    namespace clock {
        namespace svcs {
            class GroupsCreatorProcessBehaviour;
        }
    }
}

class Cdmw::clock::svcs::GroupsCreatorProcessBehaviour
    : public Cdmw::CdmwInit::ProcessControl
{

    public:

        // Constructor
        GroupsCreatorProcessBehaviour(CORBA::ORB_ptr     orb,
                                      bool               point_to_point,
                                      const std::string& host1,
                                      const std::string& host2,
                                      const std::string& host3);

        // Destructor
        virtual ~GroupsCreatorProcessBehaviour() throw ();

        virtual CORBA::ULong nb_steps()
            throw (CORBA::SystemException);

        virtual void on_initialise(const CdmwPlatformMngtBase::StartupKind& startup_kind)
            throw (CORBA::SystemException);

        virtual void on_run()
            throw (CdmwPlatformMngt::ProcessDelegate::NotReadyToRun, CORBA::SystemException);

        virtual void on_stop()
            throw (CORBA::SystemException);

        void link_executor_process_and_host(const std::string& process, const std::string& host);
        void link_periodic_process_and_host(const std::string& process, const std::string& host);

    private:

        // Some type definitions
        typedef std::multimap<std::string, std::string>                 linkmmap;
	typedef	std::multimap<std::string, std::string>::iterator	iterator;
	typedef	std::multimap<std::string, std::string>::const_iterator	const_iterator;

        // Association between processes and hosts
        linkmmap executor_group_links;
        linkmmap periodic_group_links;

        std::string                             m_host1;
        std::string                             m_host2;
        std::string                             m_host3;

        CORBA::ORB_var                          m_orb;
        ::FT::ReplicationManager_var            m_replication_mgr;
        CdmwNamingAndRepository::NameDomain_var m_obj_grps_domain;

        ::FT::ObjectGroup_var                   m_executor_obj_group;
        ::FT::ObjectGroup_var                   m_periodic_obj_group;

        bool                                    m_point_to_point;
};

#endif // CARDAMOM_GROUPS_CREATOR_PROCESS_BEHAVIOUR_HPP
