/* ========================================================================== *
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati.
 * Copyright (C) SELEX Sistemi Integrati 2004-2006. All rights reserved.
 *  
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *  
 *  
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ========================================================================= */

#ifndef CARDAMOM_EVOT_FT_PERIODIC_SERVICE_PROCESS_BEHAVIOUR_H
#define CARDAMOM_EVOT_FT_PERIODIC_SERVICE_PROCESS_BEHAVIOUR_H

#include "Foundation/orbsupport/CORBA.hpp"
#include "ConfAndPlug/cdmwinit/ProcessControl.hpp"
#include "FaultTolerance/idllib/CdmwFTGroupRepository.stub.hpp"
#include "FaultTolerance/idllib/FT.stub.hpp"
#include "FaultTolerance/ftinit/FTServiceInit.hpp"
#include "FaultTolerance/idllib/CdmwFTStateTransfer.stub.hpp"

#include "clockservice/UTCTimeService_impl.hpp"
#include "clockservice/ControlledClock_impl.hpp"
#include "clockservice/ClockCatalog_impl.hpp"
#include "clockservice/Clock_impl.hpp"
#include "clockservice/UTC_impl.hpp"
#include "clockservice/FTExecutor_impl.hpp"
#include "clockservice/FTController_impl.hpp"

#include "ftperiodic/FTPeriodicActivationHandler.hpp"
#include "ftperiodic/FTPeriodicServantLocator.hpp"
#include "ftperiodic/FTPeriodic_impl.hpp"

#include "ftclocksvcs/ClockServiceWorker.hpp"

#include "CosClockServiceS.h"

/*
#include "ace/Get_Opt.h"
#include "ace/Thread.h"
#include "ace/Thread_Manager.h"
#include "ace/TP_Reactor.h"
#include "ace/Get_Opt.h"
#include "ace/Task.h"
#include "ace/Timer_Wheel.h"
*/

namespace Cdmw {
    namespace clock {
        namespace svcs {
            class FTPeriodicServiceProcessBehaviour;
        }
    }
}

class Cdmw::clock::svcs::FTPeriodicServiceProcessBehaviour
    : public Cdmw::CdmwInit::ProcessControl
{
    public:

        FTPeriodicServiceProcessBehaviour(CORBA::ORB_ptr orb, bool primary) throw ();
        ~FTPeriodicServiceProcessBehaviour() throw ();

    public:

        virtual CORBA::ULong nb_steps()
            throw (CORBA::SystemException);

        virtual void on_initialise(const CdmwPlatformMngtBase::StartupKind& startup_kind)
            throw (CORBA::SystemException);

        virtual void on_next_step()
            throw (CdmwPlatformMngt::ProcessDelegate::InvalidStep, CORBA::SystemException);

        virtual void on_run()
            throw (CdmwPlatformMngt::ProcessDelegate::NotReadyToRun, CORBA::SystemException);

        virtual void on_stop()
            throw (CORBA::SystemException);

    private:

        void add_periodic_to_periodic_group(const CosClockService::PeriodicExecution::Periodic_var& periodic_group);
        void get_executor_and_periodic_groups(CosClockService::PeriodicExecution::Executor_var& executor_group,
                                              CosClockService::PeriodicExecution::Periodic_var& periodic_group);
 
        CORBA::ORB_var                                   m_orb;
        CdmwFT::ReplicationManager_var                   m_replication_mgr;
        CdmwFT::Location::GroupRepository_var            m_group_repository;

        CosClockService::PeriodicExecution::Periodic_var m_periodic_obj;
        Cdmw::clock::FTPeriodic_impl*                    m_periodic_impl;
        Cdmw::clock::FTPeriodicActivationHandler*        m_periodic_activation_handler;
        Cdmw::clock::FTPeriodicServantLocator*           m_periodic_servant_locator;

        bool m_set_has_primary;
};

#endif /* CARDAMOM_EVOT_FT_PERIODIC_SERVICE_PROCESS_BEHAVIOUR_H */
