/* ========================================================================== *
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati.
 * Copyright (C) SELEX Sistemi Integrati 2004-2006. All rights reserved.
 *  
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *  
 *  
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ========================================================================= */

#ifndef CARDAMOM_FT_PERIODIC_IMPL_HPP
#define CARDAMOM_FT_PERIODIC_IMPL_HPP

// -- Cdmw Time Includes --
#include "Time/clockservice/CosClockService.stub.hpp"
#include "Time/clockservice/CosClockService.skel.hpp"

extern "C++" {
    namespace Cdmw {
        namespace clock {
            class FTPeriodicActivationHandler;
        }
    }
}

namespace Cdmw {
    namespace clock {
        class FTPeriodic_impl;
    }
}

/**
 * @brief This class provides a fault-tolerant implementation
 * of the <code>Periodic</code> interface.
 *
 * @author Angelo Corsaro <acorsaro@selex-si.com>
 */
class Cdmw::clock::FTPeriodic_impl
    : public POA_CosClockService::PeriodicExecution::Periodic
{
    public:

        /**
         * @brief Create an <code>FTPeriodic_impl</code>.
         * @param executor_servant a pointer to the executor servant C++ object instance.
         */
        FTPeriodic_impl(CosClockService::PeriodicExecution::Executor_ptr FT_executor);

        virtual ~FTPeriodic_impl();

    public:

        // -- Methods from the Periodic Interface --

        /**
         * @brief Register an instance of the Periodic interface for periodic execution.
         * @return TRUE if and only if the Cdmw::clock::FTPeriodic::do_work() succeded.
         */
	CORBA::Boolean do_work (const CORBA::Any&)
    	    throw (CORBA::SystemException);

    protected:

        friend class ::Cdmw::clock::FTPeriodicActivationHandler;

        /**
         * @brief This method is called by the framework to activate the clock.
         */ 
        virtual void activate();

    private:

        FTPeriodic_impl(const FTPeriodic_impl&);
        FTPeriodic_impl& operator=(const FTPeriodic_impl&);

    private:

        CosClockService::PeriodicExecution::Executor_var m_FT_executor;
};

#endif /* CARDAMOM_FT_PERIODIC_IMPL_HPP */
