/* ========================================================================== *
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati.
 * Copyright (C) SELEX Sistemi Integrati 2004-2006. All rights reserved.
 *  
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *  
 *  
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ========================================================================= */

#include "ftperiodic/FTPeriodic_impl.hpp"
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <iostream>

using namespace CosClockService::PeriodicExecution;

Cdmw::clock::FTPeriodic_impl::FTPeriodic_impl(CosClockService::PeriodicExecution::Executor_ptr FT_executor)
    : m_FT_executor(CosClockService::PeriodicExecution::Executor::_duplicate(FT_executor))
{
}

Cdmw::clock::FTPeriodic_impl::~FTPeriodic_impl()
{
}

CORBA::Boolean
Cdmw::clock::FTPeriodic_impl::do_work(const CORBA::Any&)
    throw (CORBA::SystemException)
{
    try
    {
        std::cout << "FTPeriodic_impl::do_work: current executor time: " << (m_FT_executor->current_time() / 10) << " usec" << std::endl;
    }
    catch (const CORBA::Exception& e)
    {
        std::cerr << "Unexpected CORBA exception: " << Cdmw::OrbSupport::OrbSupport::exception_to_string(e) << std::endl;
        return false;
    }
    catch (const std::exception& e)
    {
        std::cerr << "Unexpected std exception: " << e.what() << std::endl;
        return false;
    }
    catch (...)
    {
        std::cerr << "Unexpected generic exception" << std::endl;
        return false;
    }
    return true;
}

void
Cdmw::clock::FTPeriodic_impl::activate()
{
    std::cout << "############################################################" << std::endl;
    std::cout << ">>>>>>>>>>>>>>>>>>  Activating FTPeriodic Replica"            << std::endl;
    std::cout << ">>>>>>>>>>>>>>>>>>  Completed FTPeriodic Replica Activation"  << std::endl;
    std::cout << "############################################################" << std::endl;
}

