/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
 *
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */
package cdmw.valid.traceclient;

import java.io.*;
import java.util.*;

import cdmw.logging.Logger;
import cdmw.ossupport.OS;
import cdmw.performance.PerformanceMngr;

import cdmw.orbsupport.ORBSupport;
import cdmw.orbsupport.StrategyList;
import cdmw.cdmwinit.InitUtils;

import com.thalesgroup.CdmwNamingAndRepository.*;

import cdmw.generated.traceclientCdmwInterface;

public class MainTest
{
    private static final int POA_THREAD_POOL_SIZE = 5;
    
    private static final String TRACE_NBR_OPTION = "--trace-nbr";
    private static final String TRACE_REPEAT_OPTION = "--trace-repeat";
    private static final String TRACE_MULTI_DOMAIN_OPTION = "--trace-multi_domain";
    private static final String TRACE_SEQ1_OPTION = "--trace-seq1";
    private static final String TRACE_SEQ2_OPTION = "--trace-seq2";
    
    private static final String PERF_CALL_OPTION = "--perf-call";
    
    private static final int FAILURE = 1;
    
    private static String processName;
    
    
   public static void main(String[] args) {
   
      org.omg.CORBA.ORB orb = null;

      String applicationName = "";
      int retCode = 0;

      //get command line args
      if (args.length > 1) {
         if (args[1].equals("--help")) {
            MainTest.usage(args[0]);
            System.exit(retCode);
         }
      }
      
      // get Trace Message
      String traceMessage = "";
 
      // get number of Trace Messages
      long traceCount = -1;
            
      String traceCountOption = OS.getOptionValue(args, MainTest.TRACE_NBR_OPTION);
      if (traceCountOption.equals("yes")) {
         System.out.println("bad value for " + TRACE_NBR_OPTION + "=<number> option");
         System.exit(FAILURE);
      } else if (traceCountOption != "no") {
         try {
            traceCount = Long.parseLong(traceCountOption);
         } catch (NumberFormatException e) {
            System.out.println("bad value for " + TRACE_NBR_OPTION + "=<number> option");
            System.exit(FAILURE);
         }
      }

      // get trace repeat mode
      boolean traceRepeatMode = false;

      String traceRepeatModeOption = OS.getOptionValue(args, MainTest.TRACE_REPEAT_OPTION);
      if (traceRepeatModeOption.equals("yes")) {
         traceRepeatMode = true;
      }
      
      // get Performance probe call option
      boolean perfProbeCall = false;
      String perfProbeCallOption = OS.getOptionValue(args, PERF_CALL_OPTION);
      if (perfProbeCallOption.equals("yes")) {
         perfProbeCall = true;
      }
        
      // get trace multi domain mode
      boolean traceMultiDomainMode = false;
            
      String traceMultiDomainModeOption = OS.getOptionValue(args, MainTest.TRACE_MULTI_DOMAIN_OPTION);
      if (traceMultiDomainModeOption.equals("yes")) {
         traceMultiDomainMode = true;
      }

            // get trace sequence counter 1
            long traceCountSeq1 = -1;
            
            String traceCountSeq1Option = OS.getOptionValue(args, MainTest.TRACE_SEQ1_OPTION);
            if (traceCountSeq1Option.equals("yes")) {
               System.out.println("bad value for " + TRACE_SEQ1_OPTION + "=<number> option");
               System.exit(FAILURE);
            } else if (!traceCountSeq1Option.equals("no")) {
               try {
                  traceCountSeq1 = Long.parseLong(traceCountSeq1Option);
               } catch (NumberFormatException e) {
                  System.out.println("bad value for " + TRACE_SEQ1_OPTION + "=<number> option");
            	   System.exit(FAILURE);
               }
            }

            // get trace sequence counter 2
            long traceCountSeq2 = -1;
            
            String traceCountSeq2Option = OS.getOptionValue(args, TRACE_SEQ2_OPTION);
            if (traceCountSeq2Option.equals("yes")) {
               System.out.println( "bad value for " + TRACE_SEQ2_OPTION + "=<number> option");
               System.exit(FAILURE);
            } else if (!traceCountSeq2Option.equals("no")) {
               try {
                  traceCountSeq2 = Long.parseLong(traceCountSeq2Option);
               } catch (NumberFormatException e) {
                  System.out.println("bad value for " + TRACE_SEQ2_OPTION + "=<number> option");
                  System.exit(FAILURE);
               }
            }


      try {
         // Initialize the ORB
         StrategyList orbStrategies = new StrategyList();
         orbStrategies.addORBThreaded();
         orbStrategies.addPOAThreadPool(POA_THREAD_POOL_SIZE);

         orb = ORBSupport.ORBInit(args, orbStrategies);
         
         // The logger
         Logger logger;
         logger = Logger.Get_Logger("TEST", "TEST");

         // creates the process behaviour
         MyProcessBehaviour procBehaviour = new MyProcessBehaviour (orb,
                                                                    traceMessage, 
                                                                    traceCount,
                                                                    traceCountSeq1,
                                                                    traceCountSeq2,
                                                                    traceRepeatMode,
                                                                    traceMultiDomainMode,
                                                                    perfProbeCall, 
                                                                    logger);


         traceclientCdmwInterface.cdmwInit( orb, args, procBehaviour);

         // sample usage of InitUtils
         try {
            applicationName = cdmw.cdmwinit.InitUtils.getCdmwApplicationName();
            processName = cdmw.cdmwinit.InitUtils.getCdmwProcessName();
         } catch (cdmw.orbsupport.CORBASystemExceptionWrapper Ex) {
            System.err.println(Ex.getMessage());
         }

         System.out.println("Application name : " + applicationName);
         System.out.println("Process name : " + processName);
 
         //always desactivate at startup
         PerformanceMngr.deactivate();

         String baseMsg = "process <" + applicationName + "/" + processName + ">" ;
                
         // ===================================================
         // trace a message (don't use CDMW_TRACE in main)
         // ===================================================
                
         System.out.println("program started and waiting CORBA messages");
         String msg = baseMsg + "started and waiting CORBA messages"; 
                
         logger.info(msg, null);
            


         // Get the root POA 
         org.omg.CORBA.Object obj = orb.resolve_initial_references("RootPOA");
         org.omg.PortableServer.POA rootPOA = org.omg.PortableServer.POAHelper.narrow(obj);

         // Run implementation (Activate the POA manager)
         rootPOA.the_POAManager().activate();
            
         // start orb
         orb.run();            
 
      } catch (Exception ex) {
         System.err.println(ex.getMessage());
         retCode = FAILURE;
      }

      if (orb != null) {
         try {
            traceclientCdmwInterface.cdmwCleanup (orb);
            orb.destroy();
         } catch(Exception ex) {
            System.err.println(ex.getMessage());
            retCode = FAILURE;
         }
      }
 
      System.exit(retCode);
    }

    private static void usage(String programName)
    {
        System.out.println( "Usage:\r\n" +
                             " options\r\n" +
                              "\n" +
                              "Options:\r\n" +
                        "--help                             Display this information.\r\n" +
                        "[--trace-nbr=<number>]             Number of messages to trace.\r\n" +
                        "[--trace-repeat]                   generate repeated message.\r\n" +
                        "[--trace-multi_domain]             generate multi domain messages.\r\n" +
                        "[--trace-seq1=<number>]            sequence nbr to deactivate 1st set of messages.\r\n" +
                        "[--trace-seq2=<number>]            sequence nbr to activate 2nd set of messages.\r\n" +
                        "[--perf-call]                      Performance probe calling.\r\n" );
    }

    
    private void exceptionHandler()
    {
        System.out.println("Unexpected Exception : Test failed");
    }

    private void exitHandler()
    {
        System.out.println("Process '" + MainTest.processName + "' exited");
    }
    
}
