/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX-SI. It is derivative work based on PERCO Copyright (C) THALES
 * 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX-SI and THALES 2006. All rights reserved.
 *
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */
package cdmw.valid.traceclient;

import cdmw.common.BadOrderException;
import cdmw.common.BadParameterException;
import cdmw.logging.Logger;
import cdmw.cdmwinit.ProcessControl;
import cdmw.cdmwinit.InitUtils;
import cdmw.platformmngt.PlatformInterface;

import org.omg.CORBA.ORB;
import java.util.*;

public class MyProcessBehaviour extends ProcessControl {
   /**
    *The ORB.
    */            
   private org.omg.CORBA.ORB orb;
   
   /**
    * The trace message.
    */
   private String traceMessage;
   
   /**
    * The number of trace message.
    */
   private long traceCount;
   
   /**
    * The number of trace message for sequence 1.
    */        
   private long traceCountSeq1;
   
   /**
    * The number of trace message for sequence 2.
    */        
   private long traceCountSeq2;
   
   /**
    * The repetition flag.
    */        
   private boolean repetition;
   
   /**
    * The multidomain flag.
    */        
   private boolean multidomain;
   
   /**
    * The performance probe call flag.
    */        
   private boolean perfProbeCall;
   
   /**
    * The logger.
    */        
   private Logger logger;
   
   /**
    * init done flag
    */
   private boolean initDone;
   
   /**
    * The application name.
    */        
   private String applicationName;
   
   /**
    * The process name.
    */        
   private String processName;
   
   /**
    * The client thread reference
    */        
   private TraceClientThread traceClientThread;
   
   public MyProcessBehaviour(ORB orb,
                             String traceMessage,
                             long traceCount,
                             long traceCountSeq1,
                             long traceCountSeq2,
                             boolean repetition,
                             boolean multidomain,
                             boolean perfProbeCall,
                             Logger logger) {
      this.orb = orb;
      this.traceMessage = traceMessage;
      this.traceCount = traceCount;
      this.traceCountSeq1 = traceCountSeq1;
      this.traceCountSeq2 = traceCountSeq2;
      this.repetition = repetition;
      this.multidomain = multidomain;
      this.perfProbeCall = perfProbeCall;
      
      this.traceClientThread = null;
      this.initDone = false;
      this.logger = logger;
   }

   public void onRun()
      throws com.thalesgroup.CdmwPlatformMngt.ProcessDelegatePackage.NotReadyToRun {
      try {
         PlatformInterface.notifyMessage (com.thalesgroup.CdmwPlatformMngtBase.INF.value, 
                                          this.processName,
                                          "Run requested");
      } catch (BadOrderException e) {
         this.logger.error("BadOrderException", null);
         e.printStackTrace();
      } catch (BadParameterException e) {
         this.logger.error("BadParameterException", null);
         e.printStackTrace();
      }

      // start the this.traceClientThread
      if (this.traceClientThread != null) 
      {
         this.traceClientThread.start();
      }
   }


   public void onStop() {
      TraceClientThread clientThread = this.traceClientThread;
      this.traceClientThread = null;

      try {
         PlatformInterface.notifyMessage( com.thalesgroup.CdmwPlatformMngtBase.INF.value,
                                          this.processName,
                                          "Stop requested");
      } catch (BadOrderException e) {
         this.logger.error("BadOrderException", null);
         e.printStackTrace();
      } catch (BadParameterException e) {
         this.logger.error("BadParameterException", null);
         e.printStackTrace();
      }

      if (clientThread != null && Thread.State.RUNNABLE == clientThread.getState()) {
         try {
            clientThread.join();
         } catch (InterruptedException ie) {
            this.logger.error("InterruptedException", null);
            ie.printStackTrace();
         }
      }

      this.orb.shutdown(false);
   }
        
   public void onInitialise(com.thalesgroup.CdmwPlatformMngtBase.StartupKind startupKind) {
      try {
         // get the name of the application
         this.applicationName = cdmw.cdmwinit.InitUtils.getCdmwApplicationName();
         // get the name of the process
         this.processName = cdmw.cdmwinit.InitUtils.getCdmwProcessName();
      } catch (cdmw.orbsupport.CORBASystemExceptionWrapper Ex) {
         System.err.println(Ex.getMessage());
      }

      try {
         PlatformInterface.notifyMessage( com.thalesgroup.CdmwPlatformMngtBase.INF.value, 
                                          this.processName,
                                          "Initialisation requested");
      } catch (BadOrderException e) {
         this.logger.error("BadOrderException", null);
         e.printStackTrace();
      } catch (BadParameterException e) {
         this.logger.error("BadParameterException", null);
         e.printStackTrace();
      }
                                          
      // create the this.traceClientThread
      this.traceClientThread = new TraceClientThread( this.orb,
                                                      this.applicationName,
                                                      this.processName,
                                                      this.traceMessage,
                                                      this.traceCount,
                                                      this.traceCountSeq1,
                                                      this.traceCountSeq2,
                                                      this.repetition,
                                                      this.multidomain,
                                                      this.perfProbeCall,
                                                      this.logger);
   }

   public org.omg.CORBA.Object getService() {
      try {
         PlatformInterface.notifyMessage (com.thalesgroup.CdmwPlatformMngtBase.INF.value, 
                                          this.processName, 
                                          "Service requested -> NIL returned");
      } catch (BadOrderException e) {
         this.logger.error("BadOrderException", null);
         e.printStackTrace();
      } catch (BadParameterException e) {        
         this.logger.error("BadParameterException", null);
         e.printStackTrace();
      }
            
      return null;
  }
        
        
   public void onNextStep()
      throws com.thalesgroup.CdmwPlatformMngt.ProcessDelegatePackage.InvalidStep {
   }

   public synchronized void setInitDone() {
      this.initDone = true;
   }

   public int nbSteps() {
      return 1;
   }        

}
